@@ -37,7 +37,7 @@ extern void R_ProcessEvents(void);
 #include <ctype.h>
 #include <wchar.h>
 #include <limits.h>
-#include <R_ext/rlocale.h>
+#include <rlocale.h>
 #include <R_ext/Memory.h>
 #include "graphapp/ga.h"
 #ifdef USE_MDI
@@ -264,7 +264,7 @@ static size_t enctowcs(wchar_t *wc, char *s, int n)
 	nc += mbstowcs(wc, s, n);
 	pb += 3; pe = pb;
 	while(*pe &&
-	      !((pe = strchr(pb, UTF8out[0])) && *(pe+1) == UTF8out[1] &&
+	      !((pe = strchr(pe, UTF8out[0])) && *(pe+1) == UTF8out[1] &&
 	      *(pe+2) == UTF8out[2])) pe++;
 	if(!*pe) return nc; /* FIXME */;
 	*pe = '\0';
@@ -279,10 +279,18 @@ static size_t enctowcs(wchar_t *wc, char *s, int n)
 static void xbufadds(xbuf p, const char *s, int user)
 {
     int n = strlen(s) + 1; /* UCS-2 must be shorter */
-    /* wchar_t tmp[n];  this might blow the stack */
-    wchar_t *tmp = (wchar_t*) R_alloc(n, sizeof(wchar_t));
-    enctowcs(tmp, (char *) s, n);
-    xbufaddxs(p, tmp, user);
+    if (n < 1000) {
+	wchar_t tmp[n];
+	enctowcs(tmp, (char *) s, n);
+	xbufaddxs(p, tmp, user);
+    } else {
+	/* very long line */
+	void *vmax = vmaxget();
+	wchar_t *tmp = (wchar_t*) R_alloc(n, sizeof(wchar_t));
+	enctowcs(tmp, (char *) s, n);
+	xbufaddxs(p, tmp, user);
+	vmaxset(vmax);
+    }
 }
 
 static void xbuffixl(xbuf p)
@@ -1937,7 +1945,7 @@ int consoler = 25, consolec = 80, consolex = 0, consoley = 0;
 int pagerrow = 25, pagercol = 80;
 int pagerMultiple = 1, haveusedapager = 0;
 int consolebufb = DIMLBUF, consolebufl = MLBUF, consolebuffered = 1;
-int consoleblink = 1;
+static int consoleblink = 1;
 
 void
 setconsoleoptions(const char *fnname,int fnsty, int fnpoints,
