@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Langage for Statistical Data Analysis
  *  Copyright (C) 1998--2005  Guido Masarotto and Brian Ripley
- *  Copyright (C) 2004--2009  The R Foundation
+ *  Copyright (C) 2004--2011  The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -102,9 +102,10 @@ void Rconsolecmd(char *cmd)
     consolecmd(RConsole, cmd);
 }
 
-void closeconsole(control m)  /* can also be called from editor menus */
+static void closeconsole(control m)
 {
-    R_CleanUp(SA_DEFAULT, 0, 1);
+    consolecmd(RConsole, "q()");
+//    R_CleanUp(SA_DEFAULT, 0, 1);
 }
 
 static void quote_fn(wchar_t *fn, char *s)
@@ -419,6 +420,7 @@ static Rboolean isdebuggerpresent(void)
 {
     typedef BOOL (*R_CheckDebugger)(void);
     R_CheckDebugger entry;
+    /* XP or later */
     entry =
 	(R_CheckDebugger) GetProcAddress((HMODULE)GetModuleHandle("KERNEL32"),
 					 "IsDebuggerPresent");
@@ -469,27 +471,10 @@ static void menupkgload(control m)
 static void menupkgupdate(control m)
 {
     if (!ConsoleAcceptCmd) return;
-    consolecmd(RConsole, "update.packages(ask='graphics')");
+    consolecmd(RConsole, "update.packages(ask='graphics',checkBuilt=TRUE)");
 /*    show(RConsole); */
 }
 
-#if 0
-static void menupkgupdatebioc(control m)
-{
-    if (!ConsoleAcceptCmd) return;
-    consolecmd(RConsole,
-	       "update.packages(repos=getOption(\"BIOC\"))");
-/*    show(RConsole); */
-}
-
-
-static void menupkginstallbioc(control m)
-{
-    if (!ConsoleAcceptCmd) return;
-    consolecmd(RConsole, "utils:::menuInstallBioc()");
-}
-#endif
-
 static void menupkgcranmirror(control m)
 {
     if (!ConsoleAcceptCmd) return;
@@ -575,7 +560,7 @@ static void menumainman(control m)
 
 static void menumainref(control m)
 {
-    internal_shellexec("doc\\manual\\refman.pdf");
+    internal_shellexec("doc\\manual\\fullrefman.pdf");
 }
 
 static void menumaindata(control m)
@@ -934,18 +919,13 @@ int RguiPackageMenu(PkgMenuItems pmenu)
     MCHECK(newmenuitem("-", 0, NULL));
     MCHECK(pmenu->mpkgil = newmenuitem(G_("Install package(s) from local zip files..."),
 				0, menupkginstalllocal));
-/*    MCHECK(newmenuitem("-", 0, NULL));
-    MCHECK(mpkgb = newmenuitem(G_("Install package(s) from Bioconductor..."),
-			       0, menupkginstallbioc));
-    MCHECK(mpkgbu = newmenuitem(G_("Update packages from Bioconductor"),
-    0, menupkgupdatebioc)); */
     return 0;
 }
 
 static void CheckForManuals(void)
 {
     lmanintro = check_doc_file("doc\\manual\\R-intro.pdf");
-    lmanref = check_doc_file("doc\\manual\\refman.pdf");
+    lmanref = check_doc_file("doc\\manual\\fullrefman.pdf");
     lmandata = check_doc_file("doc\\manual\\R-data.pdf");
     lmanlang = check_doc_file("doc\\manual\\R-lang.pdf");
     lmanext = check_doc_file("doc\\manual\\R-exts.pdf");
@@ -976,7 +956,7 @@ int RguiCommonHelp(menu m, HelpMenuItems hmenu)
 	MCHECK(hmenu->mmanintro = newmenuitem("An &Introduction to R", 0,
 				       menumainman));
 	if (!lmanintro) disable(hmenu->mmanintro);
-	MCHECK(hmenu->mmanref = newmenuitem("R &Reference Manual", 0,
+	MCHECK(hmenu->mmanref = newmenuitem("R &Reference", 0,
 				     menumainref));
 	if (!lmanref) disable(hmenu->mmanref);
 	MCHECK(hmenu->mmandata = newmenuitem("R Data Import/Export", 0,
@@ -1064,7 +1044,7 @@ int setupui(void)
 #ifdef WIN64
 	    "RGui (64-bit)",
 #else
-	    "RGui",
+	    "RGui (32-bit)",
 #endif
 	    MDIsize,
 	    StandardWindow | Menubar | Workspace);
@@ -1079,7 +1059,7 @@ int setupui(void)
 #ifdef WIN64
 	if (!(RConsole = newconsole("R Console (64-bit)", flags ))) return 0;
 #else
-	if (!(RConsole = newconsole("R Console", flags ))) return 0;
+	if (!(RConsole = newconsole("R Console (32-bit)", flags ))) return 0;
 #endif
 	TRACERUI("Console done");
     }
