@@ -2,7 +2,7 @@
  *  R : A Computer Language for Statistical Data Analysis
  *  file run.c: a simple 'reading' pipe (and a command executor)
  *  Copyright  (C) 1999-2001  Guido Masarotto  and Brian Ripley
- *             (C) 2007-10    The R Core Team
+ *             (C) 2007-12    The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 
 #define R_USE_SIGNALS 1
 #include <Defn.h>
+#include <Internal.h>
 #include "win-nls.h"
 
 #define WIN32_LEAN_AND_MEAN 1
@@ -39,11 +40,13 @@ extern UImode  CharacterMode;
 
 static char RunError[501] = "";
 
-
-static char *expandcmd(const char *cmd)
+/* This might be given a command line (whole = 0) or just the
+   executable (whole = 0).  In the later case the path may or may not
+   be quoted */
+static char *expandcmd(const char *cmd, int whole)
 {
-    char  c;
-    char *s, *p, *q, *f, *dest, *src;
+    char c = '\0';
+    char *s, *p, *q = NULL, *f, *dest, *src;
     int   d, ext, len = strlen(cmd)+1;
     char buf[len], fl[len], fn[len];
 
@@ -57,14 +60,18 @@ static char *expandcmd(const char *cmd)
     /* skip leading spaces */
     for (p = buf; *p && isspace(*p); p++);
     /* find the command itself, possibly double-quoted */
-    for (q = p, d = 0; *q && ( d || !isspace(*q) ); q++)
-	if (*q == '\"') d = d ? 0 : 1;
-    if (d) {
-	strcpy(RunError, "A \" is missing (expandcmd)");
-	return NULL;
+    if (whole) {
+	d = 0;
+    } else {
+	for (q = p, d = 0; *q && ( d || !isspace(*q) ); q++)
+	    if (*q == '\"') d = d ? 0 : 1;
+	if (d) {
+	    strcpy(RunError, "A \" is missing (expandcmd)");
+	    return NULL;
+	}
+	c = *q; /* character after the command, normally a space */
+	*q = '\0';
     }
-    c = *q; /* character after the command, normally a space */
-    *q = '\0';
 
     /*
      * Guido resorted to this since SearchPath returned FOUND also
@@ -98,7 +105,7 @@ static char *expandcmd(const char *cmd)
     if (!d) {
 	free(s);
 	snprintf(RunError, 500, "'%s' not found", p);
-	*q = c;
+	if(!whole) *q = c;
 	return NULL;
     }
     /*
@@ -109,8 +116,10 @@ static char *expandcmd(const char *cmd)
       SearchPath seems dislikes them
     */
     GetShortPathName(fn, s, MAX_PATH);
-    *q = c;
-    strcat(s, q);
+    if (!whole) {
+	*q = c;
+	strcat(s, q);
+    }
     return s;
 }
 
@@ -144,7 +153,7 @@ static void pcreate(const char* cmd, cetype_t enc,
     sa.bInheritHandle = TRUE;
 
     /* FIXME: this might need to be done in wchar_t */
-    if (!(ecmd = expandcmd(cmd))) return; /* error message already set */
+    if (!(ecmd = expandcmd(cmd, 0))) return; /* error message already set */
 
     inpipe = (hIN != INVALID_HANDLE_VALUE)
 	|| (hOUT != INVALID_HANDLE_VALUE)
@@ -230,7 +239,7 @@ static void pcreate(const char* cmd, cetype_t enc,
 	CloseHandle(dupERR);
     }
     if (!ret)
-	snprintf(RunError, 500, _("CreateProcess failed to run '%s'"), ecmd);
+	snprintf(RunError, 500, _("'CreateProcess' failed to run '%s'"), ecmd);
     else CloseHandle(pi->hThread);
     free(ecmd);
     return;
@@ -611,7 +620,7 @@ static Rboolean Wpipe_open(Rconnection con)
 
 static void Wpipe_close(Rconnection con)
 {
-    rpipeClose( ((RWpipeconn)con->private) ->rp);
+    con->status = rpipeClose( ((RWpipeconn)con->private) ->rp);
     con->isopen = FALSE;
 }
 
@@ -778,14 +787,18 @@ SEXP do_syswhich(SEXP call, SEXP op, SEXP args, SEXP env)
     checkArity(op, args);
     nm = CAR(args);
     if(!isString(nm))
-	error(_("'names' is not a character string"));
+	error(_("'names' is not a character vector"));
     n = LENGTH(nm);
     PROTECT(ans = allocVector(STRSXP, n));
     for(i = 0; i < n; i++) {
-	const char *this = CHAR(STRING_ELT(nm, i));
-	char *that = expandcmd(this);
-	SET_STRING_ELT(ans, i, mkChar(that ? that : ""));
-	free(that);
+	if (STRING_ELT(nm, i) == NA_STRING) {
+	    SET_STRING_ELT(ans, i, NA_STRING);
+	} else {
+	    const char *this = CHAR(STRING_ELT(nm, i));
+	    char *that = expandcmd(this, 1);
+	    SET_STRING_ELT(ans, i, mkChar(that ? that : ""));
+	    free(that);
+	}
     }
     setAttrib(ans, R_NamesSymbol, nm);
     UNPROTECT(1);
