@@ -1,6 +1,8 @@
 #  File src/library/base/R/frametools.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -21,7 +23,7 @@ subset.data.frame <- function (x, subset, select, drop = FALSE, ...)
     else {
 	e <- substitute(subset)
 	r <- eval(e, x, parent.frame())
-        if(!is.logical(r)) stop("'subset' must evaluate to logical")
+        if(!is.logical(r)) stop("'subset' must be logical")
 	r <- r & !is.na(r)
     }
     if(missing(select))
