@@ -132,7 +132,7 @@ function(x = 0.1)
     ##
     ## Unspecified bounds are taken as NA_real_, and set to INT_MAX by
     ## the C code.
-    
+
     if(!is.list(x)) {
         ## Sanity checks.
         if(!is.numeric(x) || (x < 0))
@@ -188,7 +188,7 @@ function(x = NULL)
             stop("cost components must be non-negative")
         costs[pos] <- x
     }
-    
+
     costs
 }
 
@@ -281,7 +281,7 @@ function(x, m, invert = FALSE)
 function(x, m, invert = FALSE, value)
 {
     if(!length(x)) return(x)
-    
+
     y <- regmatches(x, m, !invert)
 
     ili <- is.list(m)
@@ -300,12 +300,9 @@ function(x, m, invert = FALSE, value)
         if(np != nv) {
             if(!nv)
                 stop("must have replacement values for matches")
-            value <- rep(value, length.out = np)
+            value <- rep_len(value, np)
         }
-        x[pos] <- paste(sapply(y, `[`, 1L),
-                        value,
-                        sapply(y, `[`, 2L),
-                        sep = "")
+        x[pos] <- paste0(sapply(y, `[`, 1L), value, sapply(y, `[`, 2L))
         return(x)
     }
 
@@ -316,7 +313,7 @@ function(x, m, invert = FALSE, value)
         stop("missing replacement values are not allowed")
     if(!length(value))
         stop("value does not provide any replacement values")
-    value <- rep(value, length.out = length(x))
+    value <- rep_len(value, length(x))
 
     y <- if(invert) {
         ## Replace non-matches.
@@ -328,7 +325,7 @@ function(x, m, invert = FALSE, value)
             if(nv != (nu + 1L)) {
                 if(!nv)
                     stop("must have replacements for non-matches")
-                v <- rep(v, length.out = nu + 1L)
+                v <- rep_len(v, nu + 1L)
             }
             paste0(v, c(u, ""), collapse = "")
         },
@@ -343,7 +340,7 @@ function(x, m, invert = FALSE, value)
             if(nv != (nu - 1L)) {
                 if(!nv)
                     stop("must have replacements for matches")
-                v <- rep(v, length.out = nu - 1L)
+                v <- rep_len(v, nu - 1L)
             }
             paste0(u, c(v, ""), collapse = "")
         },
