@@ -1,6 +1,8 @@
 #  File src/library/base/R/lazyload.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -14,8 +16,14 @@
 #  A copy of the GNU General Public License is available at
 #  http://www.r-project.org/Licenses/
 
-## note that there is a version in ../baseloader.R that should be kept in step
-lazyLoad <- function(filebase, envir = parent.frame(), filter)
+## This code should be kept in step with code in ../baseloader.R
+##
+## This code has been factored in a somewhat peculiar way to allow the
+## lazy load data base mechanism to be used for storing processed .Rd
+## files. This isn't quite right as the .Rd use only uses the data
+## base, not the lazy load part, but for now it will do. LT
+
+lazyLoadDBexec <- function(filebase, fun, filter)
 {
     ##
     ## bootstrapping definitions so we can load base
@@ -75,6 +83,12 @@ lazyLoad <- function(filebase, envir = parent.frame(), filter)
             vars <- names(data$bindings)
             for (i in seq_along(vars))
                 set(vars[i], data$bindings[[i]], e)
+            if (! is.null(data$attributes))
+                attributes(e) <- data$attributes
+            if (! is.null(data$isS4) && data$isS4)
+                .Internal(setS4Object(e, TRUE, TRUE))
+            if (! is.null(data$locked) && data$locked)
+                .Internal(lockEnvironment(e, FALSE))
             e
         }
     }
@@ -86,9 +100,7 @@ lazyLoad <- function(filebase, envir = parent.frame(), filter)
     } else
         vals <-  map$variables
 
-    expr <- quote(lazyLoadDBfetch(key, datafile, compressed, envhook))
-    this <- environment()
-    .Internal(makeLazy(vars, vals, expr, this, envir))
+    res <- fun(environment())
 
     ## reduce memory use
     map <- NULL
@@ -97,4 +109,17 @@ lazyLoad <- function(filebase, envir = parent.frame(), filter)
     rvars <- NULL
     mapfile <- NULL
     readRDS <- NULL
+
+    res
+}
+
+lazyLoad <- function(filebase, envir = parent.frame(), filter)
+{
+    fun <- function(db) {
+        vals <- db$vals
+        vars <- db$vars
+        expr <- quote(lazyLoadDBfetch(key, datafile, compressed, envhook))
+        .Internal(makeLazy(vars, vals, expr, db, envir))
+    }
+    lazyLoadDBexec(filebase, fun, filter)
 }
