@@ -1,6 +1,8 @@
 #  File src/library/base/R/seq.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -23,11 +25,11 @@ seq.default <-
     if((One <- nargs() == 1L) && !missing(from)) {
 	lf <- length(from)
 	return(if(mode(from) == "numeric" && lf == 1L) 1L:from else
-	       if(lf) 1L:lf else integer(0L))
+	       if(lf) 1L:lf else integer())
     }
     if(!missing(along.with)) {
 	length.out <- length(along.with)
-	if(One) return(if(length.out) seq_len(length.out) else integer(0L))
+	if(One) return(if(length.out) seq_len(length.out) else integer())
     }
     else if(!missing(length.out)) {
         len <- length(length.out)
@@ -38,6 +40,14 @@ seq.default <-
         }
 	length.out <- ceiling(length.out)
     }
+    if(!missing(...))
+        warning(sprintf(ngettext(length(list(...)),
+                                 "extra argument %s will be disregarded",
+                                 "extra arguments %s will be disregarded"),
+			 paste(sQuote(names(list(...))), collapse = ", ")),
+		domain = NA)
+    if (!missing(from) && length(from) != 1L) stop("'from' must be of length 1")
+    if (!missing(to) && length(to) != 1L) stop("'to' must be of length 1")
     if(is.null(length.out))
 	if(missing(by))
 	    from:to
@@ -69,8 +79,7 @@ seq.default <-
 	}
     else if(!is.finite(length.out) || length.out < 0L)
 	stop("length must be non-negative number")
-    else if(length.out == 0L)
-	integer(0L)
+    else if(length.out == 0L) integer()
     else if (One) seq_len(length.out)
     else if(missing(by)) {
 	# if(from == to || length.out < 2) by <- 1
@@ -78,9 +87,8 @@ seq.default <-
 	    to <- from + length.out - 1L
 	if(missing(from))
 	    from <- to - length.out + 1L
-	if(length.out > 2L)
-	    if(from == to)
-		rep.int(from, length.out)
+	if(length.out > 2L) # not clear why these have as.vector, and not others
+	    if(from == to) rep.int(from, length.out)
 	    else as.vector(c(from, from + seq_len(length.out - 2L) * by, to))
 	else as.vector(c(from, to))[seq_len(length.out)]
     }
