@@ -21,7 +21,7 @@ tapply <- function (X, INDEX, FUN = NULL, ..., simplify = TRUE)
     FUN <- if (!is.null(FUN)) match.fun(FUN)
     if (!is.list(INDEX)) INDEX <- list(INDEX)
     nI <- length(INDEX)
-    if (!nI) warning("'INDEX' is of length zero")
+    if (!nI) stop("'INDEX' is of length zero")
     namelist <- vector("list", nI)
     names(namelist) <- names(INDEX)
     extent <- integer(nI)
