@@ -1,6 +1,8 @@
 #  File src/library/base/R/unname.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -14,10 +16,11 @@
 #  A copy of the GNU General Public License is available at
 #  http://www.r-project.org/Licenses/
 
-unname <- function (obj, force= FALSE) {
-    if (length(names(obj)))
+unname <- function (obj, force = FALSE)
+{
+    if (!is.null(names(obj)))
         names(obj) <- NULL
-    if (length(dimnames(obj)) && (force || !is.data.frame(obj)))
+    if (!is.null(dimnames(obj)) && (force || !is.data.frame(obj)))
         dimnames(obj) <- NULL
     obj
 }
