@@ -1,6 +1,8 @@
 #  File src/library/grDevices/R/unix/dev2bitmap.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -34,45 +36,49 @@ dev2bitmap <- function(file, type="png16m", height = 7, width = 7, res = 72,
     gshelp <- system(paste(gsexe, "-help"), intern=TRUE)
     st <- grep("^Available", gshelp)
     en <- grep("^Search", gshelp)
-    gsdevs <- gshelp[(st+1):(en-1)]
-    devs <- c(strsplit(gsdevs, " "), recursive=TRUE)
-    if(match(type, devs, 0) == 0)
-        stop(gettextf("device '%s' is not available\n", type),
-             gettextf("Available devices are %s",
-                      paste(gsdevs, collapse="\n")),
-             domain = NA)
+    if(!length(st) || !length(en))
+        warning("unrecognized format of gs -help")
+    else {
+        gsdevs <- gshelp[(st+1):(en-1)]
+        devs <- c(strsplit(gsdevs, " "), recursive=TRUE)
+        if(match(type, devs, 0) == 0)
+            stop(gettextf("device '%s' is not available\n", type),
+                 gettextf("Available devices are %s",
+                          paste(gsdevs, collapse = "\n")),
+                 domain = NA)
+    }
     if(missing(pointsize)) pointsize <- 1.5*min(width, height)
     tmp <- tempfile("Rbit")
     on.exit(unlink(tmp))
-    din <- graphics::par("din"); w <- din[1]; h <- din[2]
+    din <- graphics::par("din"); w <- din[1L]; h <- din[2L]
     if(missing(width) && !missing(height)) width <- w/h * height
     if(missing(height) && !missing(width)) height <- h/w * width
 
     current.device <- dev.cur()
     if(method == "pdf")
-        dev.off(dev.copy(device = pdf, file=tmp, width=width,
-                         height=height,
-                         pointsize=pointsize, paper="special", ...))
+        dev.off(dev.copy(device = pdf, file = tmp, width = width,
+                         height = height,
+                         pointsize = pointsize, paper = "special", ...))
     else
-        dev.off(dev.copy(device = postscript, file=tmp, width=width,
-                         height=height,
-                         pointsize=pointsize, paper="special",
-                         horizontal=FALSE, ...))
+        dev.off(dev.copy(device = postscript, file = tmp, width = width,
+                         height = height,
+                         pointsize = pointsize, paper = "special",
+                         horizontal = FALSE, ...))
     dev.set(current.device)
     extra <- ""
-    if (!is.na(taa)) extra <- paste(" -dTextAlphaBits=", taa, sep="")
-    if (!is.na(gaa)) extra <- paste(extra, " -dGraphicsAlphaBits=", gaa, sep="")
-    cmd <- paste(shQuote(gsexe), " -dNOPAUSE -dBATCH -q -sDEVICE=", type,
-                 " -r", res,
-                 "-dAutoRotatePages=/None",
-                 " -g", ceiling(res*width), "x", ceiling(res*height),
+    if (!is.na(taa)) extra <- paste0(" -dTextAlphaBits=", taa)
+    if (!is.na(gaa)) extra <- paste0(extra, " -dGraphicsAlphaBits=", gaa)
+    cmd <- paste0(shQuote(gsexe), " -dNOPAUSE -dBATCH -q -sDEVICE=", type,
+                  " -r", res,
+                  " -dAutoRotatePages=/None",
+                  " -g", ceiling(res*width), "x", ceiling(res*height),
                   extra,
-                " -sOutputFile=", shQuote(file), " ", tmp, sep="")
+                  " -sOutputFile=", shQuote(file), " ", tmp)
     system(cmd)
     invisible()
 }
 
-bitmap <- function(file, type="png16m", height = 7, width = 7, res = 72,
+bitmap <- function(file, type = "png16m", height = 7, width = 7, res = 72,
                    units = "in", pointsize, taa = NA, gaa = NA, ...)
 {
     if(missing(file)) stop("'file' is missing with no default")
@@ -87,28 +93,32 @@ bitmap <- function(file, type="png16m", height = 7, width = 7, res = 72,
         rc <- system(paste(gsexe, "-help > /dev/null"))
         if(rc != 0) stop("sorry, 'gs' cannot be found")
     }
-    gshelp <- system(paste(gsexe, "-help"), intern=TRUE)
+    gshelp <- system(paste(gsexe, "-help"), intern = TRUE)
     st <- grep("^Available", gshelp)
     en <- grep("^Search", gshelp)
-    gsdevs <- gshelp[(st+1):(en-1)]
-    devs <- c(strsplit(gsdevs, " "), recursive=TRUE)
-    if(match(type, devs, 0) == 0)
-        stop(gettextf("device '%s' is not available\n", type),
-             gettextf("Available devices are %s",
-                      paste(gsdevs, collapse="\n")),
-             domain = NA)
+    if(!length(st) || !length(en))
+        warning("unrecognized format of gs -help")
+    else {
+        gsdevs <- gshelp[(st+1):(en-1)]
+        devs <- c(strsplit(gsdevs, " "), recursive = TRUE)
+        if(match(type, devs, 0) == 0)
+            stop(gettextf("device '%s' is not available\n", type),
+                 gettextf("Available devices are %s",
+                          paste(gsdevs, collapse = "\n")),
+                 domain = NA)
+    }
     if(missing(pointsize)) pointsize <- 1.5*min(width, height)
     extra <- ""
-    if (!is.na(taa)) extra <- paste(" -dTextAlphaBits=", taa, sep="")
-    if (!is.na(gaa)) extra <- paste(extra, " -dGraphicsAlphaBits=", gaa, sep="")
-    cmd <- paste("|", shQuote(gsexe),
-                 " -dNOPAUSE -dBATCH -q -sDEVICE=", type,
-                 " -r", res,
-                 "-dAutoRotatePages=/None",
-                 " -g", ceiling(res*width), "x", ceiling(res*height),
-                 extra,
-                 " -sOutputFile=", shQuote(file), " -", sep="")
-    postscript(file=cmd, width=width, height=height,
-               pointsize=pointsize, paper="special", horizontal=FALSE, ...)
+    if (!is.na(taa)) extra <- paste0(" -dTextAlphaBits=", taa)
+    if (!is.na(gaa)) extra <- paste0(extra, " -dGraphicsAlphaBits=", gaa)
+    cmd <- paste0("|", shQuote(gsexe),
+                  " -dNOPAUSE -dBATCH -q -sDEVICE=", type,
+                  " -r", res,
+                  " -dAutoRotatePages=/None",
+                  " -g", ceiling(res*width), "x", ceiling(res*height),
+                  extra,
+                  " -sOutputFile=", shQuote(file), " -")
+    postscript(file = cmd, width = width, height = height,
+               pointsize = pointsize, paper = "special", horizontal = FALSE, ...)
     invisible()
 }
