@@ -53,35 +53,30 @@ static SEXP cairoProps(SEXP in)
 }
 #endif
 
-
-static R_NativePrimitiveArgType R_chull_t[] = {INTSXP, REALSXP, INTSXP, INTSXP, INTSXP, INTSXP, INTSXP, INTSXP, INTSXP};
-
-#ifndef WIN32
-void *getQuartzAPI();
-#endif
-
-#define CDEF(name)  {#name, (DL_FUNC) &name, sizeof(name ## _t)/sizeof(name ## _t[0]), name ##_t}
-
-static R_CMethodDef CEntries [] = {
-    CDEF(R_chull),
-#ifndef WIN32
-    {"getQuartzAPI", (DL_FUNC) getQuartzAPI, 0},
-#endif
-    {NULL, NULL, 0}
-};
-
 #define CALLDEF(name, n)  {#name, (DL_FUNC) &name, n}
 
 static const R_CallMethodDef CallEntries[] = {
-    // NB: each *also* needs an entry in useDynLib() in ../NAMESPACE !
     CALLDEF(Type1FontInUse, 2),
     CALLDEF(CIDFontInUse, 2),
     CALLDEF(R_CreateAtVector, 4),
     CALLDEF(R_GAxisPars, 3),
-    {"R_GD_nullDevice", (DL_FUNC) &R_GD_nullDevice, 0},
+    CALLDEF(chull, 1),
+    CALLDEF(gray, 2),
+    CALLDEF(RGB2hsv, 1),
+    CALLDEF(rgb, 6),
+    CALLDEF(hsv, 4),
+    CALLDEF(hcl, 5),
+    CALLDEF(col2rgb, 2),
+    CALLDEF(colors, 0),
+    CALLDEF(palette, 1),
+    CALLDEF(palette2, 1),
+
 #ifndef WIN32
     CALLDEF(makeQuartzDefault, 0),
     CALLDEF(cairoProps, 1),
+#else
+    CALLDEF(bringToTop, 2),
+    CALLDEF(msgWindow, 2),
 #endif
     {NULL, NULL, 0}
 };
@@ -94,17 +89,51 @@ static const R_ExternalMethodDef ExtEntries[] = {
     EXTDEF(XFig, 14),
     EXTDEF(PDF, 20),
     EXTDEF(devCairo, 10),
-#ifdef WIN32
-    EXTDEF(devga, 19),
+    EXTDEF(devcap, 0),
+    EXTDEF(devcapture, 1),
+    EXTDEF(devcontrol, 1),
+    EXTDEF(devcopy, 1),
+    EXTDEF(devcur, 0),
+    EXTDEF(devdisplaylist, 0),
+    EXTDEF(devholdflush, 1),
+    EXTDEF(devnext, 1),
+    EXTDEF(devoff, 1),
+    EXTDEF(devprev, 1),
+    EXTDEF(devset, 1),
+    EXTDEF(devsize, 0),
     EXTDEF(savePlot, 3),
+    EXTDEF(contourLines, 4),
+    EXTDEF(getSnapshot, 0),
+    EXTDEF(playSnapshot, 1),
+    EXTDEF(getGraphicsEvent, 1),
+    EXTDEF(getGraphicsEventEnv, 1),
+    EXTDEF(setGraphicsEventEnv, 2),
+    EXTDEF(devAskNewPage, 1),
+
+#ifdef WIN32
+    EXTDEF(devga, 21),
 #else
-    EXTDEF(Quartz, 12),
+    EXTDEF(Quartz, 11),
+    EXTDEF(X11, 17),
 #endif
     {NULL, NULL, 0}
 };
 
+#ifdef HAVE_AQUA
+extern void setup_RdotApp(void);
+extern Rboolean useaqua;
+#endif
+
 void R_init_grDevices(DllInfo *dll)
 {
-    R_registerRoutines(dll, CEntries, CallEntries, NULL, ExtEntries);
+    initPalette();
+    R_registerRoutines(dll, NULL, CallEntries, NULL, ExtEntries);
     R_useDynamicSymbols(dll, FALSE);
+    R_forceSymbols(dll, TRUE);
+
+#ifdef HAVE_AQUA
+/* R.app will run event loop, so if we are running under that we don't
+   need to run one here */
+    if(useaqua) setup_RdotApp();
+#endif
 }
