@@ -24,16 +24,11 @@
 
 mc_pids <- new.env()
 assign("pids", integer(), envir = mc_pids)
-clean_pids <- function(e) {
-    pids <- get("pids", envir = e)
-    if(length(pids)) {
-        library.dynam("tools", "tools", .Library)
-        .Call("ps_kill", pids, 9L, PACKAGE = "tools")
-    }
-}
+clean_pids <- function(e)
+    if(length(pids <- get("pids", envir = e))) tools::pskill(pids, tools::SIGKILL)
 
 mcfork <- function() {
-    r <- .Call(C_mc_fork, PACKAGE = "parallel")
+    r <- .Call(C_mc_fork)
     assign("pids", c(get("pids",envir = mc_pids), r[1L]), envir = mc_pids)
     structure(list(pid = r[1L], fd = r[2:3]),
               class = c(if(r[1L]) "childProcess"
@@ -76,7 +71,7 @@ rmChild <- function(child)
 mckill <- function(process, signal = 2L)
 {
     process <- processID(process)
-    ## or simply parallel::pskill(process, signal)
+    ## or simply tools::pskill(process, signal)
     unlist(lapply(process, function(p)
                   .Call(C_mc_kill, as.integer(p), as.integer(signal))))
 }
@@ -92,7 +87,8 @@ sendMaster <- function(what)
 processID <- function(process) {
     if (inherits(process, "process")) process$pid
     else if (is.list(process)) unlist(lapply(process, processID))
-    else stop("'process' must be of the class \"process\"")
+    else stop(gettextf("'process' must be of class %s", dQuote("process")),
+              domain = NA)
 }
 
 # unused
