@@ -1,6 +1,8 @@
 #  File src/library/stats/R/addmargins.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -42,8 +44,7 @@ addmargins <-
     ## length as the margins vector.
     if (length(FUN) == 1 && !is.list(FUN)) {
 	fname <- if (!miss.FUN) deparse(substitute(FUN)) else "Sum"
-	FUN <- list(FUN)
-	names(FUN) <- fname
+	FUN <- setNames(list(FUN), fname)
     }
 
     if (!miss.FUN) {
@@ -99,7 +100,7 @@ addmargins <-
 		    paste("Margin ", margin[i], ".", fnames[[i]][blank],
 			  sep = "")
 	    } else {
-		fnames[[i]] <- paste(topname, ".", fnames[[i]], sep = "")
+		fnames[[i]] <- paste0(topname, ".", fnames[[i]])
 	    }
 	} else
 	    if (fnames[[i]] == "") fnames[[i]] <- paste("Margin", margin[i])
@@ -117,7 +118,7 @@ addmargins <-
 	## BxC, August 2003
 
 	## Make sure that FUN is a list
-	if(class(FUN) != "list") FUN <- list(FUN)
+	if(!inherits(FUN, "list")) FUN <- list(FUN)
 
 	## Useful constants
 	d <- dim(A)
@@ -141,7 +142,7 @@ addmargins <-
 		    n.new/(skip+n.mar*runl))
 
 	## Define a vector to hold all the values of the new table
-	values <- numeric(apos)
+	values <- double(length(apos))
 
 	## First fill in the body of the table
 	values[apos] <- as.vector(A)
@@ -178,10 +179,11 @@ addmargins <-
 
     ## Done! Now print it.
     ##
-    if(!quiet && !miss.FUN  && n.sid > 1) {
+    if(!quiet && !miss.FUN && n.sid > 1) {
 	cat("Margins computed over dimensions\nin the following order:\n")
-	for(i in 1L:n.sid)
-	    cat(paste(i), ": ", names(dimnames(A))[margin[i]], "\n", sep="")
+        ## FIXME: what is paste(i) supposed to do?
+	for(i in seq_len(n.sid))
+	    cat(paste(i), ": ", names(dimnames(A))[margin[i]], "\n", sep = "")
     }
     new.A
 }
