@@ -1,6 +1,8 @@
 #  File src/library/stats/R/kmeans.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -22,46 +24,49 @@ function(x, centers, iter.max = 10, nstart = 1,
         Z <-
             switch(nmeth,
                    { # 1
-                       Z <- .Fortran(C_kmns, as.double(x), as.integer(m),
-                                as.integer(ncol(x)),
-                                centers = as.double(centers),
+                       Z <- .Fortran(C_kmns, x, m, p,
+                                centers = centers,
                                 as.integer(k), c1 = integer(m), integer(m),
                                 nc = integer(k), double(k), double(k), integer(k),
                                 double(m), integer(k), integer(k),
-                                as.integer(iter.max), wss = double(k),
+                                iter.max, wss = double(k),
                                 ifault = 0L)
                        switch(Z$ifault,
                               stop("empty cluster: try a better set of initial centers",
-                                   call.=FALSE),
-                              warning(gettextf("did not converge in %d iterations",
-                                               iter.max), call.=FALSE, domain =NA),
+                                   call. = FALSE),
+                              warning(sprintf(ngettext(iter.max,
+                                                       "did not converge in %d iteration",
+                                                       "did not converge in %d iterations"),
+                                              iter.max), call. = FALSE, domain =NA),
                               stop("number of cluster centres must lie between 1 and nrow(x)",
                                    call.=FALSE)
                               )
                        Z
                    },
                    { # 2
-                       Z <- .C(C_kmeans_Lloyd, as.double(x), as.integer(m),
-                               as.integer(ncol(x)),
-                               centers = as.double(centers), as.integer(k),
-                               c1 = integer(m), iter = as.integer(iter.max),
+                       Z <- .C(C_kmeans_Lloyd, x, m, p,
+                               centers = centers, k,
+                               c1 = integer(m), iter = iter.max,
                                nc = integer(k), wss = double(k))
                        if(Z$iter > iter.max)
-                           warning("did not converge in ",
-                                  iter.max, " iterations", call.=FALSE)
+                           warning(sprintf(ngettext(iter.max,
+                                                    "did not converge in %d iteration",
+                                                    "did not converge in %d iterations"),
+                                           iter.max), call.=FALSE, domain = NA)
                        if(any(Z$nc == 0))
                            warning("empty cluster: try a better set of initial centers", call.=FALSE)
                        Z
                    },
                    { # 3
-                       Z <- .C(C_kmeans_MacQueen, as.double(x), as.integer(m),
-                               as.integer(ncol(x)),
-                               centers = as.double(centers), as.integer(k),
-                               c1 = integer(m), iter = as.integer(iter.max),
+                       Z <- .C(C_kmeans_MacQueen, x, m, p,
+                               centers = as.double(centers), k,
+                               c1 = integer(m), iter = iter.max,
                                nc = integer(k), wss = double(k))
                        if(Z$iter > iter.max)
-                           warning("did not converge in ",
-                                   iter.max, " iterations", call.=FALSE)
+                           warning(sprintf(ngettext(iter.max,
+                                                    "did not converge in %d iteration",
+                                                    "did not converge in %d iterations"),
+                                           iter.max), call.=FALSE, domain = NA)
                        if(any(Z$nc == 0))
                            warning("empty cluster: try a better set of initial centers", call.=FALSE)
                        Z
@@ -69,13 +74,17 @@ function(x, centers, iter.max = 10, nstart = 1,
         Z
     }
     x <- as.matrix(x)
-    m <- nrow(x)
+    m <- as.integer(nrow(x))
+    if(is.na(m)) stop("invalid nrow(x)")
+    p <- as.integer(ncol(x))
+    if(is.na(p)) stop("invalid ncol(x)")
     if(missing(centers))
 	stop("'centers' must be a number or a matrix")
     nmeth <- switch(match.arg(algorithm),
                     "Hartigan-Wong" = 1,
                     "Lloyd" = 2, "Forgy" = 2,
                     "MacQueen" = 3)
+    storage.mode(x) <- "double"
     if(length(centers) == 1L) {
 	if (centers == 1) nmeth <- 3
 	k <- centers
@@ -98,9 +107,13 @@ function(x, centers, iter.max = 10, nstart = 1,
         if(m < k)
             stop("more cluster centers than data points")
     }
-    if(iter.max < 1) stop("'iter.max' must be positive")
+    k <- as.integer(k)
+    if(is.na(k)) stop("'invalid value of 'k'")
+    iter.max <- as.integer(iter.max)
+    if(is.na(iter.max) || iter.max < 1) stop("'iter.max' must be positive")
     if(ncol(x) != ncol(centers))
 	stop("must have same number of columns in 'x' and 'centers'")
+    storage.mode(centers) <- "double"
     Z <- do_one(nmeth)
     best <- sum(Z$wss)
     if(nstart >= 2 && !is.null(cn))
@@ -141,3 +154,11 @@ print.kmeans <- function(x, ...)
     print(names(x))
     invisible(x)
 }
+
+fitted.kmeans <- function(object, method = c("centers", "classes"), ...)
+{
+	method <- match.arg(method)
+	if (method == "centers") object$centers[object$cl, , drop=FALSE]
+	else object$cl
+}
+
