@@ -77,8 +77,8 @@ na.omit.data.frame <- function(object, ...)
     }
     xx <- object[!omit, , drop = FALSE]
     if (any(omit > 0L)) {
-	temp <- seq(omit)[omit]
-	names(temp) <- attr(object, "row.names")[omit]
+	temp <- setNames(seq(omit)[omit],
+			 attr(object, "row.names")[omit])
 	attr(temp, "class") <- "omit"
 	attr(xx, "na.action") <- temp
     }
@@ -131,8 +131,8 @@ na.exclude.data.frame <- function(object, ...)
     }
     xx <- object[!omit, , drop = FALSE]
     if (any(omit > 0L)) {
-	temp <- seq(omit)[omit]
-	names(temp) <- attr(object, "row.names")[omit]
+	temp <- setNames(seq(omit)[omit],
+			 attr(object, "row.names")[omit])
 	attr(temp, "class") <- "exclude"
 	attr(xx, "na.action") <- temp
     }
