@@ -106,10 +106,10 @@ summary.prcomp <- function(object, ...)
 }
 
 print.summary.prcomp <-
-function(x, digits = max(3, getOption("digits") - 3), ...)
+function(x, digits = max(3L, getOption("digits") - 3L), ...)
 {
     cat("Importance of components:\n")
-    print(x$importance, digits = digits)
+    print(x$importance, digits = digits, ...)
     invisible(x)
 }
 
