@@ -1,6 +1,8 @@
 #  File src/library/stats/R/princomp.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -52,8 +54,6 @@ princomp.default <-
 {
     cl <- match.call()
     cl[[1L]] <- as.name("princomp")
-    if(!missing(x) && !missing(covmat))
-        warning("both 'x' and 'covmat' were supplied: 'x' will be ignored")
     z <- if(!missing(x)) as.matrix(x)[subset, , drop = FALSE]
     if (is.list(covmat)) {
         if(any(is.na(match(c("cov", "n.obs"), names(covmat)))))
@@ -62,6 +62,8 @@ princomp.default <-
         n.obs <- covmat$n.obs
         cen <- covmat$center
     } else if(is.matrix(covmat)) {
+	if(!missing(x)) ## warn only here; x is used for scores when we have 'cen'
+	    warning("both 'x' and 'covmat' were supplied: 'x' will be ignored")
         cv <- covmat
         n.obs <- NA
         cen <- NULL
@@ -78,7 +80,7 @@ princomp.default <-
     if (cor) {
         sds <- sqrt(diag(cv))
         if(any(sds == 0))
-            stop("cannot use cor=TRUE with a constant variable")
+            stop("cannot use 'cor = TRUE' with a constant variable")
         cv <- cv/(sds %o% sds)
     }
     edc <- eigen(cv, symmetric = TRUE)
@@ -95,8 +97,8 @@ princomp.default <-
     dimnames(edc$vectors) <- if(missing(x))
         list(dimnames(cv)[[2L]], cn) else list(dimnames(x)[[2L]], cn)
     sdev <- sqrt(ev)
-    sc <- if (cor) sds else rep(1, ncol(cv))
-    names(sc) <- colnames(cv)
+    sc <- setNames(if (cor) sds else rep.int(1, ncol(cv)),
+		   colnames(cv))
     scr <- if (scores && !missing(x) && !is.null(cen))
         scale(z, center = cen, scale = sc) %*% edc$vectors
     if (is.null(cen)) cen <- rep(NA_real_, nrow(cv))
