@@ -45,7 +45,7 @@ splinefun <-
         m <- c(Sx[1L], (Sx[-1L] + Sx[-n1])/2, Sx[n1]) ## 1.
 
         ## use C, as we need to "serially" progress from left to right:
-        m <- .Call(C_R_monoFC_m, m, Sx)
+        m <- .Call(C_monoFC_m, m, Sx)
 
         ## Hermite spline with (x,y,m) :
         return(splinefunH0(x = x, y = y, m = m, dx = dx))
@@ -58,18 +58,9 @@ splinefun <-
         if(!(all(dy >= 0) || all(dy <= 0)))
             stop("'y' must be increasing or decreasing")
     }
-    z <- .C(C_spline_coef,
-	    method=as.integer(min(3L, iMeth)),
-	    n=nx,
-	    x=x,
-	    y=y,
-	    b=double(nx),
-	    c=double(nx),
-	    d=double(nx),
-	    e=double(if(iMeth == 1) nx else 0))
+    z <- .Call(C_SplineCoef, min(3L, iMeth), x, y)
     if(iMeth == 5L) z <- spl_coef_conv(hyman_filter(z))
     rm(x, y, nx, method, iMeth, ties)
-    z$e <- NULL
     function(x, deriv = 0L) {
 	deriv <- as.integer(deriv)
 	if (deriv < 0L || deriv > 3L)
@@ -80,26 +71,17 @@ splinefun <-
 	    z0 <- double(z$n)
 	    z[c("y", "b", "c")] <-
 		switch(deriv,
-		       list(y=	 z$b, b = 2*z$c, c = 3*z$d), # deriv = 1
-		       list(y= 2*z$c, b = 6*z$d, c =	z0), # deriv = 2
-		       list(y= 6*z$d, b =    z0, c =	z0)) # deriv = 3
+		       list(y =	 z$b , b = 2*z$c, c = 3*z$d), # deriv = 1
+		       list(y = 2*z$c, b = 6*z$d, c =	z0), # deriv = 2
+		       list(y = 6*z$d, b =    z0, c =	z0)) # deriv = 3
 	    z[["d"]] <- z0
 	}
         ## yout[j] := y[i] + dx*(b[i] + dx*(c[i] + dx* d_i))
         ##           where dx := (u[j]-x[i]); i such that x[i] <= u[j] <= x[i+1},
         ##                u[j]:= xout[j] (unless sometimes for periodic spl.)
         ##           and  d_i := d[i] unless for natural splines at left
-	res <- .C(C_spline_eval,
-                  z$method,
-                  as.integer(length(x)),
-                  x=as.double(x),
-                  y=double(length(x)),
-                  z$n,
-                  z$x,
-                  z$y,
-                  z$b,
-                  z$c,
-                  z$d)$y
+        res <- .splinefun(x, z)
+
 
         ## deal with points to the left of first knot if natural
         ## splines are used  (Bug PR#13132)
@@ -110,6 +92,9 @@ splinefun <-
     }
 }
 
+## avoid capturing internal calls
+.splinefun <- function(x, z) .Call(C_SplineEval, x, z)
+
 ## hidden : The exported user function is splinefunH()
 splinefunH0 <- function(x, y, m, dx = x[-1L] - x[-length(x)])
 {
