@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2000--2010  The R Development Core Team
+ *  Copyright (C) 2000--2010  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -444,7 +444,7 @@ SEXP RTcl_ObjAsIntVector(SEXP args)
     ans = allocVector(INTSXP, count);
     for (i = 0 ; i < count ; i++){
 	ret = Tcl_GetIntFromObj(RTcl_interp, elem[i], &x);
-	if (ret != TCL_OK) x = NA_REAL;
+	if (ret != TCL_OK) x = NA_INTEGER;
 	INTEGER(ans)[i] = x;
     }
     return ans;
@@ -505,6 +505,20 @@ SEXP RTcl_ObjAsRawVector(SEXP args)
     return ans;
 }
 
+SEXP RTcl_ObjFromRawVector(SEXP args)
+{
+    int count;
+    Tcl_Obj *tclobj; 
+    SEXP val; 
+
+    val = CADR(args);
+
+    count = length(val);
+    tclobj = Tcl_NewByteArrayObj(RAW(val), count);
+
+    return makeRTclObject(tclobj);
+}
+
 
 SEXP RTcl_GetArrayElem(SEXP args)
 {
