@@ -18,6 +18,17 @@
 
 # Functions for making Rd and human readable versions of bibentry records.
 
+# Clean up LaTeX accents and braces
+cleanupLatex <- function(x) {
+    if (!length(x)) return(x)
+    latex <- try(parseLatex(x), silent=TRUE)
+    if (inherits(latex, "try-error")) {
+    	x
+    } else {
+    	deparseLatex(latexToUtf8(latex), dropBraces=TRUE)
+    }
+}
+
 makeJSS <- function()
     local({
 
@@ -31,21 +42,10 @@ makeJSS <- function()
 	    sub("([^.?!])$", "\\1.", string)
 
 	# Separate args by sep, add a period at the end.
-	sentence <- function(..., sep=", ") {
+	sentence <- function(..., sep = ", ") {
 	    strings <- c(...)
 	    if (length(strings)) {
-		addPeriod(paste(strings, collapse=sep))
-	    }
-	}
-
-	# Clean up LaTeX accents and braces
-	cleanup <- function(x) {
-	    if (!length(x)) return(x)
-	    latex <- try(parseLatex(x), silent=TRUE)
-	    if (inherits(latex, "try-error")) {
-	    	x
-	    } else {
-	    	deparseLatex(latexToUtf8(latex), dropBraces=TRUE)
+		addPeriod(paste(strings, collapse = sep))
 	    }
 	}
 
@@ -54,12 +54,12 @@ makeJSS <- function()
 	plain <- function(pages)
 	    if (length(pages)) collapse(pages)
 
-	plainclean <- function(s) plain(cleanup(s))
+	plainclean <- function(s) plain(cleanupLatex(s))
 
 	emph <- function(s)
 	    if (length(s)) paste0("\\emph{", collapse(s), "}")
 
-        emphclean <- function(s) emph(cleanup(s))
+        emphclean <- function(s) emph(cleanupLatex(s))
 
 	# This creates a function to label a field by adding a prefix or suffix (or both)
 
@@ -71,7 +71,7 @@ makeJSS <- function()
 
 	labelclean <- function(prefix=NULL, suffix=NULL, style=plain) {
 	    f <- label(prefix, suffix, style)
-	    function(s) f(cleanup(s))
+	    function(s) f(cleanupLatex(s))
 	}
 
 	# Now the formatters for each particular field.  These take
@@ -95,8 +95,9 @@ makeJSS <- function()
 	fmtTechreportnumber <- labelclean(prefix="Technical Report ")
 	fmtUrl <- label(prefix="\\url{", suffix="}")
 	fmtTitle <- function(title)
-	    if (length(title)) paste("\\dQuote{",
-				     addPeriod(collapse(cleanup(title))), "}", sep="")
+	    if (length(title))
+                paste0("\\dQuote{",
+                      addPeriod(collapse(cleanupLatex(title))), "}")
 
 	fmtYear <- function(year) {
 	    if (!length(year)) year <- "????"
@@ -116,15 +117,15 @@ makeJSS <- function()
 	## Format one person object in short "Murdoch DJ" format
 	shortName <- function(person) {
 	    if (length(person$family)) {
-		result <- cleanup(person$family)
+		result <- cleanupLatex(person$family)
 		if (length(person$given))
 		    paste(result,
-			  paste(substr(sapply(person$given, cleanup),
+			  paste(substr(sapply(person$given, cleanupLatex),
 				       1, 1), collapse=""))
 		else result
 	    }
 	    else
-		paste(cleanup(person$given), collapse=" ")
+		paste(cleanupLatex(person$given), collapse=" ")
 	}
 
 	# Format all authors for one paper
@@ -172,22 +173,23 @@ makeJSS <- function()
 	    if (length(book$publisher)) {
 		result <- collapse(book$publisher)
 		if (length(book$address))
-		    result <- paste(result, collapse(book$address), sep=", ")
+		    result <- paste(result, collapse(book$address), sep = ", ")
 		result
 	    }
 	}
 
 	procOrganization <- function(paper) {
 	    if (length(paper$organization)) {
-		result <- collapse(cleanup(paper$organization))
+		result <- collapse(cleanupLatex(paper$organization))
 		if (length(paper$address))
-		    result <- paste(result, collapse(cleanup(paper$address)), sep=", ")
+		    result <- paste(result, collapse(cleanupLatex(paper$address)), sep =", ")
 		result
 	    }
 	}
 
 	formatArticle <- function(paper) {
-	    collapse(c(sentence(authorList(paper), fmtYear(paper$year), sep=" "),
+	    collapse(c(fmtPrefix(paper),
+	             sentence(authorList(paper), fmtYear(paper$year), sep = " "),
 		     fmtTitle(paper$title),
 		     sentence(fmtBook(paper$journal), volNum(paper), fmtPages(paper$pages)),
 		     sentence(fmtISSN(paper$issn), extraInfo(paper))))
@@ -195,10 +197,11 @@ makeJSS <- function()
 
 	formatBook <- function(book) {
 	    authors <- authorList(book)
-	    if (is.null(authors))
+	    if(!length(authors))
 		authors <- editorList(book)
 
-	    collapse(c(sentence(authors, fmtYear(book$year), sep=" "),
+	    collapse(c(fmtPrefix(book),
+	               sentence(authors, fmtYear(book$year), sep = " "),
 		       sentence(fmtBtitle(book$title), bookVolume(book), fmtEdition(book$edition)),
 		       sentence(bookPublisher(book)),
 		       sentence(fmtISBN(book$isbn), extraInfo(book))))
@@ -207,13 +210,14 @@ makeJSS <- function()
 	formatInbook <- function(paper) {
 	    authors <- authorList(paper)
 	    editors <- editorList(paper)
-	    if (is.null(authors)) {
+	    if(!length(authors)) {
 		authors <- editors
 		editors <- NULL
 	    }
-	    collapse(c(sentence(authors, fmtYear(paper$year), sep=" "),
+	    collapse(c(fmtPrefix(paper),
+	               sentence(authors, fmtYear(paper$year), sep =" "),
 		       fmtTitle(paper$title),
-		       paste("In", sentence(editors, fmtBtitle(paper$fmtBtitle), bookVolume(paper),
+		       paste("In", sentence(editors, fmtBtitle(paper$booktitle), bookVolume(paper),
 					    fmtChapter(paper$chapter),
 					    fmtEdition(paper$edition), fmtPages(paper$pages))),
 		       sentence(bookPublisher(paper)),
@@ -221,16 +225,18 @@ makeJSS <- function()
 	}
 
 	formatIncollection <- function(paper) {
-	    collapse(c(sentence(authorList(paper), fmtYear(paper$year), sep=" "),
+	    collapse(c(fmtPrefix(paper),
+	               sentence(authorList(paper), fmtYear(paper$year), sep = " "),
 		       fmtTitle(paper$title),
-		       paste("In", sentence(editorList(paper), fmtBtitle(paper$fmtBtitle), bookVolume(paper),
+		       paste("In", sentence(editorList(paper), fmtBtitle(paper$booktitle), bookVolume(paper),
 					    fmtEdition(paper$edition), fmtPages(paper$pages))),
 		       sentence(bookPublisher(paper)),
 		       sentence(fmtISBN(paper$isbn), extraInfo(paper))))
 	}
 
 	formatInProceedings <- function(paper)
-	    collapse(c(sentence(authorList(paper), fmtYear(paper$year), sep=" "),
+	    collapse(c(fmtPrefix(paper),
+	               sentence(authorList(paper), fmtYear(paper$year), sep = " "),
 		       fmtTitle(paper$title),
 		       paste("In", sentence(editorList(paper), fmtBtitle(paper$booktitle), bookVolume(paper),
 					    fmtEdition(paper$edition), fmtPages(paper$pages))),
@@ -238,28 +244,32 @@ makeJSS <- function()
 		       sentence(fmtISBN(paper$isbn), extraInfo(paper))))
 
 	formatManual <- function(paper) {
-	    collapse(c(sentence(authorList(paper), fmtYear(paper$year), sep=" "),
+	    collapse(c(fmtPrefix(paper),
+	               sentence(authorList(paper), fmtYear(paper$year), sep = " "),
 		       sentence(fmtBtitle(paper$title), bookVolume(paper), fmtEdition(paper$edition)),
 		       sentence(procOrganization(paper)),
 		       sentence(fmtISBN(paper$isbn), extraInfo(paper))))
 	}
 
 	formatMastersthesis <- function(paper) {
-	    collapse(c(sentence(authorList(paper), fmtYear(paper$year), sep=" "),
+	    collapse(c(fmtPrefix(paper),
+	               sentence(authorList(paper), fmtYear(paper$year), sep = " "),
 		       sentence(fmtBtitle(paper$title)),
 		       sentence("Master's thesis", fmtSchool(paper$school), fmtAddress(paper$address)),
 		       sentence(extraInfo(paper))))
 	}
 
 	formatPhdthesis <- function(paper) {
-	    collapse(c(sentence(authorList(paper), fmtYear(paper$year), sep=" "),
+	    collapse(c(fmtPrefix(paper),
+	    	       sentence(authorList(paper), fmtYear(paper$year), sep = " "),
 		       sentence(fmtBtitle(paper$title)),
 		       sentence("PhD thesis", fmtSchool(paper$school), fmtAddress(paper$address)),
 		       sentence(extraInfo(paper))))
 	}
 
 	formatMisc <- function(paper) {
-	    collapse(c(sentence(authorList(paper), fmtYear(paper$year), sep=" "),
+	    collapse(c(fmtPrefix(paper),
+	               sentence(authorList(paper), fmtYear(paper$year), sep = " "),
 		       fmtTitle(paper$title),
 		       sentence(fmtHowpublished(paper$howpublished)),
 		       sentence(extraInfo(paper))))
@@ -268,7 +278,8 @@ makeJSS <- function()
 	formatProceedings <- function(book) {
 	    if (is.null(book$editor)) editor <- "Anonymous (ed.)"
 	    else editor <- editorList(book)
-	    collapse(c(sentence(editor, fmtYear(book$year), sep=" "),
+	    collapse(c(fmtPrefix(book), # not paper
+	    	       sentence(editor, fmtYear(book$year), sep = " "),
 		       sentence(fmtBtitle(book$title), bookVolume(book)),
 		       sentence(procOrganization(book)),
 		       sentence(fmtISBN(book$isbn), fmtISSN(book$issn),
@@ -276,7 +287,8 @@ makeJSS <- function()
 	}
 
 	formatTechreport <- function(paper) {
-	    collapse(c(sentence(authorList(paper), fmtYear(paper$year), sep=" "),
+	    collapse(c(fmtPrefix(paper),
+	    	       sentence(authorList(paper), fmtYear(paper$year), sep = " "),
 		       fmtTitle(paper$title),
 		       sentence(fmtTechreportnumber(paper$number),
 				fmtInstitution(paper$institution),
@@ -285,7 +297,8 @@ makeJSS <- function()
 	}
 
 	formatUnpublished <- function(paper) {
-	    collapse(c(sentence(authorList(paper), fmtYear(paper$year), sep=" "),
+	    collapse(c(fmtPrefix(paper),
+	    	       sentence(authorList(paper), fmtYear(paper$year), sep = " "),
 		       fmtTitle(paper$title),
 		       sentence(extraInfo(paper))))
 	}
@@ -303,16 +316,26 @@ makeJSS <- function()
 	    result
 	}
 
+	# Replace this if you want a bibliography style
+	# that puts a prefix on each entry, e.g. [n]
+	# The formatting routine will have added a field .index
+	# as a 1-based index within the complete list.
+
+	fmtPrefix <- function(paper) NULL
+
+	cite <- function(key, bib, ...)
+	    utils::citeNatbib(key, bib, ...) # the defaults are JSS style
+
 	environment()
     })
 
 bibstyle <- local({
     styles <- list(JSS = makeJSS())
     default <- "JSS"
-    function(style, envir, ..., .init = FALSE, .default=FALSE) {
+    function(style, envir, ..., .init = FALSE, .default=TRUE) {
         newfns <- list(...)
         if (missing(style) || is.null(style)) {
-            if (!missing(envir) || length(newfns) || .init || .default)
+            if (!missing(envir) || length(newfns) || .init)
             	stop("Changes require specified 'style'")
             style <- default
         } else {
@@ -332,12 +355,18 @@ bibstyle <- local({
     }
 })
 
+getBibstyle <- function(all = FALSE) {
+    if (all)
+    	names(environment(bibstyle)$styles)
+    else
+    	environment(bibstyle)$default
+}
+
 toRd.bibentry <- function(obj, style=NULL, ...) {
-    env <- new.env(hash = FALSE, parent = bibstyle(style)) # small
-    env$obj <- obj
-    o <- with(env, order(sortKeys(obj)))
-    rm("obj", envir=env)
-    bib <- unclass(obj[o])
+    obj <- sort(obj, .bibstyle=style)
+    style <- bibstyle(style)
+    env <- new.env(hash = FALSE, parent = style)
+    bib <- unclass(obj)
     result <- character(length(bib))
     for (i in seq_along(bib)) {
     	env$paper <- bib[[i]]
@@ -359,4 +388,3 @@ toRd.bibentry <- function(obj, style=NULL, ...) {
     }
     result
 }
-
