@@ -1,6 +1,8 @@
 #  File src/library/tools/R/dynamicHelp.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2013 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -28,16 +30,15 @@ httpd <- function(path, query, ...)
     .HTMLdirListing <- function(dir, base, up)
     {
         files <- list.files(dir)    # note, no hidden files are listed
-        out <- HTMLheader(paste("Listing of directory<br>", dir, sep=""),
+        out <- HTMLheader(paste0("Listing of directory<br>", dir),
         		  headerTitle = paste("R:", dir), logo=FALSE,
         		  up = up)
         if(!length(files))
             out <- c(out, gettext("No files in this directory"))
         else {
-            urls <- paste('<a href="', base, '/', files, '">', files, '</a>',
-                          sep = "")
+            urls <- paste0('<a href="', base, '/', files, '">', files, '</a>')
             out <- c(out, "<dl>",
-                     paste("<dd>", mono(iconv(urls, "", "UTF-8")), "</dd>", sep = ""),
+                     paste0("<dd>", mono(iconv(urls, "", "UTF-8")), "</dd>"),
                      "</dl>")
         }
         out <- c(out, "<hr>\n</body></html>")
@@ -47,14 +48,14 @@ httpd <- function(path, query, ...)
     .HTMLusermanuals <- function()
     {
         pkgs <- unlist(.get_standard_package_names())
-    
+
         out <- HTMLheader("R User Manuals")
         for (pkg in pkgs) {
             filename <- system.file(file.path("Meta", "vignette.rds"),
             			    package=pkg)
      	    if (file.exists(filename)) {
      	    	vignettes <- readRDS(filename)
-         	out <- c(out, paste('<h2>Manuals in package', sQuote(pkg),'</h2>', sep=""),
+         	out <- c(out, paste0('<h2>Manuals in package', sQuote(pkg),'</h2>'),
          		 makeVignetteTable(cbind(Package=pkg, as.matrix(vignettes[,c("File", "Title", "PDF", "R")]))))
             }
      	}
@@ -70,7 +71,7 @@ httpd <- function(path, query, ...)
         else {
             fields = c("alias", "concept", "title")
             args <- list(pattern = ".")
-            for (i in seq_along(query)) 
+            for (i in seq_along(query))
             	switch(names(query)[i],
             		pattern = args$pattern <- query[i],
             		title = if (!bool(query[i])) fields <- setdiff(fields, "title"),
@@ -78,7 +79,7 @@ httpd <- function(path, query, ...)
             		alias = if (!bool(query[i])) fields <- setdiff(fields, "alias"),
             		concept = if (!bool(query[i])) fields <- setdiff(fields, "concept"),
             		name = if (bool(query[i])) fields <- union(fields, "name"),
-            		agrep = { 
+            		agrep = {
             		    args$agrep <- as.logical(query[i])
             		    if (is.na(args$agrep))
             		    	args$agrep <- as.numeric(query[i])
@@ -89,7 +90,9 @@ httpd <- function(path, query, ...)
             		types = args$types <- strsplit(query[i], ";")[[1L]],
             		package = args$package <- strsplit(query[i], ";")[[1L]],
             		lib.loc = args$lib.loc <- strsplit(query[i], ";")[[1L]],
-            		warning("Unrecognized search field: ", names(query)[i]))
+            		warning("Unrecognized search field: ", names(query)[i],
+                                domain = NA)
+                       )
             args$fields <- fields
             args$use_UTF8 <- TRUE
             do.call(help.search, args)
@@ -98,20 +101,20 @@ httpd <- function(path, query, ...)
         res <- res$matches
         title <- "Search Results"
         out <- c(HTMLheader(title),
-                 if ("pattern" %in% names(query)) 
-                     paste('The search string was <b>"', query["pattern"], '"</b>',sep=""),
+                 if ("pattern" %in% names(query))
+                     paste0('The search string was <b>"', query["pattern"], '"</b>'),
                  '<hr>\n')
-                       
-        if(!NROW(res)) 
+
+        if(!NROW(res))
             out <- c(out, gettext("No results found"))
         else {
             vigfile0 <- ""
             vigDB <- NULL
             for (type in types) {
-		if(NROW(temp <- res[res[,"Type"] == type,,drop=FALSE]) > 0) 
+		if(NROW(temp <- res[res[,"Type"] == type,,drop=FALSE]) > 0)
 		    switch(type,
 		    vignette = {
-			out <- c(out, paste("<h3>", gettext("Vignettes:"), "</h3>", sep=""), "<dl>")
+			out <- c(out, paste0("<h3>", gettext("Vignettes:"), "</h3>"), "<dl>")
 			n <- NROW(temp)
 			vignettes <- matrix("", n, 5)
 			colnames(vignettes) <- c("Package", "File",
@@ -124,19 +127,19 @@ httpd <- function(path, query, ...)
 			    	vigDB <- readRDS(vigfile)
 			    	vigfile0 <- vigfile
 			    }
-			    vignette <- vigDB[topic == file_path_sans_ext(vigDB$File),]
-			    # There should be exactly one row in the result, but 
+			    vignette <- vigDB[topic == file_path_sans_ext(vigDB$PDF),]
+			    # There should be exactly one row in the result, but
 			    # bad packages might have more, e.g. vig.Snw and vig.Rnw
 			    vignettes[i,] <- c(pkg, unlist(vignette[1,c("File", "Title", "PDF", "R")]))
 			 }
 			 out <- c(out, makeVignetteTable(vignettes))
 		    },
 		    demo = {
-			out <- c(out, paste("<h3>", gettext("Code demonstrations:"), "</h3>", sep=""))
+			out <- c(out, paste0("<h3>", gettext("Code demonstrations:"), "</h3>"))
 			out <- c(out, makeDemoTable(temp))
 		    },
 		    help = {
-			out <- c(out, paste("<h3>", gettext("Help pages:"), "</h3>", sep=""))
+			out <- c(out, paste0("<h3>", gettext("Help pages:"), "</h3>"))
 			out <- c(out, makeHelpTable(temp))
 		    })
 	    }
@@ -176,22 +179,19 @@ httpd <- function(path, query, ...)
                "pdf" = "application/pdf",
                "eps" =,
                "ps" = "application/postscript", # in GLMMGibbs, mclust
-               "sgml"= "text/sgml", # in RGtk2
+               "sgml" = "text/sgml", # in RGtk2
                "xml" = "text/xml",  # in RCurl
                "text/plain")
     }
 
     sQuote <- function(text)
-        paste("&lsquo;", text, "&rsquo;", sep="")
+        paste0("&lsquo;", text, "&rsquo;")
     mono <- function(text)
-        paste('<span class="samp">', text, "</span>", sep="")
+        paste0('<span class="samp">', text, "</span>")
 
     error_page <- function(msg)
         list(payload =
-             paste(HTMLheader("httpd error"),
-                   msg,
-                   "\n</body></html>",
-                   sep = ""))
+             paste0(HTMLheader("httpd error"), msg, "\n</body></html>"))
 
     cssRegexp <- "^/library/([^/]*)/html/R.css$"
     if (grepl("R\\.css$", path) && !grepl(cssRegexp, path))
@@ -239,17 +239,17 @@ httpd <- function(path, query, ...)
 	    return(list(payload = error_page(msg)))
 	} else if (length(file) == 1L) {
 	    path <- dirname(dirname(file))
-	    file <- paste('../../', basename(path), '/html/',
-                          basename(file), '.html', sep='')
+	    file <- paste0('../../', basename(path), '/html/',
+                           basename(file), '.html')
             ## cat("redirect to", file, "\n")
             ## We need to do this because there are static HTML pages
             ## with links to "<file>.html" for topics in the same
             ## package, and if we served one of such a page as a link from
             ## a different package those links on the page would not work.
-	    return(list(payload = paste('Redirect to <a href="', file, '">"',
-                        basename(file), '"</a>', sep=''),
+	    return(list(payload = paste0('Redirect to <a href="', file, '">"',
+                                         basename(file), '"</a>'),
 	    		"content-type" = 'text/html',
-	    		header = paste('Location: ', file, sep=''),
+	    		header = paste0('Location: ', file),
 	    		"status code" = 302L)) # temporary redirect
 	} else if (length(file) > 1L) {
             paths <- dirname(dirname(file))
@@ -260,7 +260,7 @@ httpd <- function(path, query, ...)
                 tmp <- try(readRDS(fp[i]))
                 titles[i] <- if(inherits(tmp, "try-error"))
                     "unknown title" else
-                    tmp[file_path_sans_ext(tmp$File) == tp[i], "Title"]
+                    tmp[file_path_sans_ext(tmp$PDF) == tp[i], "Title"]
             }
             packages <- paste('<dt><a href="../../', basename(paths), '/html/',
                               basename(file), '.html">', titles,
@@ -268,13 +268,17 @@ httpd <- function(path, query, ...)
                               basename(paths),
                               '/html/00Index.html">', basename(paths),
                               '</a> in library ', dirname(paths), ")</dd>",
-                              sep="", collapse="\n")
+                              sep = "", collapse = "\n")
 
             return(list(payload =
                         paste("<p>",
-                              gettextf("Help on topic '%s' was found in the following packages:", topic),
+                              ## for languages with multiple plurals ....
+                              sprintf(ngettext(length(paths),
+                                               "Help on topic '%s' was found in the following package:",
+                                               "Help on topic '%s' was found in the following packages:"
+                                               ), topic),
                               "</p><dl>\n",
-                              packages, "</dl>", sep="", collapse="\n")
+                              packages, "</dl>", sep = "", collapse = "\n")
                         ))
         }
     } else if (grepl(fileRegexp, path)) {
@@ -323,16 +327,16 @@ httpd <- function(path, query, ...)
                               try.all.packages = TRUE)
                 if (length(files)) {
                     path <- dirname(dirname(files))
-                    files <- paste('/library/', basename(path), '/html/',
-                                   basename(files), '.html', sep='')
+                    files <- paste0('/library/', basename(path), '/html/',
+                                    basename(files), '.html')
                     msg <- c(msg, "<br>",
                              "However, you might be looking for one of",
                              "<p></p>",
-                             paste('<p><a href="', files, '">',
-                                   mono(files), "</a></p>", sep="")
+                             paste0('<p><a href="', files, '">',
+                                    mono(files), "</a></p>")
                              )
                 }
-                return(error_page(paste(msg, collapse ="\n")))
+                return(error_page(paste(msg, collapse = "\n")))
             }
             helpdoc <- tmp
         }
@@ -352,17 +356,16 @@ httpd <- function(path, query, ...)
     	pkg <- sub(docRegexp, "\\1", path)
     	rest <- sub(docRegexp, "\\2", path)
         docdir <- system.file("doc", package = pkg)
-        up <- paste("/library/", pkg, "/html/00Index.html", sep = "")
+        up <- paste0("/library/", pkg, "/html/00Index.html")
         if(!nzchar(docdir))
             return(error_page(gettextf("No docs found for package %s",
                                        mono(pkg))))
         if(nzchar(rest) && rest != "/") {
             ## FIXME should we check existence here?
-            file <- paste(docdir, rest, sep = "")
+            file <- paste0(docdir, rest)
             if(isTRUE(file.info(file)$isdir))
                 return(.HTMLdirListing(file,
-                                       paste("/library/", pkg, "/doc", rest,
-                                             sep = ""),
+                                       paste0("/library/", pkg, "/doc", rest),
                                        up))
             else
                 return(list(file = file, "content-type" = mime_type(rest)))
@@ -374,30 +377,30 @@ httpd <- function(path, query, ...)
         }
     } else if (grepl(demoRegexp, path)) {
     	pkg <- sub(demoRegexp, "\\1", path)
-    	
-    	url <- paste("http://127.0.0.1:", httpdPort,
-                      "/doc/html/Search?package=", 
-                      pkg, "&agrep=FALSE&types=demo", sep="")
-    	return(list(payload = paste('Redirect to <a href="', url, 
-    				'">help.search()</a>', sep=''),
+
+    	url <- paste0("http://127.0.0.1:", httpdPort,
+                      "/doc/html/Search?package=",
+                      pkg, "&agrep=FALSE&types=demo")
+    	return(list(payload = paste0('Redirect to <a href="', url,
+    				'">help.search()</a>'),
 		    		"content-type" = 'text/html',
-		    		header = paste('Location: ', url, sep=''),
+		    		header = paste0('Location: ', url),
 	    		"status code" = 302L)) # temporary redirect
     } else if (grepl(demosRegexp, path)) {
 	    pkg <- sub(demosRegexp, "\\1", path)
 	    demo <- sub(demosRegexp, "\\2", path)
-	    file <- system.file(file.path("demo", demo), package=pkg) 
+	    file <- system.file(file.path("demo", demo), package=pkg)
 	    return(list(file = file, "content-type" = mime_type(demo)))
 
     } else if (grepl(DemoRegexp, path)) {
     	pkg <- sub(DemoRegexp, "\\1", path)
     	demo <- sub(DemoRegexp, "\\2", path)
     	demo(demo, package=pkg, character.only=TRUE, ask=FALSE)
-	return( list(payload = paste("Demo '", pkg, "::", demo, 
+	return( list(payload = paste0("Demo '", pkg, "::", demo,
 				"' was run in the console.",
 				" To repeat, type 'demo(",
-				pkg, "::", demo, 
-				")' in the console.", sep="")) )
+				pkg, "::", demo,
+				")' in the console.")) )
     } else if (grepl(newsRegexp, path)) {
     	pkg <- sub(newsRegexp, "\\1", path)
     	formatted <- toHTML(news(package = pkg),
@@ -496,7 +499,7 @@ startDynamicHelp <- function(start=TRUE)
         for(i in seq_along(ports)) {
             ## the next can throw an R-level error,
             ## so do not assign port unless it succeeds.
-	    status <- .Internal(startHTTPD("127.0.0.1", ports[i]))
+	    status <- .Call(startHTTPD, "127.0.0.1", ports[i])
 	    if (status == 0L) {
                 OK <- TRUE
                 httpdPort <<- ports[i]
@@ -516,7 +519,7 @@ startDynamicHelp <- function(start=TRUE)
         }
     } else {
         ## Not really tested
-        .Internal(stopHTTPD())
+        .Call(stopHTTPD)
     	httpdPort <<- 0L
     }
     lockBinding("httpdPort", env)
