@@ -1,6 +1,8 @@
 #  File src/library/utils/R/objects.R
 #  Part of the R package, http://www.R-project.org
 #
+#  Copyright (C) 1995-2012 The R Core Team
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -15,8 +17,9 @@
 #  http://www.r-project.org/Licenses/
 
 ## findGeneric(fname) :  is 'fname' the name of an S3 generic ?
-##			[internal function used only in this file]
-findGeneric <- function(fname, envir)
+##			[unexported function used only in this file]
+findGeneric <-
+function(fname, envir)
 {
     if(!exists(fname, mode = "function", envir = envir)) return("")
     f <- get(fname, mode = "function", envir = envir)
@@ -62,10 +65,12 @@ findGeneric <- function(fname, envir)
     isUME(body(f))
 }
 
-getKnownS3generics <- function()
+getKnownS3generics <-
+function()
     c(names(.knownS3Generics), tools:::.get_internal_S3_generics())
 
-methods <- function (generic.function, class)
+methods <-
+function(generic.function, class)
 {
     rbindSome <- function(df, nms, msg) {
         ## rbind.data.frame() -- dropping rows with duplicated names
@@ -189,20 +194,22 @@ methods <- function (generic.function, class)
     res
 }
 
-print.MethodsFunction <- function(x, ...)
+print.MethodsFunction <-
+function(x, ...)
 {
     visible <- attr(x, "info")[["visible"]]
     if(length(x)) {
 	print(paste0(x, ifelse(visible, "", "*")), quote=FALSE, ...)
         if(any(!visible))
             cat("\n", "   ",
-                "Non-visible functions are asterisked", "\n", sep="")
+                "Non-visible functions are asterisked", "\n", sep = "")
     } else cat("no methods were found\n")
     invisible(x)
 }
 
 
-getS3method <-  function(f, class, optional = FALSE)
+getS3method <-
+function(f, class, optional = FALSE)
 {
     if(!any(f == getKnownS3generics())) {
         truegf <- findGeneric(f, parent.frame())
@@ -233,7 +240,8 @@ getS3method <-  function(f, class, optional = FALSE)
                                 domain = NA)
 }
 
-getFromNamespace <- function(x, ns, pos = -1, envir = as.environment(pos))
+getFromNamespace <-
+function(x, ns, pos = -1, envir = as.environment(pos))
 {
     if(missing(ns)) {
         nm <- attr(envir, "name", exact = TRUE)
@@ -244,9 +252,16 @@ getFromNamespace <- function(x, ns, pos = -1, envir = as.environment(pos))
     get(x, envir = ns, inherits = FALSE)
 }
 
-assignInMyNamespace <- function(x, value)
+assignInMyNamespace <-
+function(x, value)
 {
-    ns <- environment(sys.function(-1))
+    f <- sys.function(-1)
+    ns <- environment(f)
+    ## deal with subclasses of "function"
+    ## that may insert an environment in front of the namespace
+    if(isS4(f))
+        while(!isNamespace(ns))
+            ns <- parent.env(ns)
     if(bindingIsLocked(x, ns)) {
         unlockBinding(x, ns)
         assign(x, value, envir = ns, inherits = FALSE)
@@ -277,7 +292,7 @@ assignInMyNamespace <- function(x, value)
 }
 
 assignInNamespace <-
-    function(x, value, ns, pos = -1, envir = as.environment(pos))
+function(x, value, ns, pos = -1, envir = as.environment(pos))
 {
     nf <- sys.nframe()
     if(missing(ns)) {
@@ -338,7 +353,8 @@ assignInNamespace <-
     invisible(NULL)
 }
 
-fixInNamespace <- function (x, ns, pos = -1, envir = as.environment(pos), ...)
+fixInNamespace <-
+function(x, ns, pos = -1, envir = as.environment(pos), ...)
 {
     subx <- substitute(x)
     if (is.name(subx))
@@ -355,9 +371,11 @@ fixInNamespace <- function (x, ns, pos = -1, envir = as.environment(pos), ...)
     assignInNamespace(subx, x, ns)
 }
 
-getAnywhere <- function(x)
+getAnywhere <-
+function(x)
 {
-    x <- as.character(substitute(x))
+    if(tryCatch(!is.character(x), error = function(e) TRUE))
+        x <- as.character(substitute(x))
     objs <- list(); where <- character(); visible <- logical()
     ## first look on search path
     if(length(pos <- find(x, numeric = TRUE))) {
@@ -403,14 +421,11 @@ getAnywhere <- function(x)
     # now check for duplicates
     ln <- length(objs)
     dups <- rep.int(FALSE, ln)
-    objs2 <- lapply(objs, function(x) {
-        if(is.function(x)) environment(x) <- baseenv()
-        x
-    })
     if(ln > 1L)
         for(i in 2L:ln)
             for(j in 1L:(i-1L))
-                if(identical(objs2[[i]], objs2[[j]])) {
+                if(identical(objs[[i]], objs[[j]],
+                             ignore.environment = TRUE)) {
                     dups[i] <- TRUE
                     break
                 }
@@ -419,7 +434,8 @@ getAnywhere <- function(x)
     res
 }
 
-print.getAnywhere <- function(x, ...)
+print.getAnywhere <-
+function(x, ...)
 {
     n <- sum(!x$dups)
     if(n == 0L) {
@@ -440,20 +456,24 @@ print.getAnywhere <- function(x, ...)
     invisible(x)
 }
 
-`[.getAnywhere` <- function(x, i)
+`[.getAnywhere` <-
+function(x, i)
 {
     if(!is.numeric(i)) stop("only numeric indices can be used")
     if(length(i) == 1L) x$objs[[i]]
     else x$objs[i]
 }
 
-argsAnywhere <- function(x) {
-        name<-as.character(substitute(x))
-        fs<-do.call(getAnywhere, list(name))
-        if (sum(!fs$dups)==0)
-            return(NULL)
-        if (sum(!fs$dups)>1)
-            sapply(fs$objs[!fs$dups],
-                   function(f) if (is.function(f)) args(f))
-        else args(fs$objs[[1L]])
+argsAnywhere <-
+function(x)
+{
+    if(tryCatch(!is.character(x), error = function(e) TRUE))
+        x <- as.character(substitute(x))
+    fs <- getAnywhere(x)
+    if (sum(!fs$dups) == 0L)
+        return(NULL)
+    if (sum(!fs$dups) > 1L)
+        sapply(fs$objs[!fs$dups],
+               function(f) if (is.function(f)) args(f))
+    else args(fs$objs[[1L]])
 }
