@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995,1996  Robert Gentleman, Ross Ihaka
- *  Copyright (C) 1997-2011  The R Core Team
+ *  Copyright (C) 1997-2012  The R Core Team
  *  Copyright (C) 2003-2009 The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,11 @@
 #include <config.h>
 #endif
 
+/* interval at which to check interrupts */
+#define NINTERRUPT 10000000
+
 #include <Defn.h> /*-- Maybe modularize into own Coerce.h ..*/
+#include <Internal.h>
 #include <float.h> /* for DBL_DIG */
 #define R_MSG_mode	_("invalid 'mode' argument")
 #define R_MSG_list_vec	_("applies only to lists and vectors")
@@ -126,7 +130,7 @@ IntegerFromReal(double x, int *warn)
 	*warn |= WARN_NA;
 	return NA_INTEGER;
     }
-    return x;
+    return (int) x;
 }
 
 int attribute_hidden
@@ -140,7 +144,7 @@ IntegerFromComplex(Rcomplex x, int *warn)
     }
     if (x.i != 0)
 	*warn |= WARN_IMAG;
-    return x.r;
+    return (int) x.r;
 }
 
 
@@ -161,7 +165,7 @@ IntegerFromString(SEXP x, int *warn)
 		return INT_MIN;
 	    }
 	    else
-		return xdouble;
+		return (int) xdouble;
 	}
 	else *warn |= WARN_NA;
     }
@@ -385,7 +389,7 @@ SEXP VectorToPairList(SEXP x)
     int i, len, named;
     len = length(x);
     PROTECT(x);
-    PROTECT(xnew = allocList(len));
+    PROTECT(xnew = allocList(len)); /* limited to int */
     PROTECT(xnames = getAttrib(x, R_NamesSymbol));
     named = (xnames != R_NilValue);
     xptr = xnew;
@@ -395,7 +399,7 @@ SEXP VectorToPairList(SEXP x)
 	    SET_TAG(xptr, install(translateChar(STRING_ELT(xnames, i))));
 	xptr = CDR(xptr);
     }
-    if (len>0)       /* can't set attributes on NULL */
+    if (len > 0)       /* can't set attributes on NULL */
 	copyMostAttrib(x, xnew);
     UNPROTECT(3);
     return xnew;
@@ -440,8 +444,9 @@ static SEXP coerceToSymbol(SEXP v)
 static SEXP coerceToLogical(SEXP v)
 {
     SEXP ans;
-    int i, n, warn = 0;
-    PROTECT(ans = allocVector(LGLSXP, n = length(v)));
+    int warn = 0;
+    R_xlen_t i, n;
+    PROTECT(ans = allocVector(LGLSXP, n = XLENGTH(v)));
 #ifdef R_MEMORY_PROFILING
     if (RTRACE(v)){
        memtrace_report(v,ans);
@@ -451,24 +456,34 @@ static SEXP coerceToLogical(SEXP v)
     DUPLICATE_ATTRIB(ans, v);
     switch (TYPEOF(v)) {
     case INTSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    LOGICAL(ans)[i] = LogicalFromInteger(INTEGER(v)[i], &warn);
+	}
 	break;
     case REALSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    LOGICAL(ans)[i] = LogicalFromReal(REAL(v)[i], &warn);
-	    break;
+	}
+	break;
     case CPLXSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    LOGICAL(ans)[i] = LogicalFromComplex(COMPLEX(v)[i], &warn);
+	}
 	break;
     case STRSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    LOGICAL(ans)[i] = LogicalFromString(STRING_ELT(v, i), &warn);
+	}
 	break;
     case RAWSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    LOGICAL(ans)[i] = LogicalFromInteger((int)RAW(v)[i], &warn);
+	}
 	break;
     default:
 	UNIMPLEMENTED_TYPE("coerceToLogical", v);
@@ -481,8 +496,9 @@ static SEXP coerceToLogical(SEXP v)
 static SEXP coerceToInteger(SEXP v)
 {
     SEXP ans;
-    int i, n, warn = 0;
-    PROTECT(ans = allocVector(INTSXP, n = LENGTH(v)));
+    int warn = 0;
+    R_xlen_t i, n;
+    PROTECT(ans = allocVector(INTSXP, n = XLENGTH(v)));
 #ifdef R_MEMORY_PROFILING
     if (RTRACE(v)){
        memtrace_report(v,ans);
@@ -492,25 +508,35 @@ static SEXP coerceToInteger(SEXP v)
     DUPLICATE_ATTRIB(ans, v);
     switch (TYPEOF(v)) {
     case LGLSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    INTEGER(ans)[i] = IntegerFromLogical(LOGICAL(v)[i], &warn);
-	    break;
+	}
+	break;
     case REALSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    INTEGER(ans)[i] = IntegerFromReal(REAL(v)[i], &warn);
+	}
 	break;
     case CPLXSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    INTEGER(ans)[i] = IntegerFromComplex(COMPLEX(v)[i], &warn);
+	}
 	break;
     case STRSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    INTEGER(ans)[i] = IntegerFromString(STRING_ELT(v, i), &warn);
+	}
 	break;
     case RAWSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    INTEGER(ans)[i] = (int)RAW(v)[i];
-	    break;
+	}
+	break;
     default:
 	UNIMPLEMENTED_TYPE("coerceToInteger", v);
     }
@@ -522,8 +548,9 @@ static SEXP coerceToInteger(SEXP v)
 static SEXP coerceToReal(SEXP v)
 {
     SEXP ans;
-    int i, n, warn = 0;
-    PROTECT(ans = allocVector(REALSXP, n = LENGTH(v)));
+    int warn = 0;
+    R_xlen_t i, n;
+    PROTECT(ans = allocVector(REALSXP, n = XLENGTH(v)));
 #ifdef R_MEMORY_PROFILING
     if (RTRACE(v)){
        memtrace_report(v,ans);
@@ -533,24 +560,34 @@ static SEXP coerceToReal(SEXP v)
     DUPLICATE_ATTRIB(ans, v);
     switch (TYPEOF(v)) {
     case LGLSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    REAL(ans)[i] = RealFromLogical(LOGICAL(v)[i], &warn);
+	}
 	break;
     case INTSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    REAL(ans)[i] = RealFromInteger(INTEGER(v)[i], &warn);
+	}
 	break;
     case CPLXSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    REAL(ans)[i] = RealFromComplex(COMPLEX(v)[i], &warn);
+	}
 	break;
     case STRSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    REAL(ans)[i] = RealFromString(STRING_ELT(v, i), &warn);
+	}
 	break;
     case RAWSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    REAL(ans)[i] = RealFromInteger((int)RAW(v)[i], &warn);
+	}
 	break;
     default:
 	UNIMPLEMENTED_TYPE("coerceToReal", v);
@@ -563,8 +600,9 @@ static SEXP coerceToReal(SEXP v)
 static SEXP coerceToComplex(SEXP v)
 {
     SEXP ans;
-    int i, n, warn = 0;
-    PROTECT(ans = allocVector(CPLXSXP, n = LENGTH(v)));
+    int warn = 0;
+    R_xlen_t i, n;
+    PROTECT(ans = allocVector(CPLXSXP, n = XLENGTH(v)));
 #ifdef R_MEMORY_PROFILING
     if (RTRACE(v)){
        memtrace_report(v,ans);
@@ -574,24 +612,34 @@ static SEXP coerceToComplex(SEXP v)
     DUPLICATE_ATTRIB(ans, v);
     switch (TYPEOF(v)) {
     case LGLSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    COMPLEX(ans)[i] = ComplexFromLogical(LOGICAL(v)[i], &warn);
+	}
 	break;
     case INTSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    COMPLEX(ans)[i] = ComplexFromInteger(INTEGER(v)[i], &warn);
+	}
 	break;
     case REALSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    COMPLEX(ans)[i] = ComplexFromReal(REAL(v)[i], &warn);
+	}
 	break;
     case STRSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    COMPLEX(ans)[i] = ComplexFromString(STRING_ELT(v, i), &warn);
+	}
 	break;
     case RAWSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    COMPLEX(ans)[i] = ComplexFromInteger((int)RAW(v)[i], &warn);
+	}
 	break;
     default:
 	UNIMPLEMENTED_TYPE("coerceToComplex", v);
@@ -604,9 +652,10 @@ static SEXP coerceToComplex(SEXP v)
 static SEXP coerceToRaw(SEXP v)
 {
     SEXP ans;
-    int i, n, warn = 0, tmp;
+    int warn = 0, tmp;
+    R_xlen_t i, n;
 
-    PROTECT(ans = allocVector(RAWSXP, n = LENGTH(v)));
+    PROTECT(ans = allocVector(RAWSXP, n = XLENGTH(v)));
 #ifdef R_MEMORY_PROFILING
     if (RTRACE(v)){
        memtrace_report(v,ans);
@@ -617,6 +666,7 @@ static SEXP coerceToRaw(SEXP v)
     switch (TYPEOF(v)) {
     case LGLSXP:
 	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    tmp = IntegerFromLogical(LOGICAL(v)[i], &warn);
 	    if(tmp == NA_INTEGER) {
 		tmp = 0;
@@ -627,6 +677,7 @@ static SEXP coerceToRaw(SEXP v)
 	break;
     case INTSXP:
 	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    tmp = INTEGER(v)[i];
 	    if(tmp == NA_INTEGER || tmp < 0 || tmp > 255) {
 		tmp = 0;
@@ -637,6 +688,7 @@ static SEXP coerceToRaw(SEXP v)
 	break;
     case REALSXP:
 	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    tmp = IntegerFromReal(REAL(v)[i], &warn);
 	    if(tmp == NA_INTEGER || tmp < 0 || tmp > 255) {
 		tmp = 0;
@@ -647,6 +699,7 @@ static SEXP coerceToRaw(SEXP v)
 	break;
     case CPLXSXP:
 	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    tmp = IntegerFromComplex(COMPLEX(v)[i], &warn);
 	    if(tmp == NA_INTEGER || tmp < 0 || tmp > 255) {
 		tmp = 0;
@@ -657,6 +710,7 @@ static SEXP coerceToRaw(SEXP v)
 	break;
     case STRSXP:
 	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    tmp = IntegerFromString(STRING_ELT(v, i), &warn);
 	    if(tmp == NA_INTEGER || tmp < 0 || tmp > 255) {
 		tmp = 0;
@@ -676,8 +730,10 @@ static SEXP coerceToRaw(SEXP v)
 static SEXP coerceToString(SEXP v)
 {
     SEXP ans;
-    int i, n, savedigits, warn = 0;
-    PROTECT(ans = allocVector(STRSXP, n = LENGTH(v)));
+    int savedigits, warn = 0;
+    R_xlen_t i, n;
+
+    PROTECT(ans = allocVector(STRSXP, n = XLENGTH(v)));
 #ifdef R_MEMORY_PROFILING
     if (RTRACE(v)){
        memtrace_report(v,ans);
@@ -687,30 +743,40 @@ static SEXP coerceToString(SEXP v)
     DUPLICATE_ATTRIB(ans, v);
     switch (TYPEOF(v)) {
     case LGLSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    SET_STRING_ELT(ans, i, StringFromLogical(LOGICAL(v)[i], &warn));
+	}
 	break;
     case INTSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    SET_STRING_ELT(ans, i, StringFromInteger(INTEGER(v)[i], &warn));
+	}
 	break;
     case REALSXP:
 	PrintDefaults();
 	savedigits = R_print.digits; R_print.digits = DBL_DIG;/* MAX precision */
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    SET_STRING_ELT(ans, i, StringFromReal(REAL(v)[i], &warn));
+	}
 	R_print.digits = savedigits;
 	break;
     case CPLXSXP:
 	PrintDefaults();
 	savedigits = R_print.digits; R_print.digits = DBL_DIG;/* MAX precision */
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    SET_STRING_ELT(ans, i, StringFromComplex(COMPLEX(v)[i], &warn));
+	}
 	R_print.digits = savedigits;
 	break;
     case RAWSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    SET_STRING_ELT(ans, i, StringFromRaw(RAW(v)[i], &warn));
+	}
 	break;
     default:
 	UNIMPLEMENTED_TYPE("coerceToString", v);
@@ -723,9 +789,9 @@ static SEXP coerceToString(SEXP v)
 static SEXP coerceToExpression(SEXP v)
 {
     SEXP ans;
-    int i, n;
+    R_xlen_t i, n;
     if (isVectorAtomic(v)) {
-	n = LENGTH(v);
+	n = XLENGTH(v);
 	PROTECT(ans = allocVector(EXPRSXP, n));
 #ifdef R_MEMORY_PROFILING
     if (RTRACE(v)){
@@ -773,8 +839,8 @@ static SEXP coerceToExpression(SEXP v)
 static SEXP coerceToVectorList(SEXP v)
 {
     SEXP ans, tmp;
-    int i, n;
-    n = length(v);
+    R_xlen_t i, n;
+    n = xlength(v);
     PROTECT(ans = allocVector(VECSXP, n));
 #ifdef R_MEMORY_PROFILING
     if (RTRACE(v)){
@@ -784,28 +850,40 @@ static SEXP coerceToVectorList(SEXP v)
 #endif
     switch (TYPEOF(v)) {
     case LGLSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    SET_VECTOR_ELT(ans, i, ScalarLogical(LOGICAL(v)[i]));
+	}
 	break;
     case INTSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    SET_VECTOR_ELT(ans, i, ScalarInteger(INTEGER(v)[i]));
+	}
 	break;
     case REALSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    SET_VECTOR_ELT(ans, i, ScalarReal(REAL(v)[i]));
+	}
 	break;
     case CPLXSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    SET_VECTOR_ELT(ans, i, ScalarComplex(COMPLEX(v)[i]));
+	}
 	break;
     case STRSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    SET_VECTOR_ELT(ans, i, ScalarString(STRING_ELT(v, i)));
+	}
 	break;
     case RAWSXP:
-	for (i = 0; i < n; i++)
+	for (i = 0; i < n; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    SET_VECTOR_ELT(ans, i, ScalarRaw(RAW(v)[i]));
+	}
 	break;
     case LISTSXP:
     case LANGSXP:
@@ -829,7 +907,7 @@ static SEXP coerceToPairList(SEXP v)
 {
     SEXP ans, ansp;
     int i, n;
-    n = LENGTH(v);
+    n = LENGTH(v); /* limited to len */
     PROTECT(ansp = ans = allocList(n));
     for (i = 0; i < n; i++) {
 	switch (TYPEOF(v)) {
@@ -958,7 +1036,8 @@ static SEXP coercePairList(SEXP v, SEXPTYPE type)
 /* Coerce a vector list to the given type */
 static SEXP coerceVectorList(SEXP v, SEXPTYPE type)
 {
-    int i, n, warn = 0, tmp;
+    int warn = 0, tmp;
+    R_xlen_t i, n;
     SEXP rval, names;
 
     names = v;
@@ -979,7 +1058,7 @@ static SEXP coerceVectorList(SEXP v, SEXPTYPE type)
     }
 
     if (type == STRSXP) {
-	n = length(v);
+	n = xlength(v);
 	PROTECT(rval = allocVector(type, n));
 #ifdef R_MEMORY_PROFILING
 	if (RTRACE(v)){
@@ -988,7 +1067,7 @@ static SEXP coerceVectorList(SEXP v, SEXPTYPE type)
 	}
 #endif
 	for (i = 0; i < n;  i++) {
-	    if (isString(VECTOR_ELT(v, i)) && length(VECTOR_ELT(v, i)) == 1)
+	    if (isString(VECTOR_ELT(v, i)) && xlength(VECTOR_ELT(v, i)) == 1)
 		SET_STRING_ELT(rval, i, STRING_ELT(VECTOR_ELT(v, i), 0));
 #if 0
 	    /* this will make as.character(list(s)) not backquote
@@ -1008,27 +1087,36 @@ static SEXP coerceVectorList(SEXP v, SEXPTYPE type)
 	return rval;
     }
     else if (isVectorizable(v)) {
-	n = length(v);
+	n = xlength(v);
 	PROTECT(rval = allocVector(type, n));
 	switch (type) {
 	case LGLSXP:
-	    for (i = 0; i < n; i++)
+	    for (i = 0; i < n; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 		LOGICAL(rval)[i] = asLogical(VECTOR_ELT(v, i));
+	    }
 	    break;
 	case INTSXP:
-	    for (i = 0; i < n; i++)
+	    for (i = 0; i < n; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 		INTEGER(rval)[i] = asInteger(VECTOR_ELT(v, i));
+	    }
 	    break;
 	case REALSXP:
-	    for (i = 0; i < n; i++)
+	    for (i = 0; i < n; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 		REAL(rval)[i] = asReal(VECTOR_ELT(v, i));
+	    }
 	    break;
 	case CPLXSXP:
-	    for (i = 0; i < n; i++)
+	    for (i = 0; i < n; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 		COMPLEX(rval)[i] = asComplex(VECTOR_ELT(v, i));
+	    }
 	    break;
 	case RAWSXP:
 	    for (i = 0; i < n; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 		tmp = asInteger(VECTOR_ELT(v, i));
 		if (tmp < 0 || tmp > 255) { /* includes NA_INTEGER */
 		    tmp = 0;
@@ -1289,7 +1377,7 @@ SEXP asCharacterFactor(SEXP x)
     if( !inherits(x, "factor") )
         error(_("attempting to coerce non-factor"));
 
-    int i, n = LENGTH(x);
+    R_xlen_t i, n = XLENGTH(x);
     SEXP labels = getAttrib(x, install("levels"));
     PROTECT(ans = allocVector(STRSXP, n));
     for(i = 0; i < n; i++) {
@@ -1320,9 +1408,6 @@ SEXP attribute_hidden do_ascharacter(SEXP call, SEXP op, SEXP args, SEXP rho)
 	name = "as.integer"; type = INTSXP; break;
     case 2:
 	name = "as.double"; type = REALSXP; break;
-    case 20: // as.real
-	warningcall(call, "as.real is deprecated: use as.double");
-	name = "as.double"; type = REALSXP; break;
     case 3:
 	name = "as.complex"; type = CPLXSXP; break;
     case 4:
@@ -1531,7 +1616,7 @@ int asLogical(SEXP x)
     int warn = 0;
 
     if (isVectorAtomic(x)) {
-	if (LENGTH(x) < 1)
+	if (XLENGTH(x) < 1)
 	    return NA_LOGICAL;
 	switch (TYPEOF(x)) {
 	case LGLSXP:
@@ -1559,7 +1644,7 @@ int asInteger(SEXP x)
 {
     int warn = 0, res;
 
-    if (isVectorAtomic(x) && LENGTH(x) >= 1) {
+    if (isVectorAtomic(x) && XLENGTH(x) >= 1) {
 	switch (TYPEOF(x)) {
 	case LGLSXP:
 	    return IntegerFromLogical(LOGICAL(x)[0], &warn);
@@ -1593,7 +1678,7 @@ double asReal(SEXP x)
     int warn = 0;
     double res;
 
-    if (isVectorAtomic(x) && LENGTH(x) >= 1) {
+    if (isVectorAtomic(x) && XLENGTH(x) >= 1) {
 	switch (TYPEOF(x)) {
 	case LGLSXP:
 	    res = RealFromLogical(LOGICAL(x)[0], &warn);
@@ -1629,7 +1714,7 @@ Rcomplex asComplex(SEXP x)
     int warn = 0;
     Rcomplex z;
 
-    if (isVectorAtomic(x) && LENGTH(x) >= 1) {
+    if (isVectorAtomic(x) && XLENGTH(x) >= 1) {
 	switch (TYPEOF(x)) {
 	case LGLSXP:
 	    z = ComplexFromLogical(LOGICAL(x)[0], &warn);
@@ -1712,10 +1797,6 @@ SEXP attribute_hidden do_is(SEXP call, SEXP op, SEXP args, SEXP rho)
     case REALSXP:	/* is.double */
 	LOGICAL(ans)[0] = (TYPEOF(CAR(args)) == REALSXP);
 	break;
-    case 67:	/* is.real */
-	warningcall(call, "is.real is deprecated: use is.double");
-	LOGICAL(ans)[0] = (TYPEOF(CAR(args)) == REALSXP);
-	break;
     case CPLXSXP:	/* is.complex */
 	LOGICAL(ans)[0] = (TYPEOF(CAR(args)) == CPLXSXP);
 	break;
@@ -1756,6 +1837,9 @@ SEXP attribute_hidden do_is(SEXP call, SEXP op, SEXP args, SEXP rho)
     case 50:		/* is.object */
 	LOGICAL(ans)[0] = OBJECT(CAR(args));
 	break;
+    case 51:		/* isS4 */
+	LOGICAL(ans)[0] = IS_S4_OBJECT(CAR(args)) != 0;
+	break;
 /* no longer used: is.data.frame is R code
     case 80:
 	LOGICAL(ans)[0] = isFrame(CAR(args));
@@ -1887,7 +1971,7 @@ SEXP attribute_hidden do_isvector(SEXP call, SEXP op, SEXP args, SEXP rho)
 SEXP attribute_hidden do_isna(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans, dims, names, x;
-    int i, n;
+    R_xlen_t i, n;
 
     checkArity(op, args);
     check1arg(args, call, "x");
@@ -1901,7 +1985,7 @@ SEXP attribute_hidden do_isna(SEXP call, SEXP op, SEXP args, SEXP rho)
 
 #endif
     x = CAR(args);
-    n = length(x);
+    n = xlength(x);
     PROTECT(ans = allocVector(LGLSXP, n));
     if (isVector(x)) {
 	PROTECT(dims = getAttrib(x, R_DimSymbol));
@@ -2000,7 +2084,7 @@ SEXP attribute_hidden do_isna(SEXP call, SEXP op, SEXP args, SEXP rho)
 SEXP attribute_hidden do_isnan(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans, dims, names, x;
-    int i, n;
+    R_xlen_t i, n;
 
     checkArity(op, args);
     check1arg(args, call, "x");
@@ -2014,7 +2098,7 @@ SEXP attribute_hidden do_isnan(SEXP call, SEXP op, SEXP args, SEXP rho)
 	errorcall_return(call, "is.nan " R_MSG_list_vec);
 #endif
     x = CAR(args);
-    n = length(x);
+    n = xlength(x);
     PROTECT(ans = allocVector(LGLSXP, n));
     if (isVector(x)) {
 	PROTECT(dims = getAttrib(x, R_DimSymbol));
@@ -2063,7 +2147,7 @@ SEXP attribute_hidden do_isnan(SEXP call, SEXP op, SEXP args, SEXP rho)
 SEXP attribute_hidden do_isfinite(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans, x, names, dims;
-    int i, n;
+    R_xlen_t i, n;
 
     checkArity(op, args);
     check1arg(args, call, "x");
@@ -2075,7 +2159,7 @@ SEXP attribute_hidden do_isfinite(SEXP call, SEXP op, SEXP args, SEXP rho)
 	errorcall_return(call, "is.finite " R_MSG_list_vec);
 #endif
     x = CAR(args);
-    n = length(x);
+    n = xlength(x);
     ans = allocVector(LGLSXP, n);
     if (isVector(x)) {
 	dims = getAttrib(x, R_DimSymbol);
@@ -2123,7 +2207,7 @@ SEXP attribute_hidden do_isinfinite(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans, x, names, dims;
     double xr, xi;
-    int i, n;
+    R_xlen_t i, n;
 
     checkArity(op, args);
     check1arg(args, call, "x");
@@ -2135,7 +2219,7 @@ SEXP attribute_hidden do_isinfinite(SEXP call, SEXP op, SEXP args, SEXP rho)
 	errorcall_return(call, "is.infinite " R_MSG_list_vec);
 #endif
     x = CAR(args);
-    n = length(x);
+    n = xlength(x);
     ans = allocVector(LGLSXP, n);
     if (isVector(x)) {
 	dims = getAttrib(x, R_DimSymbol);
@@ -2200,7 +2284,9 @@ SEXP attribute_hidden do_call(SEXP call, SEXP op, SEXP args, SEXP rho)
      */
     if (!isString(rfun) || length(rfun) != 1)
 	errorcall_return(call, _("first argument must be a character string"));
-    PROTECT(rfun = install(translateChar(STRING_ELT(rfun, 0))));
+    const char *str = translateChar(STRING_ELT(rfun, 0));
+    if (streql(str, ".Internal")) error("illegal usage");
+    PROTECT(rfun = install(str));
     PROTECT(evargs = duplicate(CDR(args)));
     for (rest = evargs; rest != R_NilValue; rest = CDR(rest)) {
         PROTECT(tmp = eval(CAR(rest), rho));
@@ -2243,19 +2329,24 @@ SEXP attribute_hidden do_docall(SEXP call, SEXP op, SEXP args, SEXP rho)
 
     PROTECT(c = call = allocList(n + 1));
     SET_TYPEOF(c, LANGSXP);
-    if( isString(fun) )
-	SETCAR(c, install(translateChar(STRING_ELT(fun, 0))));
-    else
+    if( isString(fun) ) {
+	const char *str = translateChar(STRING_ELT(fun, 0));
+	if (streql(str, ".Internal")) error("illegal usage");
+	SETCAR(c, install(str));
+    } else {
+	if(TYPEOF(fun) == SPECIALSXP && streql(PRIMNAME(fun), ".Internal"))
+	    error("illegal usage");
 	SETCAR(c, fun);
+    }
     c = CDR(c);
     for (i = 0; i < n; i++) {
 #ifndef NEW
 	SETCAR(c, VECTOR_ELT(args, i));
 #else
 	SETCAR(c, mkPROMISE(VECTOR_ELT(args, i), rho));
-	SET_PRVALUE(CAR(c), VECTOR_ELT(args, i)); */
+	SET_PRVALUE(CAR(c), VECTOR_ELT(args, i));
 #endif
-	if (ItemName(names, i) != R_NilValue)
+	if (ItemName(names, (int)i) != R_NilValue)
 	    SET_TAG(c, install(translateChar(ItemName(names, i))));
 	c = CDR(c);
     }
@@ -2297,7 +2388,7 @@ SEXP substitute(SEXP lang, SEXP rho)
 		    return t;
 		}
 		else if (TYPEOF(t) == DOTSXP)
-		    error(_("... used in an incorrect context"));
+		    error(_("'...' used in an incorrect context"));
 		if (rho != R_GlobalEnv)
 		    return t;
 	    }
@@ -2338,7 +2429,7 @@ SEXP attribute_hidden substituteList(SEXP el, SEXP rho)
 	    else if (TYPEOF(h) == DOTSXP)
 		h = substituteList(h, R_NilValue);
 	    else
-		error(_("... used in an incorrect context"));
+		error(_("'...' used in an incorrect context"));
 	} else {
 	    h = substitute(CAR(el), rho);
 	    if (isLanguage(el))
@@ -2497,7 +2588,7 @@ static SEXP R_set_class(SEXP obj, SEXP value, SEXP call)
 	SEXP cur_class; SEXPTYPE valueType;
 	valueString = CHAR(asChar(value)); /* ASCII */
 	whichType = class2type(valueString);
-	valueType = (whichType == -1) ? -1 : classTable[whichType].sexp;
+	valueType = (whichType == -1) ? (SEXPTYPE) -1 : classTable[whichType].sexp;
 	PROTECT(cur_class = R_data_class(obj, FALSE)); nProtect++;
 	/*  assigning type as a class deletes an explicit class attribute. */
 	if(valueType != -1) {
