@@ -45,17 +45,19 @@
 #include <Print.h>
 
 /* this is just for conformity with other types */
-void formatRaw(Rbyte *x, int n, int *fieldwidth)
+attribute_hidden
+void formatRaw(Rbyte *x, R_xlen_t n, int *fieldwidth)
 {
     *fieldwidth = 2;
 }
 
-void formatString(SEXP *x, int n, int *fieldwidth, int quote)
+attribute_hidden
+void formatString(SEXP *x, R_xlen_t n, int *fieldwidth, int quote)
 {
     int xmax = 0;
-    int i, l;
+    int l;
 
-    for (i = 0; i < n; i++) {
+    for (R_xlen_t i = 0; i < n; i++) {
 	if (x[i] == NA_STRING) {
 	    l = quote ? R_print.na_width : R_print.na_width_noquote;
 	} else l = Rstrlen(x[i], quote) + (quote ? 2 : 0);
@@ -64,12 +66,10 @@ void formatString(SEXP *x, int n, int *fieldwidth, int quote)
     *fieldwidth = xmax;
 }
 
-void formatLogical(int *x, int n, int *fieldwidth)
+void formatLogical(int *x, R_xlen_t n, int *fieldwidth)
 {
-    int i;
-
     *fieldwidth = 1;
-    for(i = 0 ; i < n; i++) {
+    for(R_xlen_t i = 0 ; i < n; i++) {
 	if (x[i] == NA_LOGICAL) {
 	    if(*fieldwidth < R_print.na_width)
 		*fieldwidth =  R_print.na_width;
@@ -83,12 +83,12 @@ void formatLogical(int *x, int n, int *fieldwidth)
     }
 }
 
-void formatInteger(int *x, int n, int *fieldwidth)
+void formatInteger(int *x, R_xlen_t n, int *fieldwidth)
 {
     int xmin = INT_MAX, xmax = INT_MIN, naflag = 0;
-    int i, l;
+    int l;
 
-    for (i = 0; i < n; i++) {
+    for (R_xlen_t i = 0; i < n; i++) {
 	if (x[i] == NA_INTEGER)
 	    naflag = 1;
 	else {
@@ -121,8 +121,8 @@ void formatInteger(int *x, int n, int *fieldwidth)
  * Using GLOBAL	 R_print.digits	 -- had	 #define MAXDIG R_print.digits
 */
 
-/* long double is C99, so should always be defined */
-#if SIZEOF_LONG_DOUBLE > SIZEOF_DOUBLE
+/* long double is C99, so should always be defined but may be slow */
+#if defined(HAVE_LONG_DOUBLE) && (SIZEOF_LONG_DOUBLE > SIZEOF_DOUBLE)
 # ifdef HAVE_NEARBYINTL
 # define R_nearbyintl nearbyintl
 /* Cygwin had rintl but not nearbyintl */
@@ -130,9 +130,9 @@ void formatInteger(int *x, int n, int *fieldwidth)
 # define R_nearbyintl rintl
 # else
 # define R_nearbyintl private_nearbyintl
-long double private_nearbyintl(long double x)
+LDOUBLE private_nearbyintl(LDOUBLE x)
 {
-    long double x1;
+    LDOUBLE x1;
     x1 = - floorl(-x + 0.5);
     x = floorl(x + 0.5);
     if (x == x1) return(x);
@@ -146,7 +146,7 @@ long double private_nearbyintl(long double x)
 # else /* no long double */
 # ifdef HAVE_NEARBYINT
 #  define R_nearbyint nearbyint
-# elif defined(HAVE_RINTL)
+# elif defined(HAVE_RINT)
 #  define R_nearbyint rint
 # else
 #  define R_nearbyint private_rint
@@ -204,7 +204,7 @@ static void scientific(double *x, int *sgn, int *kpower, int *nsig)
             return;
         }
         kp = (int) floor(log10(r)) - R_print.digits + 1;/* r = |x|; 10^(kp + digits - 1) <= r */
-#if SIZEOF_LONG_DOUBLE > SIZEOF_DOUBLE
+#if defined(HAVE_LONG_DOUBLE) && (SIZEOF_LONG_DOUBLE > SIZEOF_DOUBLE)
         long double r_prec = r;
         /* use exact scaling factor in long double precision, if possible */
         if (abs(kp) <= 27) {
@@ -276,12 +276,12 @@ static void scientific(double *x, int *sgn, int *kpower, int *nsig)
    it is 0 except when called from do_format.
 */
 
-void formatReal(double *x, int n, int *w, int *d, int *e, int nsmall)
+void formatReal(double *x, R_xlen_t n, int *w, int *d, int *e, int nsmall)
 {
     int left, right, sleft;
     int mnl, mxl, rgt, mxsl, mxns, wF;
     int neg, sgn, kpower, nsig;
-    int i, naflag, nanflag, posinf, neginf;
+    int naflag, nanflag, posinf, neginf;
 
     nanflag = 0;
     naflag = 0;
@@ -291,7 +291,7 @@ void formatReal(double *x, int n, int *w, int *d, int *e, int nsmall)
     rgt = mxl = mxsl = mxns = INT_MIN;
     mnl = INT_MAX;
 
-    for (i = 0; i < n; i++) {
+    for (R_xlen_t i = 0; i < n; i++) {
 	if (!R_FINITE(x[i])) {
 	    if(ISNA(x[i])) naflag = 1;
 	    else if(ISNAN(x[i])) nanflag = 1;
@@ -362,7 +362,7 @@ void formatReal(double *x, int n, int *w, int *d, int *e, int nsmall)
    together, not separately */
 void z_prec_r(Rcomplex *r, Rcomplex *x, double digits);
 
-void formatComplex(Rcomplex *x, int n, int *wr, int *dr, int *er,
+void formatComplex(Rcomplex *x, R_xlen_t n, int *wr, int *dr, int *er,
 		   int *wi, int *di, int *ei, int nsmall)
 {
 /* format.info() or  x[1..l] for both Re & Im */
@@ -370,7 +370,7 @@ void formatComplex(Rcomplex *x, int n, int *wr, int *dr, int *er,
     int rt, mnl, mxl, mxsl, mxns, wF, i_wF;
     int i_rt, i_mnl, i_mxl, i_mxsl, i_mxns;
     int neg, sgn;
-    int i, kpower, nsig;
+    int kpower, nsig;
     int naflag;
     int rnanflag, rposinf, rneginf, inanflag, iposinf;
     Rcomplex tmp;
@@ -388,7 +388,7 @@ void formatComplex(Rcomplex *x, int n, int *wr, int *dr, int *er,
     i_rt= i_mxl= i_mxsl= i_mxns= INT_MIN;
     i_mnl = mnl = INT_MAX;
 
-    for (i = 0; i < n; i++) {
+    for (R_xlen_t i = 0; i < n; i++) {
 	/* Now round */
 	z_prec_r(&tmp, &(x[i]), R_print.digits);
 	if(ISNA(tmp.r) || ISNA(tmp.i)) {
