@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2001-2010  The R Development Core Team
+ *  Copyright (C) 2001-2012  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@ typedef enum {
     single_NA__num_eq  = 3,/* R's default: one kind of NA or NaN; for num, use '==' */
 } ne_strictness_type;
 
-/* NOTE:  ne_strict = num_eq + (single_NA * 2)  = num_eq | (single_NA << 1)   */
+/* NOTE:  ne_strict = NUM_EQ + (SINGLE_NA * 2)  = NUM_EQ | (SINGLE_NA << 1)   */
 
 static Rboolean neWithNaN(double x, double y, ne_strictness_type str);
 
@@ -42,38 +42,48 @@ static Rboolean neWithNaN(double x, double y, ne_strictness_type str);
 /* .Internal(identical(..)) */
 SEXP attribute_hidden do_identical(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    int num_eq, single_NA, attr_as_set, nargs = length(args);
-    /* avoid problems with earlier version captured in S4 methods */
-    /* checkArity(op, args); */
-    if (nargs != 2 && nargs != 5)
+    int num_eq = 1, single_NA = 1, attr_as_set = 1, ignore_bytecode = 1, 
+	ignore_env = 0, nargs = length(args), flags;
+    /* avoid problems with earlier (and future) versions captured in S4
+       methods: but this should be fixed where it is caused, in
+       'methods'!
+
+       checkArity(op, args); */
+    if (nargs < 5)
 	error("%d arguments passed to .Internal(%s) which requires %d",
 	      length(args), PRIMNAME(op), PRIMARITY(op));
 
-    if (nargs == 5) {
-	num_eq      = asLogical(CADDR(args));
-	single_NA   = asLogical(CADDDR(args));
-	attr_as_set = asLogical(CAD4R(args));
+    SEXP x = CAR(args); args = CDR(args);
+    SEXP y = CAR(args); args = CDR(args);
+    num_eq = asLogical(CAR(args)); args = CDR(args);
+    single_NA = asLogical(CAR(args)); args = CDR(args);
+    attr_as_set = asLogical(CAR(args)); args = CDR(args);
+    if (nargs >= 6) 
+	ignore_bytecode = asLogical(CAR(args));
+    if (nargs >= 7) 
+	ignore_env = asLogical(CADR(args));
 
-	if(num_eq      == NA_LOGICAL) error(_("invalid '%s' value"), "num.eq");
-	if(single_NA   == NA_LOGICAL) error(_("invalid '%s' value"), "single.NA");
-	if(attr_as_set == NA_LOGICAL) error(_("invalid '%s' value"), "attrib.as.set");
-    } else {
-	num_eq      = 1;
-	single_NA   = 1;
-	attr_as_set = 1;
-    }
-
-    return ScalarLogical(R_compute_identical(CAR(args), CADR(args),
-					     (Rboolean) num_eq,
-					     (Rboolean) single_NA,
-					     (Rboolean) attr_as_set));
+    if(num_eq == NA_LOGICAL) error(_("invalid '%s' value"), "num.eq");
+    if(single_NA == NA_LOGICAL) error(_("invalid '%s' value"), "single.NA");
+    if(attr_as_set == NA_LOGICAL) error(_("invalid '%s' value"), "attrib.as.set");
+    if(ignore_bytecode == NA_LOGICAL) error(_("invalid '%s' value"), "ignore.bytecode");
+    if(ignore_env == NA_LOGICAL) error(_("invalid '%s' value"), "ignore.environment");
+    
+    flags = (num_eq ? 0 : 1) + (single_NA ? 0 : 2) + (attr_as_set ? 0 : 4) + 
+	(ignore_bytecode ? 0 : 8) + (ignore_env ? 0 : 16);
+    return ScalarLogical(R_compute_identical(x, y, flags));
 }
 
+#define NUM_EQ 		(!(flags & 1))
+#define SINGLE_NA       (!(flags & 2))
+#define ATTR_AS_SET     (!(flags & 4))
+#define IGNORE_BYTECODE (!(flags & 8))
+#define IGNORE_ENV      (!(flags & 16))
+
 /* do the two objects compute as identical?
-   used in unique.c */
+   Also used in unique.c */
 Rboolean
-R_compute_identical(SEXP x, SEXP y, Rboolean num_eq,
-		    Rboolean single_NA, Rboolean attr_as_set)
+R_compute_identical(SEXP x, SEXP y, int flags)
 {
     SEXP ax, ay, atrx, atry;
     if(x == y) /* same pointer */
@@ -83,9 +93,17 @@ R_compute_identical(SEXP x, SEXP y, Rboolean num_eq,
     if(OBJECT(x) != OBJECT(y))
 	return FALSE;
 
+    /* Skip attribute checks for CHARSXP
+       -- such attributes are used for the cache.  */
+    if(TYPEOF(x) == CHARSXP)
+    {
+	/* This matches NAs */
+	return Seql(x, y);
+    }
+
     ax = ATTRIB(x); ay = ATTRIB(y);
-    if (!attr_as_set) {
-	if(!R_compute_identical(ax, ay, num_eq, single_NA, FALSE)) return FALSE;
+    if (!ATTR_AS_SET) {
+	if(!R_compute_identical(ax, ay, flags)) return FALSE;
     }
     /* Attributes are special: they should be tagged pairlists.  We
        don't test them if they are not, and we do not test the order
@@ -93,8 +111,8 @@ R_compute_identical(SEXP x, SEXP y, Rboolean num_eq,
 
        This code is not very efficient, but then neither is using
        pairlists for attributes.  If long attribute lists become more
-       common (and they are used for S4 slots) we should store them in a hash
-       table.
+       common (and they are used for S4 slots) we should store them in
+       a hash table.
     */
     else if(ax != R_NilValue || ay != R_NilValue) {
 	if(ax == R_NilValue || ay == R_NilValue)
@@ -114,15 +132,13 @@ R_compute_identical(SEXP x, SEXP y, Rboolean num_eq,
 			if(streql(tx, "row.names")) {
 			    PROTECT(atrx = getAttrib(x, R_RowNamesSymbol));
 			    PROTECT(atry = getAttrib(y, R_RowNamesSymbol));
-			    if(!R_compute_identical(atrx, atry,
-						    num_eq, single_NA, TRUE)) {
+			    if(!R_compute_identical(atrx, atry, flags)) {
 				UNPROTECT(2);
 				return FALSE;
 			    } else
 				UNPROTECT(2);
 			} else
-			    if(!R_compute_identical(CAR(elx), CAR(ely),
-						    num_eq, single_NA, TRUE))
+			    if(!R_compute_identical(CAR(elx), CAR(ely), flags))
 				return FALSE;
 			break;
 		    }
@@ -134,35 +150,35 @@ R_compute_identical(SEXP x, SEXP y, Rboolean num_eq,
     case NILSXP:
 	return TRUE;
     case LGLSXP:
-	if (length(x) != length(y)) return FALSE;
-	/* Use memcmp (which is ISO C) to speed up the comparison */
+	if (xlength(x) != xlength(y)) return FALSE;
+	/* Use memcmp (which is ISO C90) to speed up the comparison */
 	return memcmp((void *)LOGICAL(x), (void *)LOGICAL(y),
-		      length(x) * sizeof(int)) == 0 ? TRUE : FALSE;
+		      xlength(x) * sizeof(int)) == 0 ? TRUE : FALSE;
     case INTSXP:
-	if (length(x) != length(y)) return FALSE;
-	/* Use memcmp (which is ISO C) to speed up the comparison */
+	if (xlength(x) != xlength(y)) return FALSE;
+	/* Use memcmp (which is ISO C90) to speed up the comparison */
 	return memcmp((void *)INTEGER(x), (void *)INTEGER(y),
-		      length(x) * sizeof(int)) == 0 ? TRUE : FALSE;
+		      xlength(x) * sizeof(int)) == 0 ? TRUE : FALSE;
     case REALSXP:
     {
-	int n = length(x);
-	if(n != length(y)) return FALSE;
+	R_xlen_t n = xlength(x);
+	if(n != xlength(y)) return FALSE;
 	else {
 	    double *xp = REAL(x), *yp = REAL(y);
-	    int i, ne_strict = num_eq | (single_NA << 1);
-	    for(i = 0; i < n; i++)
+	    int ne_strict = NUM_EQ | (SINGLE_NA << 1);
+	    for(R_xlen_t i = 0; i < n; i++)
 		if(neWithNaN(xp[i], yp[i], ne_strict)) return FALSE;
 	}
 	return TRUE;
     }
     case CPLXSXP:
     {
-	int n = length(x);
-	if(n != length(y)) return FALSE;
+	R_xlen_t n = xlength(x);
+	if(n != xlength(y)) return FALSE;
 	else {
 	    Rcomplex *xp = COMPLEX(x), *yp = COMPLEX(y);
-	    int i, ne_strict = num_eq | (single_NA << 1);
-	    for(i = 0; i < n; i++)
+	    int ne_strict = NUM_EQ | (SINGLE_NA << 1);
+	    for(R_xlen_t i = 0; i < n; i++)
 		if(neWithNaN(xp[i].r, yp[i].r, ne_strict) ||
 		   neWithNaN(xp[i].i, yp[i].i, ne_strict))
 		    return FALSE;
@@ -171,8 +187,8 @@ R_compute_identical(SEXP x, SEXP y, Rboolean num_eq,
     }
     case STRSXP:
     {
-	int i, n = length(x);
-	if(n != length(y)) return FALSE;
+	R_xlen_t i, n = xlength(x);
+	if(n != xlength(y)) return FALSE;
 	for(i = 0; i < n; i++) {
 	    /* This special-casing for NAs is not needed */
 	    Rboolean na1 = (STRING_ELT(x, i) == NA_STRING),
@@ -183,7 +199,7 @@ R_compute_identical(SEXP x, SEXP y, Rboolean num_eq,
 	}
 	return TRUE;
     }
-    case CHARSXP:
+    case CHARSXP: /* Probably unreachable, but better safe than sorry... */
     {
 	/* This matches NAs */
 	return Seql(x, y);
@@ -191,11 +207,10 @@ R_compute_identical(SEXP x, SEXP y, Rboolean num_eq,
     case VECSXP:
     case EXPRSXP:
     {
-	int i, n = length(x);
-	if(n != length(y)) return FALSE;
+	R_xlen_t i, n = xlength(x);
+	if(n != xlength(y)) return FALSE;
 	for(i = 0; i < n; i++)
-	    if(!R_compute_identical(VECTOR_ELT(x, i),VECTOR_ELT(y, i),
-				    num_eq, single_NA, attr_as_set))
+	    if(!R_compute_identical(VECTOR_ELT(x, i),VECTOR_ELT(y, i), flags))
 		return FALSE;
 	return TRUE;
     }
@@ -205,11 +220,9 @@ R_compute_identical(SEXP x, SEXP y, Rboolean num_eq,
 	while (x != R_NilValue) {
 	    if(y == R_NilValue)
 		return FALSE;
-	    if(!R_compute_identical(CAR(x), CAR(y),
-				    num_eq, single_NA, attr_as_set))
+	    if(!R_compute_identical(CAR(x), CAR(y), flags))
 		return FALSE;
-	    if(!R_compute_identical(PRINTNAME(TAG(x)), PRINTNAME(TAG(y)),
-				    num_eq, single_NA, attr_as_set))
+	    if(!R_compute_identical(PRINTNAME(TAG(x)), PRINTNAME(TAG(y)), flags))
 		return FALSE;
 	    x = CDR(x);
 	    y = CDR(y);
@@ -217,36 +230,34 @@ R_compute_identical(SEXP x, SEXP y, Rboolean num_eq,
 	return(y == R_NilValue);
     }
     case CLOSXP:
-	return(R_compute_identical(FORMALS(x), FORMALS(y),
-				   num_eq, single_NA, attr_as_set) &&
-	       R_compute_identical(BODY_EXPR(x), BODY_EXPR(y),
-				   num_eq, single_NA, attr_as_set) &&
-	       CLOENV(x) == CLOENV(y) ? TRUE : FALSE);
+	return(R_compute_identical(FORMALS(x), FORMALS(y), flags) &&
+	       R_compute_identical(BODY_EXPR(x), BODY_EXPR(y), flags) &&
+	       (IGNORE_ENV || CLOENV(x) == CLOENV(y) ? TRUE : FALSE) &&
+	       (IGNORE_BYTECODE || R_compute_identical(BODY(x), BODY(y), flags))
+	       );
     case SPECIALSXP:
     case BUILTINSXP:
 	return(PRIMOFFSET(x) == PRIMOFFSET(y) ? TRUE : FALSE);
     case ENVSXP:
     case SYMSXP:
     case WEAKREFSXP:
-#ifdef BYTECODE
     case BCODESXP: /**** is this the best approach? */
-#endif
 	return(x == y ? TRUE : FALSE);
     case EXTPTRSXP:
 	return (EXTPTR_PTR(x) == EXTPTR_PTR(y) ? TRUE : FALSE);
     case RAWSXP:
-	if (length(x) != length(y)) return FALSE;
-	/* Use memcmp (which is ISO C) to speed up the comparison */
+	if (xlength(x) != xlength(y)) return FALSE;
+	/* Use memcmp (which is ISO C90) to speed up the comparison */
 	return memcmp((void *)RAW(x), (void *)RAW(y),
-		      length(x) * sizeof(Rbyte)) == 0 ? TRUE : FALSE;
+		      xlength(x) * sizeof(Rbyte)) == 0 ? TRUE : FALSE;
 
-	/*  case PROMSXP: args are evaluated, so will not be seen */
+/*  case PROMSXP: args are evaluated, so will not be seen */
 	/* test for equality of the substituted expression -- or should
 	   we require both expression and environment to be identical? */
 	/*#define PREXPR(x)	((x)->u.promsxp.expr)
 	  #define PRENV(x)	((x)->u.promsxp.env)
 	  return(R_compute_identical(subsititute(PREXPR(x), PRENV(x),
-	                             num_eq, single_NA, attr_as_set),
+	                             flags),
 	  subsititute(PREXPR(y), PRENV(y))));*/
     case S4SXP:
 	/* attributes already tested, so all slots identical */
@@ -267,11 +278,11 @@ R_compute_identical(SEXP x, SEXP y, Rboolean num_eq,
  * @param x
  * @param y  the two "number"s to be compared
  * @param str a "strictness" indicator, one of 2*2 (one|bit)_NA__num_(eq|bit)
- *  "single_NA" means: x and y differ in the case
+ *  "SINGLE_NA" means: x and y differ in the case
  *    that one, but not both are NaN.  Two NaN values are judged
  *    identical for this purpose, but NA != NaN
  *
- *  "num_eq" means: (x != y) is used when both are not NA or NaN
+ *  "NUM_EQ" means: (x != y) is used when both are not NA or NaN
  *  whereas "bit_NA" and "num_bit" use the bitwise memory comparison  memcmp();
  *  notably "*_num_bit" will differentiate '+0.' and '-0.'.
  *
