@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995-1998	Robert Gentleman and Ross Ihaka.
- *  Copyright (C) 2000-2011	The R Core Team.
+ *  Copyright (C) 2000-2012	The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -62,6 +62,7 @@
 #endif
 
 #include "Defn.h"
+#include <Internal.h>
 #include "Print.h"
 #include "Fileio.h"
 #include "Rconnections.h"
@@ -69,7 +70,7 @@
 
 
 /* Global print parameter struct: */
-attribute_hidden R_print_par_t R_print;
+R_print_par_t R_print;
 
 static void printAttributes(SEXP, SEXP, Rboolean);
 static void PrintSpecial(SEXP);
@@ -81,13 +82,14 @@ static char tagbuf[TAGBUFLEN + 5];
 
 
 /* Used in X11 module for dataentry */
-/* 'rho' is unused */
+/* NB this is called by R.app even though it is in no public header, so 
+   alter there if you alter this */
 void PrintDefaults(void)
 {
     R_print.na_string = NA_STRING;
     R_print.na_string_noquote = mkChar("<NA>");
-    R_print.na_width = strlen(CHAR(R_print.na_string));
-    R_print.na_width_noquote = strlen(CHAR(R_print.na_string_noquote));
+    R_print.na_width = (int) strlen(CHAR(R_print.na_string));
+    R_print.na_width_noquote = (int) strlen(CHAR(R_print.na_string_noquote));
     R_print.quote = 1;
     R_print.right = Rprt_adj_left;
     R_print.digits = GetOptionDigits();
@@ -95,9 +97,11 @@ void PrintDefaults(void)
     if (R_print.scipen == NA_INTEGER) R_print.scipen = 0;
     R_print.max = asInteger(GetOption1(install("max.print")));
     if (R_print.max == NA_INTEGER || R_print.max < 0) R_print.max = 99999;
+    else if(R_print.max == INT_MAX) R_print.max--; // so we can add
     R_print.gap = 1;
     R_print.width = GetOptionWidth();
     R_print.useSource = USESOURCE;
+    R_print.cutoff = GetOptionCutoff();
 }
 
 SEXP attribute_hidden do_invisible(SEXP call, SEXP op, SEXP args, SEXP rho)
@@ -143,7 +147,7 @@ SEXP attribute_hidden do_prmatrix(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    error(_("invalid 'na.print' specification"));
 	R_print.na_string = R_print.na_string_noquote = STRING_ELT(naprint, 0);
 	R_print.na_width = R_print.na_width_noquote =
-	    strlen(CHAR(R_print.na_string));
+	    (int) strlen(CHAR(R_print.na_string));
     }
 
     if (length(rowlab) == 0) rowlab = R_NilValue;
@@ -186,7 +190,7 @@ static void PrintLanguageEtc(SEXP s, Rboolean useSource, Rboolean isClosure)
     int i;
     SEXP t = getAttrib(s, R_SrcrefSymbol);
     if (!isInteger(t) || !useSource)
-	t = deparse1(s, 0, useSource | DEFAULTDEPARSE);
+	t = deparse1w(s, 0, useSource | DEFAULTDEPARSE);
     else {
         PROTECT(t = lang2(install("as.character"), t));
         t = eval(t, R_BaseEnv);
@@ -204,11 +208,13 @@ static void PrintLanguageEtc(SEXP s, Rboolean useSource, Rboolean isClosure)
     }
 }
 
+static
 void PrintClosure(SEXP s, Rboolean useSource)
 {
     PrintLanguageEtc(s, useSource, TRUE);
 }
 
+static
 void PrintLanguage(SEXP s, Rboolean useSource)
 {
     PrintLanguageEtc(s, useSource, FALSE);
@@ -247,7 +253,7 @@ SEXP attribute_hidden do_printdefault(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    error(_("invalid 'na.print' specification"));
 	R_print.na_string = R_print.na_string_noquote = STRING_ELT(naprint, 0);
 	R_print.na_width = R_print.na_width_noquote =
-	    strlen(CHAR(R_print.na_string));
+	    (int) strlen(CHAR(R_print.na_string));
     }
     args = CDR(args);
 
@@ -267,6 +273,7 @@ SEXP attribute_hidden do_printdefault(SEXP call, SEXP op, SEXP args, SEXP rho)
 	R_print.max = asInteger(CAR(args));
 	if(R_print.max == NA_INTEGER || R_print.max < 0)
 	    error(_("invalid '%s' argument"), "max");
+	else if(R_print.max == INT_MAX) R_print.max--; // so we can add
     }
     args = CDR(args);
 
@@ -377,7 +384,7 @@ static void PrintGenericVector(SEXP s, SEXP env)
 		if (LENGTH(tmp) == 1) {
 		    /* This can potentially overflow */
 		    const char *ctmp = translateChar(STRING_ELT(tmp, 0));
-		    int len = strlen(ctmp);
+		    int len = (int) strlen(ctmp);
 		    if(len < 100)
 			snprintf(pbuf, 115, "\"%s\"", ctmp);
 		    else {
@@ -422,7 +429,7 @@ static void PrintGenericVector(SEXP s, SEXP env)
     }
     else { /* .. no dim() .. */
 	names = getAttrib(s, R_NamesSymbol);
-	taglen = strlen(tagbuf);
+	taglen = (int) strlen(tagbuf);
 	ptag = tagbuf + taglen;
 	PROTECT(newcall = allocList(2));
 	SETCAR(newcall, install("print"));
@@ -575,7 +582,7 @@ static void printList(SEXP s, SEXP env)
     }
     else {
 	i = 1;
-	taglen = strlen(tagbuf);
+	taglen = (int) strlen(tagbuf);
 	ptag = tagbuf + taglen;
 	PROTECT(newcall = allocList(2));
 	SETCAR(newcall, install("print"));
@@ -629,7 +636,7 @@ static void PrintExpression(SEXP s)
     SEXP u;
     int i, n;
 
-    u = deparse1(s, 0, R_print.useSource | DEFAULTDEPARSE);
+    u = deparse1w(s, 0, R_print.useSource | DEFAULTDEPARSE);
     n = LENGTH(u);
     for (i = 0; i < n; i++)
 	Rprintf("%s\n", CHAR(STRING_ELT(u, i))); /*translated */
@@ -1006,9 +1013,9 @@ int F77_NAME(dblep0) (const char *label, int *nchar, double *data, int *ndata)
 {
     int k, nc = *nchar;
 
-    if(nc < 0) nc = strlen(label);
+    if(nc < 0) nc = (int) strlen(label);
     if(nc > 255) {
-	warning(_("invalid character length in dblepr"));
+	warning(_("invalid character length in 'dblepr'"));
 	nc = 0;
     } else if(nc > 0) {
 	for (k = 0; k < nc; k++)
@@ -1024,9 +1031,9 @@ int F77_NAME(intpr0) (const char *label, int *nchar, int *data, int *ndata)
 {
     int k, nc = *nchar;
 
-    if(nc < 0) nc = strlen(label);
+    if(nc < 0) nc = (int) strlen(label);
     if(nc > 255) {
-	warning(_("invalid character length in intpr"));
+	warning(_("invalid character length in 'intpr'"));
 	nc = 0;
     } else if(nc > 0) {
 	for (k = 0; k < nc; k++)
@@ -1043,9 +1050,9 @@ int F77_NAME(realp0) (const char *label, int *nchar, float *data, int *ndata)
     int k, nc = *nchar, nd = *ndata;
     double *ddata;
 
-    if(nc < 0) nc = strlen(label);
+    if(nc < 0) nc = (int) strlen(label);
     if(nc > 255) {
-	warning(_("invalid character length in realpr"));
+	warning(_("invalid character length in 'realpr'"));
 	nc = 0;
     }
     else if(nc > 0) {
@@ -1055,7 +1062,7 @@ int F77_NAME(realp0) (const char *label, int *nchar, float *data, int *ndata)
     }
     if(nd > 0) {
 	ddata = (double *) malloc(nd*sizeof(double));
-	if(!ddata) error(_("memory allocation error in realpr"));
+	if(!ddata) error(_("memory allocation error in 'realpr'"));
 	for (k = 0; k < nd; k++) ddata[k] = (double) data[k];
 	printRealVector(ddata, nd, 1);
 	free(ddata);
