@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995-1998  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1998-2011  The R Development Core Team.
+ *  Copyright (C) 1998-2012  The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #endif
 
 #include <Defn.h>
+#include <Internal.h>
 #include <float.h>  /* for DBL_EPSILON */
 #include <Rmath.h>
 
@@ -39,7 +40,7 @@ static R_StringBuffer cbuff = {NULL, 0, MAXELTSIZE};
 static SEXP cross_colon(SEXP call, SEXP s, SEXP t)
 {
     SEXP a, la, ls, lt, rs, rt;
-    int i, j, k, n, nls, nlt, vs, vt;
+    int i, j, k, n, nls, nlt;
     char *cbuf;
 
     if (length(s) != length(t))
@@ -53,8 +54,8 @@ static SEXP cross_colon(SEXP call, SEXP s, SEXP t)
     PROTECT(rs = coerceVector(s, INTSXP));
     PROTECT(rt = coerceVector(t, INTSXP));
     for (i = 0; i < n; i++) {
-	vs = INTEGER(rs)[i];
-	vt = INTEGER(rt)[i];
+	int vs = INTEGER(rs)[i];
+	int vt = INTEGER(rt)[i];
 	if ((vs == NA_INTEGER) || (vt == NA_INTEGER))
 	    INTEGER(a)[i] = NA_INTEGER;
 	else
@@ -67,10 +68,10 @@ static SEXP cross_colon(SEXP call, SEXP s, SEXP t)
 	/* FIXME: possibly UTF-8 version */
 	for (i = 0; i < nls; i++) {
 	    const char *vi = translateChar(STRING_ELT(ls, i));
-	    vs = strlen(vi);
+	    size_t vs = strlen(vi);
 	    for (j = 0; j < nlt; j++) {
 		const char *vj = translateChar(STRING_ELT(lt, j));
-		vt = strlen(vj);
+		size_t vt = strlen(vj);
 		cbuf = R_AllocStringBuffer(vs + vt + 1, &cbuff);
 		sprintf(cbuf, "%s:%s", vi, vj);
 		SET_STRING_ELT(la, k, mkChar(cbuf));
@@ -87,17 +88,22 @@ static SEXP cross_colon(SEXP call, SEXP s, SEXP t)
     return(a);
 }
 
+/* interval at which to check interrupts */
+#define NINTERRUPT 1000000U
+
 static SEXP seq_colon(double n1, double n2, SEXP call)
 {
-    int i, n, in1;
+    int in1;
+    R_xlen_t n;
     double r;
     SEXP ans;
     Rboolean useInt;
 
     r = fabs(n2 - n1);
-    if(r >= INT_MAX) errorcall(call,_("result would be too long a vector"));
+    if(r >= R_XLEN_T_MAX) 
+	errorcall(call, _("result would be too long a vector"));
 
-    n = r + 1 + FLT_EPSILON;
+    n = (R_xlen_t)(r + 1 + FLT_EPSILON);
 
     in1 = (int)(n1);
     useInt = (n1 == in1);
@@ -106,23 +112,35 @@ static SEXP seq_colon(double n1, double n2, SEXP call)
 	    useInt = FALSE;
 	else {
 	    /* r := " the effective 'to' "  of  from:to */
-	    r = n1 + ((n1 <= n2) ? n-1 : -(n-1));
-	    if(r <= INT_MIN || r > INT_MAX)
-		useInt = FALSE;
+	    double dn = (double) n;
+	    r = n1 + ((n1 <= n2) ? dn-1 : -(dn-1));
+	    if(r <= INT_MIN || r > INT_MAX) useInt = FALSE;
 	}
     }
     if (useInt) {
 	ans = allocVector(INTSXP, n);
 	if (n1 <= n2)
-	    for (i = 0; i < n; i++) INTEGER(ans)[i] = in1 + i;
+	    for (int i = 0; i < n; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+		INTEGER(ans)[i] = in1 + i;
+	    }
 	else
-	    for (i = 0; i < n; i++) INTEGER(ans)[i] = in1 - i;
+	    for (int i = 0; i < n; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+		INTEGER(ans)[i] = in1 - i;
+	    }
     } else {
 	ans = allocVector(REALSXP, n);
 	if (n1 <= n2)
-	    for (i = 0; i < n; i++) REAL(ans)[i] = n1 + i;
+	    for (R_xlen_t i = 0; i < n; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+		REAL(ans)[i] = n1 + (double)i;
+	    }
 	else
-	    for (i = 0; i < n; i++) REAL(ans)[i] = n1 - i;
+	    for (R_xlen_t i = 0; i < n; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+		REAL(ans)[i] = n1 - (double)i;
+	    }
     }
     return ans;
 }
@@ -153,186 +171,151 @@ SEXP attribute_hidden do_colon(SEXP call, SEXP op, SEXP args, SEXP rho)
     return seq_colon(n1, n2, call);
 }
 
+/* rep.int(x, times) for a vector times */
 static SEXP rep2(SEXP s, SEXP ncopy)
 {
-    int i, na, nc, n, j;
-    SEXP a, t, u;
+    R_xlen_t i, na, nc, n;
+    int j;
+    SEXP a, t;
 
     PROTECT(t = coerceVector(ncopy, INTSXP));
 
-    nc = length(ncopy);
+    nc = xlength(ncopy);
     na = 0;
     for (i = 0; i < nc; i++) {
-	if (INTEGER(t)[i] == NA_INTEGER || INTEGER(t)[i]<0)
+//	if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+	if (INTEGER(t)[i] == NA_INTEGER || INTEGER(t)[i] < 0)
 	    error(_("invalid '%s' value"), "times");
 	na += INTEGER(t)[i];
     }
 
-    if (isVector(s))
-	a = allocVector(TYPEOF(s), na);
-    else
-	a = allocList(na);
-    PROTECT(a);
+/*    R_xlen_t ni = NINTERRUPT, ratio;
+    if(nc > 0) {
+	ratio = na/nc; // average no of replications
+	if (ratio > 1000U) ni = 1000U;
+	} */
+    PROTECT(a = allocVector(TYPEOF(s), na));
     n = 0;
     switch (TYPEOF(s)) {
     case LGLSXP:
-	for (i = 0; i < nc; i++)
-	    for (j = 0; j < (INTEGER(t)[i]); j++)
+	for (i = 0; i < nc; i++) {
+//	    if ((i+1) % ni == 0) R_CheckUserInterrupt();
+	    for (j = 0; j < INTEGER(t)[i]; j++)
 		LOGICAL(a)[n++] = LOGICAL(s)[i];
+	}
 	break;
     case INTSXP:
-	for (i = 0; i < nc; i++)
-	    for (j = 0; j < (INTEGER(t)[i]); j++)
+	for (i = 0; i < nc; i++) {
+//	    if ((i+1) % ni == 0) R_CheckUserInterrupt();
+	    for (j = 0; j < INTEGER(t)[i]; j++)
 		INTEGER(a)[n++] = INTEGER(s)[i];
+	}
 	break;
     case REALSXP:
-	for (i = 0; i < nc; i++)
-	    for (j = 0; j < (INTEGER(t)[i]); j++)
+	for (i = 0; i < nc; i++) {
+//	    if ((i+1) % ni == 0) R_CheckUserInterrupt();
+	    for (j = 0; j < INTEGER(t)[i]; j++)
 		REAL(a)[n++] = REAL(s)[i];
+	}
 	break;
     case CPLXSXP:
-	for (i = 0; i < nc; i++)
-	    for (j = 0; j < (INTEGER(t)[i]); j++)
+	for (i = 0; i < nc; i++) {
+//	    if ((i+1) % ni == 0) R_CheckUserInterrupt();
+	    for (j = 0; j < INTEGER(t)[i]; j++)
 		COMPLEX(a)[n++] = COMPLEX(s)[i];
+	}
 	break;
     case STRSXP:
-	for (i = 0; i < nc; i++)
-	    for (j = 0; j < (INTEGER(t)[i]); j++)
+	for (i = 0; i < nc; i++) {
+//	    if ((i+1) % ni == 0) R_CheckUserInterrupt();
+	    for (j = 0; j < INTEGER(t)[i]; j++)
 		SET_STRING_ELT(a, n++, STRING_ELT(s, i));
+	}
 	break;
     case VECSXP:
     case EXPRSXP:
-	for (i = 0; i < nc; i++)
-	    for (j = 0; j < (INTEGER(t)[i]); j++)
+	for (i = 0; i < nc; i++) {
+//	    if ((i+1) % ni == 0) R_CheckUserInterrupt();
+	    for (j = 0; j < INTEGER(t)[i]; j++)
 		SET_VECTOR_ELT(a, n++, VECTOR_ELT(s, i));
-	break;
-    case LISTSXP:
-	u = a;
-	for (i = 0; i < nc; i++)
-	    for (j = 0; j < (INTEGER(t)[i]); j++) {
-		SETCAR(u, duplicate(CAR(nthcdr(s, i))));
-		u = CDR(u);
-	    }
+	}
 	break;
     case RAWSXP:
-	for (i = 0; i < nc; i++)
-	    for (j = 0; j < (INTEGER(t)[i]); j++)
+	for (i = 0; i < nc; i++) {
+//	    if ((i+1) % ni == 0) R_CheckUserInterrupt();
+	    for (j = 0; j < INTEGER(t)[i]; j++)
 		RAW(a)[n++] = RAW(s)[i];
+	}
 	break;
     default:
 	UNIMPLEMENTED_TYPE("rep2", s);
     }
-#ifdef _S4_rep_keepClass
-    if(IS_S4_OBJECT(s)) { /* e.g. contains = "list" */
-	setAttrib(a, R_ClassSymbol, getAttrib(s, R_ClassSymbol));
-	SET_S4_OBJECT(a);
-    }
-#endif
-    if (inherits(s, "factor")) {
-	SEXP tmp;
-	if(inherits(s, "ordered")) {
-	    PROTECT(tmp = allocVector(STRSXP, 2));
-	    SET_STRING_ELT(tmp, 0, mkChar("ordered"));
-	    SET_STRING_ELT(tmp, 1, mkChar("factor"));
-	}
-	else {
-	    PROTECT(tmp = mkString("factor"));
-	}
-	setAttrib(a, R_ClassSymbol, tmp);
-	UNPROTECT(1);
-	setAttrib(a, R_LevelsSymbol, getAttrib(s, R_LevelsSymbol));
-    }
     UNPROTECT(2);
     return a;
 }
 
-/* called in do_rep_int() only */
-static SEXP rep1(SEXP s, SEXP ncopy)
+/* rep_len(x, len), also used for rep.int() with scalar 'times' */
+static SEXP rep3(SEXP s, R_xlen_t ns, R_xlen_t na)
 {
-    int i, ns, na, nc;
-    SEXP a, t;
+    R_xlen_t i, j;
+    SEXP a;
 
-    if (!isVector(ncopy))
-	error(_("incorrect type for second argument"));
-
-    if (!isVector(s) && (!isList(s)))
-	error(_("attempt to replicate non-vector"));
-
-    if ((length(ncopy) == length(s)))
-	return rep2(s, ncopy);
-
-    if ((length(ncopy) != 1))
-	error(_("invalid '%s' value"), "times");
-
-    if ((nc = asInteger(ncopy)) == NA_INTEGER || nc < 0)/* nc = 0 ok */
-	error(_("invalid '%s' value"), "times");
-
-    ns = length(s);
-    na = nc * ns;
-    if (isVector(s))
-	a = allocVector(TYPEOF(s), na);
-    else
-	a = allocList(na);
-    PROTECT(a);
-
-#ifdef _S4_rep_keepClass
-    if(IS_S4_OBJECT(s)) { /* e.g. contains = "list" */
-	setAttrib(a, R_ClassSymbol, getAttrib(s, R_ClassSymbol));
-	SET_S4_OBJECT(a);
-    }
-#endif
+    PROTECT(a = allocVector(TYPEOF(s), na));
 
+    // i % ns is slow, especially with long R_xlen_t
     switch (TYPEOF(s)) {
     case LGLSXP:
-	for (i = 0; i < na; i++)
-	    LOGICAL(a)[i] = LOGICAL(s)[i % ns];
+	for (i = 0, j = 0; i < na;) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+	    if (j >= ns) j = 0;
+	    LOGICAL(a)[i++] = LOGICAL(s)[j++];
+	}
 	break;
     case INTSXP:
-	for (i = 0; i < na; i++)
-	    INTEGER(a)[i] = INTEGER(s)[i % ns];
+	for (i = 0, j = 0; i < na;) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+	    if (j >= ns) j = 0;
+	    INTEGER(a)[i++] = INTEGER(s)[j++];
+	}
 	break;
     case REALSXP:
-	for (i = 0; i < na; i++)
-	    REAL(a)[i] = REAL(s)[i % ns];
+	for (i = 0, j = 0; i < na;) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+	    if (j >= ns) j = 0;
+	    REAL(a)[i++] = REAL(s)[j++];
+	}
 	break;
     case CPLXSXP:
-	for (i = 0; i < na; i++)
-	    COMPLEX(a)[i] = COMPLEX(s)[i % ns];
-	break;
-    case STRSXP:
-	for (i = 0; i < na; i++)
-	    SET_STRING_ELT(a, i, STRING_ELT(s, i% ns));
-	break;
-    case LISTSXP:
-	i = 0;
-	for (t = a; t != R_NilValue; t = CDR(t), i++)
-	    SETCAR(t, duplicate(CAR(nthcdr(s, (i % ns)))));
-	break;
-    case VECSXP:
-	i = 0;
-	for (i = 0; i < na; i++)
-	    SET_VECTOR_ELT(a, i, duplicate(VECTOR_ELT(s, i% ns)));
+	for (i = 0, j = 0; i < na;) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+	    if (j >= ns) j = 0;
+	    COMPLEX(a)[i++] = COMPLEX(s)[j++];
+	}
 	break;
     case RAWSXP:
-	for (i = 0; i < na; i++)
-	    RAW(a)[i] = RAW(s)[i % ns];
+	for (i = 0, j = 0; i < na;) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+	    if (j >= ns) j = 0;
+	    RAW(a)[i++] = RAW(s)[j++];
+	}
+	break;
+    case STRSXP:
+	for (i = 0, j = 0; i < na;) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+	    if (j >= ns) j = 0;
+	    SET_STRING_ELT(a, i++, STRING_ELT(s, j++));
+	}
+	break;
+    case VECSXP:
+    case EXPRSXP:
+	for (i = 0, j = 0; i < na;) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+	    if (j >= ns) j = 0;
+	    SET_VECTOR_ELT(a, i++, duplicate(VECTOR_ELT(s, j++)));
+	}
 	break;
     default:
-	UNIMPLEMENTED_TYPE("rep", s);
-    }
-    if (inherits(s, "factor")) {
-	SEXP tmp;
-	if(inherits(s, "ordered")) {
-	    PROTECT(tmp = allocVector(STRSXP, 2));
-	    SET_STRING_ELT(tmp, 0, mkChar("ordered"));
-	    SET_STRING_ELT(tmp, 1, mkChar("factor"));
-	}
-	else {
-	    PROTECT(tmp = mkString("factor"));
-	}
-	setAttrib(a, R_ClassSymbol, tmp);
-	UNPROTECT(1);
-	setAttrib(a, R_LevelsSymbol, getAttrib(s, R_LevelsSymbol));
+	UNIMPLEMENTED_TYPE("rep3", s);
     }
     UNPROTECT(1);
     return a;
@@ -341,7 +324,255 @@ static SEXP rep1(SEXP s, SEXP ncopy)
 SEXP attribute_hidden do_rep_int(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     checkArity(op, args);
-    return rep1(CAR(args), CADR(args));
+    SEXP s = CAR(args), ncopy = CADR(args);
+    R_xlen_t nc;
+    SEXP a;
+
+    if (!isVector(ncopy))
+	error(_("incorrect type for second argument"));
+
+    if (!isVector(s) && s != R_NilValue)
+	error(_("attempt to replicate an object of type '%s'"), 
+	      type2char(TYPEOF(s)));
+
+    nc = xlength(ncopy); // might be 0
+    if (nc == xlength(s)) 
+	PROTECT(a = rep2(s, ncopy));
+    else {
+	if (nc != 1) error(_("invalid '%s' value"), "times");
+	
+#ifdef LONG_VECTOR_SUPPORT
+	double snc = asReal(ncopy);
+	if (!R_FINITE(snc) || snc < 0)
+	    error(_("invalid '%s' value"), "times");
+	nc = (R_xlen_t) snc;
+#else
+	if ((nc = asInteger(ncopy)) == NA_INTEGER || nc < 0)/* nc = 0 ok */
+	    error(_("invalid '%s' value"), "times");
+#endif
+	R_xlen_t ns = xlength(s);
+	PROTECT(a = rep3(s, ns, nc * ns));
+    }
+
+#ifdef _S4_rep_keepClass
+    if(IS_S4_OBJECT(s)) { /* e.g. contains = "list" */
+	setAttrib(a, R_ClassSymbol, getAttrib(s, R_ClassSymbol));
+	SET_S4_OBJECT(a);
+    }
+#endif
+
+    if (inherits(s, "factor")) {
+	SEXP tmp;
+	if(inherits(s, "ordered")) {
+	    PROTECT(tmp = allocVector(STRSXP, 2));
+	    SET_STRING_ELT(tmp, 0, mkChar("ordered"));
+	    SET_STRING_ELT(tmp, 1, mkChar("factor"));
+	} else PROTECT(tmp = mkString("factor"));
+	setAttrib(a, R_ClassSymbol, tmp);
+	UNPROTECT(1);
+	setAttrib(a, R_LevelsSymbol, getAttrib(s, R_LevelsSymbol));
+    }
+    UNPROTECT(1);
+    return a;
+}
+
+SEXP attribute_hidden do_rep_len(SEXP call, SEXP op, SEXP args, SEXP rho)
+{
+    R_xlen_t ns, na;
+    SEXP a, s, len;
+
+    checkArity(op, args);
+    s = CAR(args);
+
+    if (!isVector(s) && s != R_NilValue)
+	error(_("attempt to replicate non-vector"));
+
+    len = CADR(args);
+    if(length(len) != 1)
+	error(_("invalid '%s' value"), "length.out");
+#ifdef LONG_VECTOR_SUPPORT
+    double sna = asReal(len);
+    if (!R_FINITE(sna) || sna < 0)
+	error(_("invalid '%s' value"), "length.out");
+    na = (R_xlen_t) sna;
+#else
+    if ((na = asInteger(len)) == NA_INTEGER || na < 0) /* na = 0 ok */
+	error(_("invalid '%s' value"), "length.out");
+#endif
+
+    if (TYPEOF(s) == NILSXP && na > 0)
+	error(_("cannot replicate NULL to a non-zero length"));
+    ns = xlength(s);
+    if (ns == 0) {
+	SEXP a;
+	PROTECT(a = duplicate(s));
+	if(na > 0) a = xlengthgets(a, na);
+	UNPROTECT(1);
+	return a;
+    }
+    PROTECT(a = rep3(s, ns, na));
+
+#ifdef _S4_rep_keepClass
+    if(IS_S4_OBJECT(s)) { /* e.g. contains = "list" */
+	setAttrib(a, R_ClassSymbol, getAttrib(s, R_ClassSymbol));
+	SET_S4_OBJECT(a);
+    }
+#endif
+
+    if (inherits(s, "factor")) {
+	SEXP tmp;
+	if(inherits(s, "ordered")) {
+	    PROTECT(tmp = allocVector(STRSXP, 2));
+	    SET_STRING_ELT(tmp, 0, mkChar("ordered"));
+	    SET_STRING_ELT(tmp, 1, mkChar("factor"));
+	} else PROTECT(tmp = mkString("factor"));
+	setAttrib(a, R_ClassSymbol, tmp);
+	UNPROTECT(1);
+	setAttrib(a, R_LevelsSymbol, getAttrib(s, R_LevelsSymbol));
+    }
+    UNPROTECT(1);
+    return a;
+}
+
+/* rep(), allowing for both times and each */
+static SEXP rep4(SEXP x, SEXP times, R_xlen_t len, int each, R_xlen_t nt)
+{
+    SEXP a;
+    R_xlen_t lx = xlength(x);
+    R_xlen_t i, j, k, k2, k3, sum;
+
+    // faster code for common special case
+    if (each == 1 && nt == 1) return rep3(x, lx, len);
+
+    PROTECT(a = allocVector(TYPEOF(x), len));
+
+    switch (TYPEOF(x)) {
+    case LGLSXP:
+	if(nt == 1)
+	    for(i = 0; i < len; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+		LOGICAL(a)[i] = LOGICAL(x)[(i/each) % lx];
+	    }
+	else {
+	    for(i = 0, k = 0, k2 = 0; i < lx; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+		for(j = 0, sum = 0; j < each; j++) sum += INTEGER(times)[k++];
+		for(k3 = 0; k3 < sum; k3++) {
+		    LOGICAL(a)[k2++] = LOGICAL(x)[i];
+		    if(k2 == len) goto done;
+		}
+	    }
+	}
+	break;
+    case INTSXP:
+	if(nt == 1)
+	    for(i = 0; i < len; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+		INTEGER(a)[i] = INTEGER(x)[(i/each) % lx];
+	    }
+	else {
+	    for(i = 0, k = 0, k2 = 0; i < lx; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+		for(j = 0, sum = 0; j < each; j++) sum += INTEGER(times)[k++];
+		for(k3 = 0; k3 < sum; k3++) {
+		    INTEGER(a)[k2++] = INTEGER(x)[i];
+		    if(k2 == len) goto done;
+		}
+	    }
+	}
+	break;
+    case REALSXP:
+	if(nt == 1)
+	    for(i = 0; i < len; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+		REAL(a)[i] = REAL(x)[(i/each) % lx];
+	    }
+	else {
+	    for(i = 0, k = 0, k2 = 0; i < lx; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+		for(j = 0, sum = 0; j < each; j++) sum += INTEGER(times)[k++];
+		for(k3 = 0; k3 < sum; k3++) {
+		    REAL(a)[k2++] = REAL(x)[i];
+		    if(k2 == len) goto done;
+		}
+	    }
+	}
+	break;
+    case CPLXSXP:
+	if(nt == 1)
+	    for(i = 0; i < len; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+		COMPLEX(a)[i] = COMPLEX(x)[(i/each) % lx];
+	    }
+	else {
+	    for(i = 0, k = 0, k2 = 0; i < lx; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+		for(j = 0, sum = 0; j < each; j++) sum += INTEGER(times)[k++];
+		for(k3 = 0; k3 < sum; k3++) {
+		    COMPLEX(a)[k2++] = COMPLEX(x)[i];
+		    if(k2 == len) goto done;
+		}
+	    }
+	}
+	break;
+    case STRSXP:
+	if(nt == 1)
+	    for(i = 0; i < len; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+		SET_STRING_ELT(a, i, STRING_ELT(x, (i/each) % lx));
+	    }
+	else {
+	    for(i = 0, k = 0, k2 = 0; i < lx; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+		for(j = 0, sum = 0; j < each; j++) sum += INTEGER(times)[k++];
+		for(k3 = 0; k3 < sum; k3++) {
+		    SET_STRING_ELT(a, k2++, STRING_ELT(x, i));
+		    if(k2 == len) goto done;
+		}
+	    }
+	}
+	break;
+    case VECSXP:
+    case EXPRSXP:
+	if(nt == 1)
+	    for(i = 0; i < len; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+		SET_VECTOR_ELT(a, i, VECTOR_ELT(x, (i/each) % lx));
+	    }
+	else {
+	    for(i = 0, k = 0, k2 = 0; i < lx; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+		for(j = 0, sum = 0; j < each; j++) sum += INTEGER(times)[k++];
+		for(k3 = 0; k3 < sum; k3++) {
+		    SET_VECTOR_ELT(a, k2++, VECTOR_ELT(x, i));
+		    if(k2 == len) goto done;
+		}
+	    }
+	}
+	break;
+    case RAWSXP:
+	if(nt == 1)
+	    for(i = 0; i < len; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+		RAW(a)[i] = RAW(x)[(i/each) % lx];
+	    }
+	else {
+	    for(i = 0, k = 0, k2 = 0; i < lx; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+		for(j = 0, sum = 0; j < each; j++) sum += INTEGER(times)[k++];
+		for(k3 = 0; k3 < sum; k3++) {
+		    RAW(a)[k2++] = RAW(x)[i];
+		    if(k2 == len) goto done;
+		}
+	    }
+	}
+	break;
+    default:
+	UNIMPLEMENTED_TYPE("rep4", x);
+    }
+done:
+    UNPROTECT(1);
+    return a;
 }
 
 /* We are careful to use evalListKeepMissing here (inside
@@ -351,9 +582,11 @@ SEXP attribute_hidden do_rep_int(SEXP call, SEXP op, SEXP args, SEXP rho)
 /* This is a primitive SPECIALSXP with internal argument matching */
 SEXP attribute_hidden do_rep(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
-    SEXP ans, x, ap, times = R_NilValue /* -Wall */, ind;
-    int i, lx, len = NA_INTEGER, each = 1, nt, nprotect = 4;
+    SEXP ans, x, ap, times = R_NilValue /* -Wall */;
+    int each = 1, nprotect = 4;
+    R_xlen_t i, lx, len = NA_INTEGER, nt;
 
+    /* includes factors, POSIX[cl]t, Date */
     if (DispatchOrEval(call, op, "rep", args, rho, &ans, 0, 0))
 	return(ans);
 
@@ -366,7 +599,7 @@ SEXP attribute_hidden do_rep(SEXP call, SEXP op, SEXP args, SEXP rho)
     */
     PROTECT(ap = CONS(R_NilValue,
 		      list4(R_NilValue, R_NilValue, R_NilValue, R_NilValue)));
-    SET_TAG(ap,  install("x"));
+    SET_TAG(ap, install("x"));
     SET_TAG(CDR(ap), install("times"));
     SET_TAG(CDDR(ap), install("length.out"));
     SET_TAG(CDR(CDDR(ap)), install("each"));
@@ -374,11 +607,22 @@ SEXP attribute_hidden do_rep(SEXP call, SEXP op, SEXP args, SEXP rho)
     PROTECT(args = matchArgs(ap, args, call));
 
     x = CAR(args);
-    lx = length(x);
+    /* supported in R 2.15.x */
+    if (TYPEOF(x) == LISTSXP)
+	errorcall(call, "replication of pairlists is defunct");
 
-    len = asInteger(CADDR(args));
-    if(len != NA_INTEGER && len < 0)
-	errorcall(call, _("invalid '%s' argument"), "length.out");
+    lx = xlength(x);
+
+    double slen = asReal(CADDR(args));
+    if (R_FINITE(slen)) {
+	if(slen < 0)
+	    errorcall(call, _("invalid '%s' argument"), "length.out");
+	len = (R_xlen_t) slen;
+    } else {
+	len = asInteger(CADDR(args));
+	if(len != NA_INTEGER && len < 0)
+	    errorcall(call, _("invalid '%s' argument"), "length.out");
+    }
     if(length(CADDR(args)) != 1)
 	warningcall(call, _("first element used of '%s' argument"), 
 		    "length.out");
@@ -391,29 +635,40 @@ SEXP attribute_hidden do_rep(SEXP call, SEXP op, SEXP args, SEXP rho)
     if(each == NA_INTEGER) each = 1;
 
     if(lx == 0) {
-	UNPROTECT(3);
-	if(len == NA_INTEGER) return x;
-	else return lengthgets(duplicate(x), len);
+	if(len > 0 && x == R_NilValue) 
+	    warningcall(call, "'x' is NULL so the result will be NULL");
+	SEXP a;
+	PROTECT(a = duplicate(x));
+	if(len != NA_INTEGER && len > 0) a = xlengthgets(a, len);
+	UNPROTECT(4);
+	return a;
     }
+    if (!isVector(x))
+	errorcall(call, "attempt to replicate an object of type '%s'",
+		  type2char(TYPEOF(x)));
 
+    /* So now we know x is a vector of positive length.  We need to
+       replicate it, and its names if it has them. */
+
+    /* First find the final length using 'times' and 'each' */
     if(len != NA_INTEGER) { /* takes precedence over times */
 	nt = 1;
     } else {
-	int it, sum = 0;
+	R_xlen_t sum = 0;
 	if(CADR(args) == R_MissingArg) PROTECT(times = ScalarInteger(1));
 	else PROTECT(times = coerceVector(CADR(args), INTSXP));
 	nprotect++;
-	nt = LENGTH(times);
+	nt = XLENGTH(times);
 	if(nt != 1 && nt != lx * each)
 	    errorcall(call, _("invalid '%s' argument"), "times");
 	if(nt == 1) {
-	    it = INTEGER(times)[0];
+	    int it = INTEGER(times)[0];
 	    if (it == NA_INTEGER || it < 0)
 		errorcall(call, _("invalid '%s' argument"), "times");
 	    len = lx * it * each;
 	} else {
 	    for(i = 0; i < nt; i++) {
-		it = INTEGER(times)[i];
+		int it = INTEGER(times)[i];
 		if (it == NA_INTEGER || it < 0)
 		    errorcall(call, _("invalid '%s' argument"), "times");
 		sum += it;
@@ -421,33 +676,22 @@ SEXP attribute_hidden do_rep(SEXP call, SEXP op, SEXP args, SEXP rho)
             len = sum;
 	}
     }
-    PROTECT(ind = allocVector(INTSXP, len));
+
     if(len > 0 && each == 0)
 	errorcall(call, _("invalid '%s' argument"), "each");
-    if(nt == 1)
-	for(i = 0; i < len; i++) INTEGER(ind)[i] = 1 + ((i/each) % lx);
-    else {
-	int j, k, k2, k3;
-	for(i = 0, k = 0, k2 = 0; i < lx; i++) {
-	    int sum = 0;
-	    for(j = 0; j < each; j++) sum += INTEGER(times)[k++];
-	    for(k3 = 0; k3 < sum; k3++) {
-		INTEGER(ind)[k2++] = i+1;
-		if(k2 == len) goto done;
-	    }
-	}
-    }
 
-done:
-    ans = do_subset_dflt(R_NilValue, R_NilValue, list2(x, ind), rho);
+    SEXP xn = getAttrib(x, R_NamesSymbol);
+
+    PROTECT(ans = rep4(x, times, len, each, nt));
+    if (length(xn) > 0)
+	setAttrib(ans, R_NamesSymbol, rep4(xn, times, len, each, nt));
+
 #ifdef _S4_rep_keepClass
     if(IS_S4_OBJECT(x)) { /* e.g. contains = "list" */
 	setAttrib(ans, R_ClassSymbol, getAttrib(x, R_ClassSymbol));
 	SET_S4_OBJECT(ans);
     }
 #endif
-    /* 1D arrays get dimensions preserved */
-    setAttrib(ans, R_DimSymbol, R_NilValue);
     UNPROTECT(nprotect);
     return ans;
 }
@@ -462,12 +706,14 @@ done:
  */
 
 #define FEPS 1e-10
+#define myabs(x) (x < 0 ? x : -x)
 /* to match seq.default */
 SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans = R_NilValue /* -Wall */, ap, tmp, from, to, by, len, along;
-    int i, nargs = length(args), lf, lout = NA_INTEGER;
+    int nargs = length(args), lf;
     Rboolean One = nargs == 1;
+    R_xlen_t i, lout = NA_INTEGER;
 
     if (DispatchOrEval(call, op, "seq", args, rho, &ans, 0, 1))
 	return(ans);
@@ -506,7 +752,7 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 	goto done;
     }
     if(along != R_MissingArg) {
-	lout = LENGTH(along);
+	lout = XLENGTH(along);
 	if(One) {
 	    ans = lout ? seq_colon(1.0, (double)lout, call) : allocVector(INTSXP, 0);
 	    goto done;
@@ -518,18 +764,21 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 	if(length(len) != 1)
 	    warningcall(call, _("first element used of '%s' argument"), 
 			"length.out");
-	lout = (int) ceil(rout);
+	lout = (R_xlen_t) ceil(rout);
     }
 
     if(lout == NA_INTEGER) {
 	double rfrom = asReal(from), rto = asReal(to), rby = asReal(by), *ra;
 	if(from == R_MissingArg) rfrom = 1.0;
+	else if(length(from) != 1) error("'from' must be of length 1");
 	if(to == R_MissingArg) rto = 1.0;
+	else if(length(to) != 1) error("'to' must be of length 1");
 	if(by == R_MissingArg)
 	    ans = seq_colon(rfrom, rto, call);
 	else {
+	    if(length(by) != 1) error("'by' must be of length 1");
 	    double del = rto - rfrom, n, dd;
-	    int nn;
+	    R_xlen_t nn;
 	    if(!R_FINITE(rfrom))
 		errorcall(call, _("'from' must be finite"));
 	    if(!R_FINITE(rto))
@@ -552,7 +801,11 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 		ans = from;
 		goto done;
 	    }
+#ifdef LONG_VECTOR_SUPPORT
+	    if(n > 100 * (double) INT_MAX)
+#else
 	    if(n > (double) INT_MAX)
+#endif
 		errorcall(call, _("'by' argument is much too small"));
 	    if(n < - FEPS)
 		errorcall(call, _("wrong sign in 'by' argument"));
@@ -564,20 +817,20 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 		   reduced below 1/INT_MAX this is the same as the
 		   next, so this is future-proofing against longer integers.
 		*/
-		nn = (int)n;
 		/* seq.default gives integer result from
 		   from + (0:n)*by
 		*/
+		nn = (R_xlen_t) n;
 		ans = allocVector(INTSXP, nn+1);
 		ia = INTEGER(ans);
 		for(i = 0; i <= nn; i++)
-		    ia[i] = ifrom + i * iby;
+		    ia[i] = (int)(ifrom + i * iby);
 	    } else {
 		nn = (int)(n + FEPS);
 		ans = allocVector(REALSXP, nn+1);
 		ra = REAL(ans);
 		for(i = 0; i <= nn; i++)
-		    ra[i] = rfrom + i * rby;
+		    ra[i] = rfrom + (double)i * rby;
 		/* Added in 2.9.0 */
 		if (nn > 0)
 		    if((rby > 0 && ra[nn] > rto) || (rby < 0 && ra[nn] < rto))
@@ -590,8 +843,8 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 	ans = seq_colon(1.0, (double)lout, call);
     } else if (by == R_MissingArg) {
 	double rfrom = asReal(from), rto = asReal(to), rby;
-	if(to == R_MissingArg) rto = rfrom + lout - 1;
-	if(from == R_MissingArg) rfrom = rto - lout + 1;
+	if(to == R_MissingArg) rto = rfrom + (double)lout - 1;
+	if(from == R_MissingArg) rfrom = rto - (double)lout + 1;
 	if(!R_FINITE(rfrom))
 	    errorcall(call, _("'from' must be finite"));
 	if(!R_FINITE(rto))
@@ -601,7 +854,10 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 	if(lout > 1) REAL(ans)[lout - 1] = rto;
 	if(lout > 2) {
 	    rby = (rto - rfrom)/(double)(lout - 1);
-	    for(i = 1; i < lout-1; i++) REAL(ans)[i] = rfrom + i*rby;
+	    for(i = 1; i < lout-1; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+		REAL(ans)[i] = rfrom + (double)i*rby;
+	    }
 	}
     } else if (to == R_MissingArg) {
 	double rfrom = asReal(from), rby = asReal(by), rto;
@@ -610,20 +866,24 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    errorcall(call, _("'from' must be finite"));
 	if(!R_FINITE(rby))
 	    errorcall(call, _("'by' must be finite"));
-	rto = rfrom +(lout-1)*rby;
+	rto = rfrom + (double)(lout-1)*rby;
 	if(rby == (int)rby && rfrom <= INT_MAX && rfrom >= INT_MIN
 	   && rto <= INT_MAX && rto >= INT_MIN) {
 	    ans = allocVector(INTSXP, lout);
-	    for(i = 0; i < lout; i++)
-		INTEGER(ans)[i] = rfrom + i*rby;
+	    for(i = 0; i < lout; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+		INTEGER(ans)[i] = (int)(rfrom + (double)i*rby);
+	    }
 	} else {
 	    ans = allocVector(REALSXP, lout);
-	    for(i = 0; i < lout; i++)
-		REAL(ans)[i] = rfrom + i*rby;
+	    for(i = 0; i < lout; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+		REAL(ans)[i] = rfrom + (double)i*rby;
+	    }
 	}
     } else if (from == R_MissingArg) {
 	double rto = asReal(to), rby = asReal(by),
-	    rfrom = rto - (lout-1)*rby;
+	    rfrom = rto - (double)(lout-1)*rby;
 	if(!R_FINITE(rto))
 	    errorcall(call, _("'to' must be finite"));
 	if(!R_FINITE(rby))
@@ -631,12 +891,16 @@ SEXP attribute_hidden do_seq(SEXP call, SEXP op, SEXP args, SEXP rho)
 	if(rby == (int)rby && rfrom <= INT_MAX && rfrom >= INT_MIN
 	   && rto <= INT_MAX && rto >= INT_MIN) {
 	    ans = allocVector(INTSXP, lout);
-	    for(i = 0; i < lout; i++)
-		INTEGER(ans)[i] = rto - (lout - 1 - i)*rby;
+	    for(i = 0; i < lout; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+		INTEGER(ans)[i] = (int)(rto - (double)(lout - 1 - i)*rby);
+	    }
 	} else {
 	    ans = allocVector(REALSXP, lout);
-	    for(i = 0; i < lout; i++)
-		REAL(ans)[i] = rto - (lout - 1 - i)*rby;
+	    for(i = 0; i < lout; i++) {
+//		if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+		REAL(ans)[i] = rto - (double)(lout - 1 - i)*rby;
+	    }
 	}
     } else
 	errorcall(call, _("too many arguments"));
@@ -649,7 +913,7 @@ done:
 SEXP attribute_hidden do_seq_along(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans;
-    int i, len, *p;
+    R_xlen_t len;
     static SEXP length_op = NULL;
 
     /* Store the .Primitive for 'length' for DispatchOrEval to use. */
@@ -666,7 +930,7 @@ SEXP attribute_hidden do_seq_along(SEXP call, SEXP op, SEXP args, SEXP rho)
     checkArity(op, args);
     check1arg(args, call, "along.with");
 
-    /* Try to dispatch to S3 or S4 metods for 'length'.  For cases
+    /* Try to dispatch to S3 or S4 methods for 'length'.  For cases
        where no methods are defined this is more efficient than an
        unconditional callback to R */
     if (isObject(CAR(args)) &&
@@ -674,30 +938,68 @@ SEXP attribute_hidden do_seq_along(SEXP call, SEXP op, SEXP args, SEXP rho)
 	len = asInteger(ans);
     }
     else
-	len = length(CAR(args));
+	len = xlength(CAR(args));
 
-    ans = allocVector(INTSXP, len);
-    p = INTEGER(ans);
-    for(i = 0; i < len; i++) p[i] = i+1;
+#ifdef LONG_VECTOR_SUPPORT
+    if (len > INT_MAX) {
+	ans = allocVector(REALSXP, len);
+	double *p = REAL(ans);
+	for(R_xlen_t i = 0; i < len; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+	    p[i] = (double) (i+1);
+	}
+    } else
+#endif
+    {
+	ans = allocVector(INTSXP, len);
+	int *p = INTEGER(ans);
+	for(int i = 0; i < len; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+	    p[i] = i+1;
+	}
+    }
     return ans;
 }
 
 SEXP attribute_hidden do_seq_len(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans;
-    int i, len, *p;
+    R_xlen_t len;
 
     checkArity(op, args);
     check1arg(args, call, "length.out");
-    len = asInteger(CAR(args));
-    if(len == NA_INTEGER || len < 0)
-	errorcall(call, _("argument must be coercible to non-negative integer"));
     if(length(CAR(args)) != 1)
 	warningcall(call, _("first element used of '%s' argument"),
 		    "length.out");
-    ans = allocVector(INTSXP, len);
-    p = INTEGER(ans);
-    for(i = 0; i < len; i++) p[i] = i+1;
 
+ #ifdef LONG_VECTOR_SUPPORT
+    double dlen = asReal(CAR(args));
+    if(!R_FINITE(dlen) || dlen < 0)
+	errorcall(call, _("argument must be coercible to non-negative integer"));
+    len = (R_xlen_t) dlen;
+#else
+    len = asInteger(CAR(args));
+    if(len == NA_INTEGER || len < 0)
+	errorcall(call, _("argument must be coercible to non-negative integer"));
+#endif
+
+ #ifdef LONG_VECTOR_SUPPORT
+    if (len > INT_MAX) {
+	ans = allocVector(REALSXP, len);
+	double *p = REAL(ans);
+	for(R_xlen_t i = 0; i < len; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();	    
+	    p[i] = (double) (i+1);
+	}
+    } else
+#endif
+    {
+	ans = allocVector(INTSXP, len);
+	int *p = INTEGER(ans);
+	for(int i = 0; i < len; i++) {
+//	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
+	    p[i] = i+1;
+	}
+    }
     return ans;
 }
