@@ -34,7 +34,7 @@
 
 /* Relative size of subscripts/superscripts (i.e. `indexical' size) */
 
-#define SCRIPTSIZE (0.6)	
+#define SCRIPTSIZE (0.6)
 
 /* Positioning of subscripts/superscripts */
 
@@ -72,16 +72,16 @@ typedef struct {
 } vfontContext;
 
 /* forward references */
-static bool _composite_char (unsigned char *composite, 
-			     unsigned char *character, 
+static bool _composite_char (unsigned char *composite,
+			     unsigned char *character,
 			     unsigned char *accent);
-static void _draw_stroke (vfontContext *vc, const pGEcontext gc, 
-			  pGEDevDesc dd, 
+static void _draw_stroke (vfontContext *vc, const pGEcontext gc,
+			  pGEDevDesc dd,
 			  bool pendown, double deltax, double deltay);
-static double _label_width_hershey (const pGEcontext gc, pGEDevDesc dd, 
-				    const unsigned short *label); 
-static void _draw_hershey_string (vfontContext *vc, const pGEcontext gc, 
-				  pGEDevDesc dd, 
+static double _label_width_hershey (const pGEcontext gc, pGEDevDesc dd,
+				    const unsigned short *label);
+static void _draw_hershey_string (vfontContext *vc, const pGEcontext gc,
+				  pGEDevDesc dd,
 				  const unsigned short *string);
 
 /* _draw_hershey_stroke() draws a stroke, taking into account the
@@ -94,11 +94,11 @@ static void _draw_hershey_string (vfontContext *vc, const pGEcontext gc,
 */
 
 static
-void _draw_hershey_stroke (vfontContext *vc, const pGEcontext gc, pGEDevDesc dd, 
+void _draw_hershey_stroke (vfontContext *vc, const pGEcontext gc, pGEDevDesc dd,
 			   bool pendown, double deltax, double deltay)
 {
-    _draw_stroke(vc, gc, dd, pendown, 
-		 fromDeviceWidth(HERSHEY_X_UNITS_TO_USER_UNITS (deltax), 
+    _draw_stroke(vc, gc, dd, pendown,
+		 fromDeviceWidth(HERSHEY_X_UNITS_TO_USER_UNITS (deltax),
 				 GE_INCHES, dd),
 		 fromDeviceHeight(HERSHEY_Y_UNITS_TO_USER_UNITS (deltay),
 				  GE_INCHES, dd));
@@ -109,12 +109,12 @@ static void moverel(double dx, double dy, vfontContext *vc) {
     vc->currY += dy;
 }
 
-static void linerel(double dx, double dy, 
-		    vfontContext *vc, const pGEcontext gc, 
+static void linerel(double dx, double dy,
+		    vfontContext *vc, const pGEcontext gc,
 		    pGEDevDesc dd) {
-    GELine(toDeviceX(vc->currX, GE_INCHES, dd), 
+    GELine(toDeviceX(vc->currX, GE_INCHES, dd),
 	   toDeviceY(vc->currY, GE_INCHES, dd),
-	   toDeviceX(vc->currX+dx, GE_INCHES, dd), 
+	   toDeviceX(vc->currX+dx, GE_INCHES, dd),
 	   toDeviceY(vc->currY+dy, GE_INCHES, dd),
 	   gc, dd);
     vc->currX += dx;
@@ -126,14 +126,14 @@ static void linerel(double dx, double dy,
 #define M_PI		3.141592653589793238462643383279502884197169399375
 #endif
 
-static void _draw_stroke (vfontContext *vc, const pGEcontext gc, pGEDevDesc dd, 
+static void _draw_stroke (vfontContext *vc, const pGEcontext gc, pGEDevDesc dd,
 			  bool pendown, double deltax, double deltay)
 {
   double dx, dy;
   double theta;
-  
+
   theta = M_PI * vc->angle / 180.0;
-  
+
   dx = cos(theta) * deltax - sin(theta) * deltay;
   dy = sin(theta) * deltax + cos(theta) * deltay;
 
@@ -152,18 +152,18 @@ static void _draw_stroke (vfontContext *vc, const pGEcontext gc, pGEDevDesc dd,
 /* this is the version of the flabelwidth() method that is specific to the
    case when the current Plotter font is a Hershey font; called in
    g_flabelwidth () */
-static double R_VF_VStrWidth (const char *s, 
+static double R_VF_VStrWidth (const char *s,
 			      const pGEcontext gc, pGEDevDesc dd)
 {
   double label_width;
   unsigned short *codestring;
-  
+
   /* PAUL MURRELL
      _controlify using R_alloc instead of xmalloc so need to do
      vmaxget() ... vmaxset() instead of free()
   */
 
-  char *vmax = vmaxget();
+  const void *vmax = vmaxget();
 
   /* convert string to a codestring, including annotations */
   codestring = _controlify (dd, (const unsigned char *) s,
@@ -173,7 +173,7 @@ static double R_VF_VStrWidth (const char *s,
 
   vmaxset(vmax);
   /*  free (codestring); */
-  
+
   return label_width;
 }
 
@@ -181,8 +181,8 @@ static double R_VF_VStrWidth (const char *s,
    Added _label_height_hershey and GVStrHeight function
 */
 
-static double _label_height_hershey (const pGEcontext gc, 
-				     pGEDevDesc dd, 
+static double _label_height_hershey (const pGEcontext gc,
+				     pGEDevDesc dd,
 				     const unsigned short *label)
 {
     return( HERSHEY_Y_UNITS_TO_USER_UNITS(HERSHEY_LARGE_CAPHEIGHT) );
@@ -192,8 +192,8 @@ static double R_VF_VStrHeight (const char *s, const pGEcontext gc, pGEDevDesc dd
 {
   double label_height;
   unsigned short *codestring;
-  
-  char *vmax = vmaxget();
+
+  const void *vmax = vmaxget();
 
   /* convert string to a codestring, including annotations */
   codestring = _controlify (dd, (const unsigned char *) s,
@@ -202,7 +202,7 @@ static double R_VF_VStrHeight (const char *s, const pGEcontext gc, pGEDevDesc dd
   label_height = _label_height_hershey (gc, dd, codestring);
 
   vmaxset(vmax);
-  
+
   return label_height;
 }
 
@@ -218,7 +218,7 @@ static double R_VF_VStrHeight (const char *s, const pGEcontext gc, pGEDevDesc dd
 
 /* this is the version of the falabel() method that is specific
    to the case when the current Plotter font is a Hershey font */
-static void R_VF_VText (double x, double y, const char *s, 
+static void R_VF_VText (double x, double y, const char *s,
 			double x_justify, double y_justify, double rotation,
 			const pGEcontext gc,
 			pGEDevDesc dd)
@@ -226,13 +226,13 @@ static void R_VF_VText (double x, double y, const char *s,
   unsigned short *codestring;
   double label_width, label_height;
   double x_offset, y_offset;
-  double x_displacement;
-  
+/*  double x_displacement; */
+
   /* PAUL MURRELL
      _controlify using R_alloc instead of xmalloc so need to do
      vmaxget() ... vmaxset() instead of free()
   */
-  char *vmax = vmaxget();
+  const void *vmax = vmaxget();
 
   /* PAUL MURRELL
      initialise the local currX and currY
@@ -244,7 +244,7 @@ static void R_VF_VText (double x, double y, const char *s,
   vc.angle = rotation;
 
   /* PAUL MURRELL
-   * Override gc settings for lty and lwd 
+   * Override gc settings for lty and lwd
    */
   gc->lty = LTY_SOLID;
   gc->lwd = HERSHEY_LINE_WIDTH_TO_LWD (HERSHEY_STROKE_WIDTH);
@@ -252,12 +252,12 @@ static void R_VF_VText (double x, double y, const char *s,
   gc->ljoin = GE_ROUND_JOIN;
 
   /* convert string to a codestring, including annotations */
-  codestring = _controlify (dd, (const unsigned char *)s, 
+  codestring = _controlify (dd, (const unsigned char *)s,
 			    gc->fontfamily[3] - 1, gc->fontface);
 
-  
+
   /* PAUL MURRELL
-     Justification changed from char (e.g., 'l', 'c', 'r') to 
+     Justification changed from char (e.g., 'l', 'c', 'r') to
      double (e.g., 0, .5, 1)
      Also added handling of NaN x_justify and y_justify
      ... and replaced HERSHEY_Y_UNITS_TO_USER_UNITS(HERSHEY_HEIGHT)
@@ -272,11 +272,11 @@ static void R_VF_VText (double x, double y, const char *s,
 
   if (!R_FINITE(x_justify))
       x_justify = 0.5;
-  if (!R_FINITE(y_justify)) 
+  if (!R_FINITE(y_justify))
       y_justify = 0.5;
 
   x_offset = 0 - x_justify;
-  x_displacement = 1 - 2 * x_justify;
+  /* x_displacement = 1 - 2 * x_justify; */
 
   y_offset = 0 - y_justify * 1.0;
 
@@ -293,14 +293,14 @@ static void R_VF_VText (double x, double y, const char *s,
     old_cap_mode = (char *)_plot_xmalloc (strlen (_plotter->drawstate->cap_mode) + 1);
     old_join_mode = (char *)_plot_xmalloc (strlen (_plotter->drawstate->join_mode) + 1);
     oldposx = (_plotter->drawstate->pos).x;
-    oldposy = (_plotter->drawstate->pos).y;    
+    oldposy = (_plotter->drawstate->pos).y;
 
     strcpy (old_line_mode, _plotter->drawstate->line_mode);
     strcpy (old_cap_mode, _plotter->drawstate->cap_mode);
     strcpy (old_join_mode, _plotter->drawstate->join_mode);
     old_fill_type = _plotter->drawstate->fill_type;
     old_dash_array_in_effect = _plotter->drawstate->dash_array_in_effect;
-*/    
+*/
     /* Our choices for rendering: solid lines, rounded capitals and joins,
        a line width equal to slightly more than 1 Hershey unit, and
        transparent filling. */
@@ -310,7 +310,7 @@ static void R_VF_VText (double x, double y, const char *s,
     _plotter->joinmod (R___(_plotter) "round");
     _plotter->filltype (R___(_plotter) 0);
 */
-    
+
     /* move to take horizontal and vertical justification into account;
        arguments here are in user units */
       /* PAUL MURRELL
@@ -318,14 +318,14 @@ static void R_VF_VText (double x, double y, const char *s,
 	 so it needs to use absolute coordinates
       */
       _draw_stroke (&vc, gc, dd,
-		    false, 
+		    false,
 		    fromDeviceWidth(x_offset * label_width, GE_INCHES, dd),
 		    fromDeviceHeight(y_offset * label_height, GE_INCHES, dd));
     /* call stroker on the sequence of strokes obtained from each char (the
        stroker may manipulate the line width) */
       /* _draw_hershey_stroke (dd, true, 0, HERSHEY_EM);     */
       _draw_hershey_string (&vc, gc, dd, codestring);
-    
+
     /* Restore original values of relevant drawing attributes, free
        storage.  endpath() will be invoked in here automatically, flushing
        the created polyline object comprising the stroked text. */
@@ -335,7 +335,7 @@ static void R_VF_VText (double x, double y, const char *s,
     _plotter->joinmod (R___(_plotter) old_join_mode);
     _plotter->filltype (R___(_plotter) old_fill_type);
     _plotter->drawstate->dash_array_in_effect = old_dash_array_in_effect;
-    
+
     free (old_line_mode);
     free (old_cap_mode);
     free (old_join_mode);
@@ -360,61 +360,61 @@ static void R_VF_VText (double x, double y, const char *s,
   /*  free (codestring); */
 
   /* PAUL MURRELL
-     No return value 
+     No return value
   */
   /* return label_width;		 user units */
 }
 
 /* In addition to scaling the character sizes and the `width', we perform
    the following (dx, dy):
-   
+
    enter subscript	(dx, dy) = (-1/9, -1/2) * width
    exit subscript	(dx, dy) = (+1/6, +1/2) * width
-   
+
    enter superscript	(dx, dy) = (-1/9, +1/2) * width
    exit superscript	(dx, dy) = (+1/6, -1/2) * width
-   
+
    For clarity here, `width' refers to the width _before_ it is
    multiplied by a factor 2/3.
-   
+
    [N.B. In Bob Beach's original UGS character stroke generator,
    the +1/6's here were +2/9 instead.  Better?] */
 
 /* _label_width_hershey() computes the width (total delta x) of a
    controlified character string to be rendered in a vector font, in user
    units */
-static double _label_width_hershey (const pGEcontext gc, pGEDevDesc dd, 
-				    const unsigned short *label) 
-{ 
+static double _label_width_hershey (const pGEcontext gc, pGEDevDesc dd,
+				    const unsigned short *label)
+{
   const unsigned short *ptr = label;
   unsigned short c;
   double charsize = 1.0;	/* relative char size, 1.0 means full size */
   double saved_charsize = 1.0;
   double width = 0.0;		/* label width */
   double saved_width = 0.0;
-  
+
   /* loop through unsigned shorts in label */
-  while ((c = (*ptr)) != (unsigned short)'\0') 
+  while ((c = (*ptr)) != (unsigned short)'\0')
     {
       int glyphnum;		/* glyph in Hershey array */
       const unsigned char *glyph;
-      
-      if (c & RAW_HERSHEY_GLYPH) 
+
+      if (c & RAW_HERSHEY_GLYPH)
 	/* glyph was spec'd via an escape, not as a char in a font */
 	{
 	  glyphnum = c & GLYPH_SPEC;
 	  glyph = (const unsigned char *)(_occidental_hershey_glyphs[glyphnum]);
-	  
+
 	  if (*glyph != '\0')	/* nonempty glyph */
 	    /* 1st two chars are bounds */
 	    width += charsize * ((int)glyph[1] - (int)glyph[0]);
 	}
-      else if (c & RAW_ORIENTAL_HERSHEY_GLYPH) 
+      else if (c & RAW_ORIENTAL_HERSHEY_GLYPH)
 	/* glyph was spec'd via an escape, not as a char in a font */
 	{
 	  glyphnum = c & GLYPH_SPEC;
 	  glyph = (const unsigned char *)_oriental_hershey_glyphs[glyphnum];
-	  
+
 	  if (*glyph != '\0')	/* nonempty glyph */
 	    /* 1st two chars are bounds */
 	    width += charsize * ((int)glyph[1] - (int)glyph[0]);
@@ -427,70 +427,70 @@ static double _label_width_hershey (const pGEcontext gc, pGEDevDesc dd,
 	    case C_BEGIN_SUPERSCRIPT :
 	      charsize *= SCRIPTSIZE;
 	      break;
-	      
+
 	    case C_END_SUBSCRIPT:
 	    case C_END_SUPERSCRIPT:
 	      charsize /= SCRIPTSIZE;
 	      break;
-	      
+
 	    case C_PUSH_LOCATION:
 	      saved_width = width;
 	      saved_charsize = charsize;
 	      break;
-	      
+
 	    case C_POP_LOCATION:
 	      width = saved_width;
 	      charsize = saved_charsize;
 	      break;
-	      
+
 	    case C_RIGHT_ONE_EM:
 	      width += charsize * HERSHEY_EM;
 	      break;
-	      
+
 	    case C_RIGHT_HALF_EM:
 	      width += charsize * HERSHEY_EM / 2.0;
 	      break;
-	      
+
 	    case C_RIGHT_QUARTER_EM:
 	      width += charsize * HERSHEY_EM / 4.0;
 	      break;
-	      
+
 	    case C_RIGHT_SIXTH_EM:
 	      width += charsize * HERSHEY_EM / 6.0;
 	      break;
-	      
+
 	    case C_RIGHT_EIGHTH_EM:
 	      width += charsize * HERSHEY_EM / 8.0;
 	      break;
-	      
+
 	    case C_RIGHT_TWELFTH_EM:
 	      width += charsize * HERSHEY_EM / 12.0;
 	      break;
-	      
+
 	    case C_LEFT_ONE_EM:
 	      width -= charsize * HERSHEY_EM;
 	      break;
-	      
+
 	    case C_LEFT_HALF_EM:
 	      width -= charsize * HERSHEY_EM / 2.0;
 	      break;
-	      
+
 	    case C_LEFT_QUARTER_EM:
 	      width -= charsize * HERSHEY_EM / 4.0;
 	      break;
-	      
+
 	    case C_LEFT_SIXTH_EM:
 	      width -= charsize * HERSHEY_EM / 6.0;
 	      break;
-	      
+
 	    case C_LEFT_EIGHTH_EM:
 	      width -= charsize * HERSHEY_EM / 8.0;
 	      break;
-	      
+
 	    case C_LEFT_TWELFTH_EM:
 	      width -= charsize * HERSHEY_EM / 12.0;
 	      break;
-	      
+
 	      /* unrecognized control code */
 	    default:
 	      break;
@@ -499,10 +499,10 @@ static double _label_width_hershey (const pGEcontext gc, pGEDevDesc dd,
       else			/* yow, an actual character */
 	{
 	  int raw_fontnum;
-	  
+
 	  /* compute index of font, in table in g_fontdb.c */
 	  raw_fontnum = (c >> FONT_SHIFT) & ONE_BYTE;
-	  
+
 	  c &= ~FONT_SPEC;	/* extract character proper */
 	  glyphnum = (_hershey_font_info[raw_fontnum].chars)[c];
 
@@ -530,21 +530,21 @@ static double _label_width_hershey (const pGEcontext gc, pGEDevDesc dd,
 	    /* 1st two chars are bounds */
 	    width += charsize * ((int)glyph[1] - (int)glyph[0]);
 	}
-      
+
       ptr++;			/* bump pointer in string */
     }
 
   return HERSHEY_X_UNITS_TO_USER_UNITS (width);
-}  
+}
 
 /* _draw_hershey_penup_stroke() draws a penup stroke, along a vector
    specified in Hershey units.  Size scaling and obliquing (true/false) are
    specified.  This is used for repositioning during rendering of
    composite (accented) characters. */
 static
-void _draw_hershey_penup_stroke(vfontContext *vc, const pGEcontext gc, 
+void _draw_hershey_penup_stroke(vfontContext *vc, const pGEcontext gc,
 				pGEDevDesc dd,
-				double dx, double dy, 
+				double dx, double dy,
 				double charsize, bool oblique)
 {
   double shear;
@@ -552,16 +552,16 @@ void _draw_hershey_penup_stroke(vfontContext *vc, const pGEcontext gc,
   shear = oblique ? (SHEAR) : 0.0;
   _draw_hershey_stroke (vc, gc, dd,
 			false,	/* pen up */
-			charsize * (dx + shear * dy), 
+			charsize * (dx + shear * dy),
 			charsize * dy);
 }
 
 /* _draw_hershey_glyph() invokes move() and cont() to draw a raw Hershey
    glyph, specified by index in the occidental or oriental glyph arrays.
    Size scaling and obliquing (true/false) are specified. */
-static 
+static
 void _draw_hershey_glyph (vfontContext *vc, const pGEcontext gc, pGEDevDesc dd,
-			  int glyphnum, 
+			  int glyphnum,
 			  double charsize, int type, bool oblique)
 {
   double xcurr, ycurr;
@@ -570,7 +570,7 @@ void _draw_hershey_glyph (vfontContext *vc, const pGEcontext gc, pGEDevDesc dd,
   const unsigned char *glyph;
   double dx, dy;
   double shear;
-  
+
   shear = oblique ? (SHEAR) : 0.0;
   switch (type)
     {
@@ -579,7 +579,7 @@ void _draw_hershey_glyph (vfontContext *vc, const pGEcontext gc, pGEDevDesc dd,
       glyph = (const unsigned char *)(_occidental_hershey_glyphs[glyphnum]);
       break;
     case ORIENTAL:
-      glyph = (const unsigned char *)(_oriental_hershey_glyphs[glyphnum]); 
+      glyph = (const unsigned char *)(_oriental_hershey_glyphs[glyphnum]);
       break;
     }
 
@@ -592,9 +592,9 @@ void _draw_hershey_glyph (vfontContext *vc, const pGEcontext gc, pGEDevDesc dd,
       while (*glyph)
 	{
 	  int xnewint;
-	  
+
 	  xnewint = (int)glyph[0];
-	  
+
 	  if (xnewint == (int)' ')
 	    pendown = false;
 	  else
@@ -602,8 +602,8 @@ void _draw_hershey_glyph (vfontContext *vc, const pGEcontext gc, pGEDevDesc dd,
 	      double xnew, ynew;
 
 	      xnew = (double)charsize * xnewint;
-	      ynew = (double)charsize 
-		* ((int)'R' 
+	      ynew = (double)charsize
+		* ((int)'R'
 		   - ((int)glyph[1] + (double)HERSHEY_BASELINE));
 	      dx = xnew - xcurr;
 	      dy = ynew - ycurr;
@@ -612,10 +612,10 @@ void _draw_hershey_glyph (vfontContext *vc, const pGEcontext gc, pGEDevDesc dd,
 	      xcurr = xnew, ycurr = ynew;
 	      pendown = true;
 	    }
-	  
+
 	  glyph +=2;	/* on to next pair */
 	}
-      
+
       /* final penup stroke, to end where we should */
       dx = xfinal - xcurr;
       dy = yfinal - ycurr;
@@ -665,108 +665,108 @@ void _draw_hershey_string (vfontContext *vc, const pGEcontext gc, pGEDevDesc dd,
 			       c & GLYPH_SPEC, charsize, ORIENTAL, false);
 	}
 
-      else if (c & CONTROL_CODE)	
+      else if (c & CONTROL_CODE)
 	switch (c & ~CONTROL_CODE) /* parse control codes */
 	  {
 	  case C_BEGIN_SUPERSCRIPT :
 	    _draw_hershey_stroke (vc, gc, dd,
-				  false, 
+				  false,
 				  SUPERSCRIPT_DX * charsize * HERSHEY_EM,
 				  SUPERSCRIPT_DY * charsize * HERSHEY_EM);
 	    charsize *= SCRIPTSIZE;
 	    break;
-		
+
 	  case C_END_SUPERSCRIPT:
 	    charsize /= SCRIPTSIZE;
 	    _draw_hershey_stroke (vc, gc, dd,
-				  false, 
+				  false,
 				  - SUPERSCRIPT_DX * charsize * HERSHEY_EM,
 				  - SUPERSCRIPT_DY * charsize * HERSHEY_EM);
 	    break;
-		
+
 	  case C_BEGIN_SUBSCRIPT:
 	    _draw_hershey_stroke (vc, gc, dd,
-				  false, 
+				  false,
 				  SUBSCRIPT_DX * charsize * HERSHEY_EM,
 				  SUBSCRIPT_DY * charsize * HERSHEY_EM);
 	    charsize *= SCRIPTSIZE;
 	    break;
-		
+
 	  case C_END_SUBSCRIPT:
 	    charsize /= SCRIPTSIZE;
 	    _draw_hershey_stroke (vc, gc, dd,
-				  false, 
+				  false,
 				  - SUBSCRIPT_DX * charsize * HERSHEY_EM,
 				  - SUBSCRIPT_DY * charsize * HERSHEY_EM);
 	    break;
-		
+
 	  case C_PUSH_LOCATION:
 	      /* saved_charsize = charsize;
 		 saved_position_x = _plotter->drawstate->pos.x;
 		 saved_position_y = _plotter->drawstate->pos.y; */
 	    break;
-		
+
 	  case C_POP_LOCATION:
 	      /* charsize = saved_charsize;
 		 _plotter->fmove (R___(_plotter)
 		 saved_position_x, saved_position_y); */
 	    break;
-		
+
 	  case C_RIGHT_ONE_EM:
 	    _draw_hershey_stroke (vc, gc, dd,
 				  false, charsize * HERSHEY_EM, 0.0);
 	    break;
-		
+
 	  case C_RIGHT_HALF_EM:
 	    _draw_hershey_stroke (vc, gc, dd,
 				  false, charsize * HERSHEY_EM / 2.0, 0.0);
 	    break;
-		
+
 	  case C_RIGHT_QUARTER_EM:
 	    _draw_hershey_stroke (vc, gc, dd,
 				  false, charsize * HERSHEY_EM / 4.0, 0.0);
 	    break;
-		
+
 	  case C_RIGHT_SIXTH_EM:
 	    _draw_hershey_stroke (vc, gc, dd,
 				  false, charsize * HERSHEY_EM / 6.0, 0.0);
 	    break;
-		
+
 	  case C_RIGHT_EIGHTH_EM:
 	    _draw_hershey_stroke (vc, gc, dd,
 				  false, charsize * HERSHEY_EM / 8.0, 0.0);
 	    break;
-		
+
 	  case C_LEFT_ONE_EM:
 	    _draw_hershey_stroke (vc, gc, dd,
 				  false, - charsize * HERSHEY_EM, 0.0);
 	    break;
-		
+
 	  case C_LEFT_HALF_EM:
 	    _draw_hershey_stroke (vc, gc, dd,
 				  false, - charsize * HERSHEY_EM / 2.0, 0.0);
 	    break;
-		
+
 	  case C_LEFT_QUARTER_EM:
 	    _draw_hershey_stroke (vc, gc, dd,
 				  false, - charsize * HERSHEY_EM / 4.0, 0.0);
 	    break;
-		
+
 	  case C_LEFT_SIXTH_EM:
 	    _draw_hershey_stroke (vc, gc, dd,
 				  false, - charsize * HERSHEY_EM / 6.0, 0.0);
 	    break;
-		
+
 	  case C_LEFT_EIGHTH_EM:
 	    _draw_hershey_stroke (vc, gc, dd,
 				  false, - charsize * HERSHEY_EM / 8.0, 0.0);
 	    break;
-		
+
 	    /* unrecognized control code, punt */
 	  default:
 	    break;
 	  }
-      
+
       else
 	/* yow, an actual font character!  Several possibilities: could be
 	   a composite (accented) character, could be a small Kana, or
@@ -779,21 +779,21 @@ void _draw_hershey_string (vfontContext *vc, const pGEcontext gc, pGEDevDesc dd,
 	  const unsigned char *char_glyph, *accent_glyph;
 	  unsigned char composite, character = '\0', accent = '\0' /* -Wall */;
 	  bool oblique, small_kana = false;
-	  
+
 	  /* compute index of font, in font table in g_fontdb.c */
 	  raw_fontnum = (c >> FONT_SHIFT) & ONE_BYTE;
 	  /* shear font?  (for HersheySans-Oblique, etc.) */
 	  oblique = _hershey_font_info[raw_fontnum].obliquing;
-	  
+
 	  c &= ~FONT_SPEC;	/* extract character proper */
 	  glyphnum = (_hershey_font_info[raw_fontnum].chars)[c];
-	  
+
 	  if (glyphnum & KS) /* a small kana? */
 	    {
 	      glyphnum -= KS;
 	      small_kana = true;
 	    }
-	  
+
 	  switch (glyphnum)
 	    {
 	      /* special case: this is a composite (accented) character;
@@ -804,21 +804,21 @@ void _draw_hershey_string (vfontContext *vc, const pGEcontext gc, pGEDevDesc dd,
 	      composite = (unsigned char)c;
 	      if (_composite_char (&composite, &character, &accent))
 		{
-		  char_glyphnum = 
+		  char_glyphnum =
 		    (_hershey_font_info[raw_fontnum].chars)[character];
-		  accent_glyphnum = 
+		  accent_glyphnum =
 		    (_hershey_font_info[raw_fontnum].chars)[accent];
 		}
 	      else
 		{		/* hope this won't happen */
-		  char_glyphnum = UNDE; 
+		  char_glyphnum = UNDE;
 		  accent_glyphnum = 0;
 		}
-	      char_glyph = 
+	      char_glyph =
 		(const unsigned char *)_occidental_hershey_glyphs[char_glyphnum];
-	      accent_glyph = 
+	      accent_glyph =
 		(const unsigned char *)_occidental_hershey_glyphs[accent_glyphnum];
-	  
+
 	      if (*char_glyph != '\0') /* nonempty glyph */
 		/* 1st two chars are bounds, in Hershey units */
 		char_width = (int)char_glyph[1] - (int)char_glyph[0];
@@ -838,7 +838,7 @@ void _draw_hershey_string (vfontContext *vc, const pGEcontext gc, pGEDevDesc dd,
 		  line_width_type = 1;
 		}
 	      _draw_hershey_glyph (vc, gc, dd,
-				   char_glyphnum, charsize, 
+				   char_glyphnum, charsize,
 				   OCCIDENTAL, oblique);
 	      /* back up to draw accent */
 	      _draw_hershey_penup_stroke (vc, gc, dd,
@@ -849,7 +849,7 @@ void _draw_hershey_string (vfontContext *vc, const pGEcontext gc, pGEDevDesc dd,
 	      /* repositioning for uppercase and uppercase italic */
 	      if (glyphnum == ACC1)
 		_draw_hershey_penup_stroke (vc, gc, dd,
-					    0.0, 
+					    0.0,
 					    (double)(ACCENT_UP_SHIFT),
 					    charsize, oblique);
 	      else if (glyphnum == ACC2)
@@ -860,13 +860,13 @@ void _draw_hershey_string (vfontContext *vc, const pGEcontext gc, pGEDevDesc dd,
 
 	      /* draw the accent */
 	      _draw_hershey_glyph (vc, gc, dd,
-				   accent_glyphnum, charsize, 
+				   accent_glyphnum, charsize,
 				   OCCIDENTAL, oblique);
 
 	      /* undo special repositioning if any */
 	      if (glyphnum == ACC1)
 		_draw_hershey_penup_stroke (vc, gc, dd,
-					    0.0, 
+					    0.0,
 					    -(double)(ACCENT_UP_SHIFT),
 					    charsize, oblique);
 	      else if (glyphnum == ACC2)
@@ -892,7 +892,7 @@ void _draw_hershey_string (vfontContext *vc, const pGEcontext gc, pGEDevDesc dd,
 		  const unsigned char *kana_glyph;
 		  double shift = 0.5 * (1.0 - (SMALL_KANA_SIZE));
 
-		  kana_glyph = 
+		  kana_glyph =
 		    (const unsigned char *)_occidental_hershey_glyphs[glyphnum];
 		  kana_width = (int)kana_glyph[1] - (int)kana_glyph[0];
 
@@ -908,7 +908,7 @@ void _draw_hershey_string (vfontContext *vc, const pGEcontext gc, pGEDevDesc dd,
 		      line_width_type = 2;
 		    }
 		  _draw_hershey_glyph (vc, gc, dd,
-				       glyphnum, 
+				       glyphnum,
 				       (SMALL_KANA_SIZE) * charsize,
 				       OCCIDENTAL, oblique);
 		  _draw_hershey_penup_stroke (vc, gc, dd,
@@ -935,7 +935,7 @@ void _draw_hershey_string (vfontContext *vc, const pGEcontext gc, pGEDevDesc dd,
 			  line_width_type = 1;
 			}
 		_draw_hershey_glyph (vc, gc, dd,
-				     glyphnum, charsize, 
+				     glyphnum, charsize,
 				     OCCIDENTAL, oblique);
 		}
 	      break;
@@ -944,20 +944,20 @@ void _draw_hershey_string (vfontContext *vc, const pGEcontext gc, pGEDevDesc dd,
 	} /* end of font character case */
 
     } /* end of loop through unsigned shorts in the codestring */
-    
+
   return;
 }
 
 /* retrieve the two elements of a composite character from the table in
    g_fontdb.c */
-static bool _composite_char (unsigned char *composite, 
-			     unsigned char *character, 
+static bool _composite_char (unsigned char *composite,
+			     unsigned char *character,
 			     unsigned char *accent)
 {
   const struct plHersheyAccentedCharInfoStruct *compchar = _hershey_accented_char_info;
   bool found = false;
   unsigned char given = *composite;
-  
+
   while (compchar->composite)
     {
       if (compchar->composite == given)
